@@ -37,6 +37,10 @@
 #define SECURITY_WIN32
 #include <security.h>
 
+#include <cstdio>
+
+#include <QtCore/private/qfunctions_win_p.h>
+
 #ifndef SPI_GETPLATFORMTYPE
 #define SPI_GETPLATFORMTYPE 257
 #endif
@@ -379,8 +383,36 @@ constexpr QFileDevice::Permissions toSpecificPermissions(PermissionTag tag,
 } // anonymous namespace
 #endif // QT_CONFIG(fslibs)
 
+#if QT_DEPRECATED_SINCE(6,6)
+int qt_ntfs_permission_lookup = 0;
+#endif
 
-Q_CORE_EXPORT int qt_ntfs_permission_lookup = 0;
+static QBasicAtomicInt qt_ntfs_permission_lookup_v2 = Q_BASIC_ATOMIC_INITIALIZER(0);
+
+QT_WARNING_PUSH
+QT_WARNING_DISABLE_DEPRECATED
+
+bool qEnableNtfsPermissionChecks() noexcept
+{
+    return qt_ntfs_permission_lookup_v2.fetchAndAddRelaxed(1)
+QT_IF_DEPRECATED_SINCE(6, 6, /*nothing*/, + qt_ntfs_permission_lookup)
+        != 0;
+}
+
+bool qDisableNtfsPermissionChecks() noexcept
+{
+    return qt_ntfs_permission_lookup_v2.fetchAndSubRelaxed(1)
+QT_IF_DEPRECATED_SINCE(6, 6, /*nothing*/, + qt_ntfs_permission_lookup)
+        == 1;
+}
+
+bool qAreNtfsPermissionChecksEnabled() noexcept
+{
+    return qt_ntfs_permission_lookup_v2.loadRelaxed()
+QT_IF_DEPRECATED_SINCE(6, 6, /*nothing*/, + qt_ntfs_permission_lookup)
+        ;
+}
+QT_WARNING_POP
 
 /*!
     \class QNativeFilePermissions
@@ -668,21 +700,16 @@ static QString readLink(const QFileSystemEntry &link)
 #if QT_CONFIG(fslibs)
     QString ret;
 
-    bool neededCoInit = false;
     IShellLink *psl;                            // pointer to IShellLink i/f
     WIN32_FIND_DATA wfd;
     wchar_t szGotPath[MAX_PATH];
 
+    QComHelper comHelper;
+
     // Get pointer to the IShellLink interface.
     HRESULT hres = CoCreateInstance(CLSID_ShellLink, nullptr, CLSCTX_INPROC_SERVER, IID_IShellLink,
                                     (LPVOID *)&psl);
 
-    if (hres == CO_E_NOTINITIALIZED) { // COM was not initialized
-        neededCoInit = true;
-        CoInitializeEx(nullptr, COINIT_APARTMENTTHREADED | COINIT_DISABLE_OLE1DDE);
-        hres = CoCreateInstance(CLSID_ShellLink, nullptr, CLSCTX_INPROC_SERVER, IID_IShellLink,
-                                (LPVOID *)&psl);
-    }
     if (SUCCEEDED(hres)) {    // Get pointer to the IPersistFile interface.
         IPersistFile *ppf;
         hres = psl->QueryInterface(IID_IPersistFile, (LPVOID *)&ppf);
@@ -698,8 +725,6 @@ static QString readLink(const QFileSystemEntry &link)
         }
         psl->Release();
     }
-    if (neededCoInit)
-        CoUninitialize();
 
     return ret;
 #else
@@ -799,9 +824,10 @@ public:
         return (dwFlags & TSF_DELETE_RECYCLE_IF_POSSIBLE) ? S_OK : E_FAIL;
     }
     HRESULT STDMETHODCALLTYPE PostDeleteItem(DWORD /* dwFlags */, IShellItem * /* psiItem */,
-                                             HRESULT /* hrDelete */,
+                                             HRESULT hrDelete,
                                              IShellItem *psiNewlyCreated) override
     {
+        deleteResult = hrDelete;
         if (psiNewlyCreated) {
             wchar_t *pszName = nullptr;
             psiNewlyCreated->GetDisplayName(SIGDN_FILESYSPATH, &pszName);
@@ -822,6 +848,7 @@ public:
     HRESULT STDMETHODCALLTYPE ResumeTimer() override { return S_OK; }
 
     QString targetPath;
+    HRESULT deleteResult = S_OK;
 private:
     ULONG ref;
 };
@@ -860,6 +887,18 @@ void QFileSystemEngine::clearWinStatData(QFileSystemMetaData &data)
 //static
 QFileSystemEntry QFileSystemEngine::getLinkTarget(const QFileSystemEntry &link,
                                                   QFileSystemMetaData &data)
+{
+    QFileSystemEntry ret = getRawLinkPath(link, data);
+    if (!ret.isEmpty() && ret.isRelative()) {
+        QString target = absoluteName(link).path() + u'/' + ret.filePath();
+        ret = QFileSystemEntry(QDir::cleanPath(target));
+    }
+    return ret;
+}
+
+//static
+QFileSystemEntry QFileSystemEngine::getRawLinkPath(const QFileSystemEntry &link,
+                                                   QFileSystemMetaData &data)
 {
     Q_CHECK_FILE_NAME(link, link);
 
@@ -871,12 +910,7 @@ QFileSystemEntry QFileSystemEngine::getLinkTarget(const QFileSystemEntry &link,
         target = readLink(link);
     else if (data.isLink())
         target = readSymLink(link);
-    QFileSystemEntry ret(target);
-    if (!target.isEmpty() && ret.isRelative()) {
-        target.prepend(absoluteName(link).path() + u'/');
-        ret = QFileSystemEntry(QDir::cleanPath(target));
-    }
-    return ret;
+    return QFileSystemEntry(target);
 }
 
 //static
@@ -978,10 +1012,10 @@ static inline QByteArray fileId(HANDLE handle)
     BY_HANDLE_FILE_INFORMATION info;
     if (GetFileInformationByHandle(handle, &info)) {
         char buffer[sizeof "01234567:0123456701234567"];
-        qsnprintf(buffer, sizeof(buffer), "%lx:%08lx%08lx",
-                  info.dwVolumeSerialNumber,
-                  info.nFileIndexHigh,
-                  info.nFileIndexLow);
+        std::snprintf(buffer, sizeof(buffer), "%lx:%08lx%08lx",
+                      info.dwVolumeSerialNumber,
+                      info.nFileIndexHigh,
+                      info.nFileIndexLow);
         return buffer;
     }
     return QByteArray();
@@ -1037,7 +1071,7 @@ QByteArray QFileSystemEngine::id(HANDLE fHandle)
 
 //static
 bool QFileSystemEngine::setFileTime(HANDLE fHandle, const QDateTime &newDate,
-                                    QAbstractFileEngine::FileTime time, QSystemError &error)
+                                    QFile::FileTime time, QSystemError &error)
 {
     FILETIME fTime;
     FILETIME *pLastWrite = nullptr;
@@ -1045,15 +1079,15 @@ bool QFileSystemEngine::setFileTime(HANDLE fHandle, const QDateTime &newDate,
     FILETIME *pCreationTime = nullptr;
 
     switch (time) {
-    case QAbstractFileEngine::ModificationTime:
+    case QFile::FileModificationTime:
         pLastWrite = &fTime;
         break;
 
-    case QAbstractFileEngine::AccessTime:
+    case QFile::FileAccessTime:
         pLastAccess = &fTime;
         break;
 
-    case QAbstractFileEngine::BirthTime:
+    case QFile::FileBirthTime:
         pCreationTime = &fTime;
         break;
 
@@ -1076,8 +1110,7 @@ QString QFileSystemEngine::owner(const QFileSystemEntry &entry, QAbstractFileEng
 {
     QString name;
 #if QT_CONFIG(fslibs)
-    extern int qt_ntfs_permission_lookup;
-    if (qt_ntfs_permission_lookup > 0) {
+    if (qAreNtfsPermissionChecksEnabled()) {
         initGlobalSid();
         {
             PSID pOwner = 0;
@@ -1131,7 +1164,7 @@ bool QFileSystemEngine::fillPermissions(const QFileSystemEntry &entry, QFileSyst
                                         QFileSystemMetaData::MetaDataFlags what)
 {
 #if QT_CONFIG(fslibs)
-    if (qt_ntfs_permission_lookup > 0) {
+    if (qAreNtfsPermissionChecksEnabled()) {
         initGlobalSid();
 
         QString fname = entry.nativeFilePath();
@@ -1357,7 +1390,7 @@ bool QFileSystemEngine::fillMetaData(const QFileSystemEntry &entry, QFileSystemM
     // Check for ".lnk": Directories named ".lnk" should be skipped, corrupted
     // link files should still be detected as links.
     const QString origFilePath = entry.filePath();
-    if (origFilePath.endsWith(".lnk"_L1) && !isDirPath(origFilePath, nullptr)) {
+    if (bool exists; origFilePath.endsWith(".lnk"_L1) && !isDirPath(origFilePath, &exists) && exists) {
         data.entryFlags |= QFileSystemMetaData::WinLnkType;
         fname = QFileSystemEntry(readLink(entry));
     } else {
@@ -1661,18 +1694,11 @@ bool QFileSystemEngine::createLink(const QFileSystemEntry &source, const QFileSy
                                    QSystemError &error)
 {
     bool ret = false;
+    QComHelper comHelper;
     IShellLink *psl = nullptr;
     HRESULT hres = CoCreateInstance(CLSID_ShellLink, nullptr, CLSCTX_INPROC_SERVER, IID_IShellLink,
                                     reinterpret_cast<void **>(&psl));
 
-    bool neededCoInit = false;
-    if (hres == CO_E_NOTINITIALIZED) { // COM was not initialized
-        neededCoInit = true;
-        CoInitializeEx(nullptr, COINIT_APARTMENTTHREADED | COINIT_DISABLE_OLE1DDE);
-        hres = CoCreateInstance(CLSID_ShellLink, nullptr, CLSCTX_INPROC_SERVER, IID_IShellLink,
-                                reinterpret_cast<void **>(&psl));
-    }
-
     if (SUCCEEDED(hres)) {
         const auto name = QDir::toNativeSeparators(source.filePath());
         const auto pathName = QDir::toNativeSeparators(source.path());
@@ -1692,9 +1718,6 @@ bool QFileSystemEngine::createLink(const QFileSystemEntry &source, const QFileSy
     if (!ret)
         error = QSystemError(::GetLastError(), QSystemError::NativeError);
 
-    if (neededCoInit)
-        CoUninitialize();
-
     return ret;
 }
 
@@ -1762,7 +1785,8 @@ bool QFileSystemEngine::moveFileToTrash(const QFileSystemEntry &source,
     // we need the "display name" of the file, so can't use nativeAbsoluteFilePath
     const QString sourcePath = QDir::toNativeSeparators(absoluteName(source).filePath());
 
-    CoInitializeEx(nullptr, COINIT_APARTMENTTHREADED | COINIT_DISABLE_OLE1DDE);
+    QComHelper comHelper;
+
     IFileOperation *pfo = nullptr;
     IShellItem *deleteItem = nullptr;
     FileOperationProgressSink *sink = nullptr;
@@ -1775,7 +1799,6 @@ bool QFileSystemEngine::moveFileToTrash(const QFileSystemEntry &source,
             deleteItem->Release();
         if (pfo)
             pfo->Release();
-        CoUninitialize();
         if (!SUCCEEDED(hres))
             error = QSystemError(hres, QSystemError::NativeError);
     });
@@ -1796,8 +1819,12 @@ bool QFileSystemEngine::moveFileToTrash(const QFileSystemEntry &source,
     hres = pfo->PerformOperations();
     if (!SUCCEEDED(hres))
         return false;
-    newLocation = QFileSystemEntry(sink->targetPath);
 
+    if (!SUCCEEDED(sink->deleteResult)) {
+        error = QSystemError(sink->deleteResult, QSystemError::NativeError);
+        return false;
+    }
+    newLocation = QFileSystemEntry(sink->targetPath);
     return true;
 }
 
@@ -1828,6 +1855,12 @@ bool QFileSystemEngine::setPermissions(const QFileSystemEntry &entry,
     return ret;
 }
 
+bool QFileSystemEngine::isCaseSensitive(const QFileSystemEntry &, QFileSystemMetaData &)
+{
+    // FIXME: This may not be accurate for all file systems (QTBUG-28246)
+    return false;
+}
+
 static inline QDateTime fileTimeToQDateTime(const FILETIME *time)
 {
     if (time->dwHighDateTime == 0 && time->dwLowDateTime == 0)
@@ -1837,7 +1870,7 @@ static inline QDateTime fileTimeToQDateTime(const FILETIME *time)
     FileTimeToSystemTime(time, &sTime);
     return QDateTime(QDate(sTime.wYear, sTime.wMonth, sTime.wDay),
                      QTime(sTime.wHour, sTime.wMinute, sTime.wSecond, sTime.wMilliseconds),
-                     Qt::UTC);
+                     QTimeZone::UTC);
 }
 
 QDateTime QFileSystemMetaData::birthTime() const
