@@ -116,6 +116,10 @@ Q_LOGGING_CATEGORY(lcPermissions, "qt.permissions", QtWarningMsg);
         </manifest>
     \endcode
 
+    To ensure the relevant permission backend is included with your
+    application, please \l {QT_ANDROID_PACKAGE_SOURCE_DIR}
+    {point the build system to your custom \c AndroidManifest.xml}.
+
     The relevant permission names are described in the documentation
     for each permission type.
 
@@ -194,7 +198,7 @@ Q_LOGGING_CATEGORY(lcPermissions, "qt.permissions", QtWarningMsg);
 
     \code
     qApp->requestPermission(QCameraPermission{}, [](const QPermission &permission) {
-        if (permission.status() == Qt::PermissionStatus:Granted)
+        if (permission.status() == Qt::PermissionStatus::Granted)
             takePhoto();
     });
     \endcode
@@ -211,7 +215,7 @@ Q_LOGGING_CATEGORY(lcPermissions, "qt.permissions", QtWarningMsg);
     \code
     void LocationWidget::permissionUpdated(const QPermission &permission)
     {
-        if (permission.status() != Qt::PermissionStatus:Granted)
+        if (permission.status() != Qt::PermissionStatus::Granted)
             return;
         auto locationPermission = permission.value<QLocationPermission>();
         if (!locationPermission || locationPermission->accuracy() != QLocationPermission::Precise)
@@ -573,7 +577,7 @@ QT_PERMISSION_IMPL_COMMON(QContactsPermission)
 {}
 
 /*!
-    Sets whether the request is for read-write (\a mode == AccessMode::ReadOnly) or
+    Sets whether the request is for read-write (\a mode == AccessMode::ReadWrite) or
     read-only (\a mode == AccessMode::ReadOnly) access to the contacts.
 */
 void QContactsPermission::setAccessMode(AccessMode mode)
@@ -630,7 +634,7 @@ QT_PERMISSION_IMPL_COMMON(QCalendarPermission)
 {}
 
 /*!
-    Sets whether the request is for read-write (\a mode == AccessMode::ReadOnly) or
+    Sets whether the request is for read-write (\a mode == AccessMode::ReadWrite) or
     read-only (\a mode == AccessMode::ReadOnly) access to the calendar.
 */
 void QCalendarPermission::setAccessMode(AccessMode mode)
