@@ -15,13 +15,17 @@
 #include "qdatastream.h"
 #include "qendian.h"
 #include "qfile.h"
-#include "qmap.h"
 #include "qalgorithms.h"
 #include "qtranslator_p.h"
 #include "qlocale.h"
+#include "qlogging.h"
+#include "qloggingcategory.h"
+#include "qdebug.h"
 #include "qendian.h"
 #include "qresource.h"
 
+#include <QtCore/private/qduplicatetracker_p.h>
+
 #if defined(Q_OS_UNIX) && !defined(Q_OS_INTEGRITY)
 #  define QT_USE_MMAP
 #  include "private/qcore_unix_p.h"
@@ -39,8 +43,13 @@
 
 QT_BEGIN_NAMESPACE
 
+static Q_LOGGING_CATEGORY(lcTranslator, "qt.core.qtranslator")
+
+namespace {
 enum Tag { Tag_End = 1, Tag_SourceText16, Tag_Translation, Tag_Context16, Tag_Obsolete1,
            Tag_SourceText, Tag_Context, Tag_Comment, Tag_Obsolete2 };
+}
+
 /*
 $ mcookie
 3cb86418caef9c95cd211cbf60a1bddd
@@ -597,7 +606,10 @@ Q_NEVER_INLINE
 static bool is_readable_file(const QString &name)
 {
     const QFileInfo fi(name);
-    return fi.isReadable() && fi.isFile();
+    const bool isReadableFile = fi.isReadable() && fi.isFile();
+    qCDebug(lcTranslator) << "Testing file" << name << isReadableFile;
+
+    return isReadableFile;
 }
 
 static QString find_translation(const QLocale & locale,
@@ -606,6 +618,9 @@ static QString find_translation(const QLocale & locale,
                                 const QString & directory,
                                 const QString & suffix)
 {
+    qCDebug(lcTranslator).noquote().nospace() << "Searching translation for "
+                          << filename << prefix << locale << suffix
+                          << " in " << directory;
     QString path;
     if (QFileInfo(filename).isRelative()) {
         path = directory;
@@ -616,7 +631,7 @@ static QString find_translation(const QLocale & locale,
 
     QString realname;
     realname += path + filename + prefix; // using += in the hope for some reserve capacity
-    const int realNameBaseSize = realname.size();
+    const qsizetype realNameBaseSize = realname.size();
 
     // see http://www.unicode.org/reports/tr35/#LanguageMatching for inspiration
 
@@ -627,35 +642,70 @@ static QString find_translation(const QLocale & locale,
     // "prefix_en_us.qm" won't be found under the "en_US" locale. Note
     // that the Qt resource system is always case-sensitive, even on
     // Windows (in other words: this codepath is *not* UNIX-only).
-    QStringList languages = locale.uiLanguages();
-    for (int i = languages.size()-1; i >= 0; --i) {
-        QString lang = languages.at(i);
+    QStringList languages = locale.uiLanguages(QLocale::TagSeparator::Underscore);
+    qCDebug(lcTranslator) << "Requested UI languages" << languages;
+
+    QDuplicateTracker<QString> duplicates(languages.size() * 2);
+    for (const auto &l : std::as_const(languages))
+        (void)duplicates.hasSeen(l);
+
+    for (qsizetype i = languages.size() - 1; i >= 0; --i) {
+        QString language = languages.at(i);
+
+        // Add candidates for each entry where we progressively truncate sections
+        // from the end, until a matching language tag is found. For compatibility
+        // reasons (see QTBUG-124898) we add a special case: if we find a
+        // language_Script_Territory entry (i.e. an entry with two sections), try
+        // language_Territory as well as language_Script. Use QDuplicateTracker
+        // so that we don't add any entries as fallbacks that are already in the
+        // list anyway.
+        // This is a kludge, and such entries are added at the end of the candidate
+        // list; from 6.9 on, this is fixed in QLocale::uiLanguages().
+        QStringList fallbacks;
+        const auto addIfNew = [&duplicates, &fallbacks](const QString &fallback) {
+            if (!duplicates.hasSeen(fallback))
+                fallbacks.append(fallback);
+        };
+
+        while (true) {
+            const qsizetype last = language.lastIndexOf(u'_');
+            if (last < 0) // no more sections
+                break;
+
+            const qsizetype first = language.indexOf(u'_');
+             // two sections, add fallback without script
+            if (first != last && language.count(u'_') == 2) {
+                QString fallback = language.left(first) + language.mid(last);
+                addIfNew(fallback);
+            }
+            QString fallback = language.left(last);
+            addIfNew(fallback);
+
+            language.truncate(last);
+        }
+        for (qsizetype j = fallbacks.size() - 1; j >= 0; --j)
+            languages.insert(i + 1, fallbacks.at(j));
+    }
+
+    qCDebug(lcTranslator) << "Augmented UI languages" << languages;
+    for (qsizetype i = languages.size() - 1; i >= 0; --i) {
+        const QString &lang = languages.at(i);
         QString lowerLang = lang.toLower();
         if (lang != lowerLang)
             languages.insert(i + 1, lowerLang);
     }
 
     for (QString localeName : std::as_const(languages)) {
-        localeName.replace(u'-', u'_');
+        // try each locale with and without suffix
+        realname += localeName + suffixOrDotQM;
+        if (is_readable_file(realname))
+            return realname;
 
-        // try the complete locale name first and progressively truncate from
-        // the end until a matching language tag is found (with or without suffix)
-        for (;;) {
-            realname += localeName + suffixOrDotQM;
-            if (is_readable_file(realname))
-                return realname;
+        realname.truncate(realNameBaseSize + localeName.size());
+        if (is_readable_file(realname))
+            return realname;
 
-            realname.truncate(realNameBaseSize + localeName.size());
-            if (is_readable_file(realname))
-                return realname;
-
-            realname.truncate(realNameBaseSize);
-
-            int rightmost = localeName.lastIndexOf(u'_');
-            if (rightmost <= 0)
-                break; // no truncations anymore, break
-            localeName.truncate(rightmost);
-        }
+        realname.truncate(realNameBaseSize);
     }
 
     const int realNameBaseSizeFallbacks = path.size() + filename.size();
