@@ -305,11 +305,18 @@ static QVariant::Private clonePrivate(const QVariant::Private &other)
     if (d.is_shared) {
         d.data.shared->ref.ref();
     } else if (const QtPrivate::QMetaTypeInterface *iface = d.typeInterface()) {
-        Q_ASSERT(d.canUseInternalSpace(iface));
+        if (Q_LIKELY(d.canUseInternalSpace(iface))) {
+            // if not trivially copyable, ask to copy (if it's trivially
+            // copyable, we've already copied it)
+            if (iface->copyCtr)
+                QtMetaTypePrivate::copyConstruct(iface, d.data.data, other.data.data);
+        } else {
+            // highly unlikely, but possible case: type has changed relocatability
+            // between builds
+            d.data.shared = QVariant::PrivateShared::create(iface->size, iface->alignment);
+            QtMetaTypePrivate::copyConstruct(iface, d.data.shared->data(), other.data.data);
+        }
 
-        // if not trivially copyable, ask to copy
-        if (iface->copyCtr)
-            QtMetaTypePrivate::copyConstruct(iface, d.data.data, other.data.data);
     }
     return d;
 }
