@@ -1,42 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Copyright (C) 2013 John Layt <jlayt@kde.org>
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2022 The Qt Company Ltd.
+// Copyright (C) 2013 John Layt <jlayt@kde.org>
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 
 #ifndef QTIMEZONEPRIVATE_P_H
@@ -56,6 +20,7 @@
 #include "qlist.h"
 #include "qtimezone.h"
 #include "private/qlocale_p.h"
+#include "private/qdatetime_p.h"
 
 #if QT_CONFIG(icu)
 #include <unicode/ucal.h>
@@ -69,22 +34,36 @@ Q_FORWARD_DECLARE_OBJC_CLASS(NSTimeZone);
 #include <qt_windows.h>
 #endif // Q_OS_WIN
 
-#if defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_EMBEDDED)
+#ifdef Q_OS_ANDROID
 #include <QJniObject>
 #endif
 
+QT_REQUIRE_CONFIG(timezone);
 QT_BEGIN_NAMESPACE
 
 class Q_AUTOTEST_EXPORT QTimeZonePrivate : public QSharedData
 {
 public:
-    //Version of QTimeZone::OffsetData struct using msecs for efficiency
+    // Version of QTimeZone::OffsetData struct using msecs for efficiency
     struct Data {
         QString abbreviation;
         qint64 atMSecsSinceEpoch;
         int offsetFromUtc;
         int standardTimeOffset;
         int daylightTimeOffset;
+        Data()
+            : atMSecsSinceEpoch(QTimeZonePrivate::invalidMSecs()),
+              offsetFromUtc(QTimeZonePrivate::invalidSeconds()),
+              standardTimeOffset(QTimeZonePrivate::invalidSeconds()),
+              daylightTimeOffset(QTimeZonePrivate::invalidSeconds())
+        {}
+        Data(const QString &name, qint64 when, int offset, int standard)
+            : abbreviation(name),
+              atMSecsSinceEpoch(when),
+              offsetFromUtc(offset),
+              standardTimeOffset(standard),
+              daylightTimeOffset(offset - standard)
+        {}
     };
     typedef QList<Data> DataList;
 
@@ -120,7 +99,10 @@ public:
     virtual bool isDaylightTime(qint64 atMSecsSinceEpoch) const;
 
     virtual Data data(qint64 forMSecsSinceEpoch) const;
-    Data dataForLocalTime(qint64 forLocalMSecs, int hint) const;
+    virtual Data data(QTimeZone::TimeType timeType) const;
+    virtual bool isDataLocale(const QLocale &locale) const;
+    QDateTimePrivate::ZoneState stateAtZoneTime(qint64 forLocalMSecs,
+                                                QDateTimePrivate::TransitionOptions resolve) const;
 
     virtual bool hasTransitions() const;
     virtual Data nextTransition(qint64 afterMSecsSinceEpoch) const;
@@ -137,17 +119,21 @@ public:
     virtual void serialize(QDataStream &ds) const;
 
     // Static Utility Methods
-    static inline qint64 maxMSecs() { return std::numeric_limits<qint64>::max(); }
-    static inline qint64 minMSecs() { return std::numeric_limits<qint64>::min() + 1; }
-    static inline qint64 invalidMSecs() { return std::numeric_limits<qint64>::min(); }
-    static inline qint64 invalidSeconds() { return std::numeric_limits<int>::min(); }
-    static Data invalidData();
+    [[nodiscard]] static constexpr qint64 maxMSecs()
+    { return (std::numeric_limits<qint64>::max)(); }
+    [[nodiscard]] static constexpr qint64 minMSecs()
+    { return (std::numeric_limits<qint64>::min)() + 1; }
+    [[nodiscard]] static constexpr qint64 invalidMSecs()
+    { return (std::numeric_limits<qint64>::min)(); }
+    [[nodiscard]] static constexpr int invalidSeconds()
+    { return (std::numeric_limits<int>::min)(); }
     static QTimeZone::OffsetData invalidOffsetData();
     static QTimeZone::OffsetData toOffsetData(const Data &data);
     static bool isValidId(const QByteArray &ianaId);
     static QString isoOffsetFormat(int offsetFromUtc,
                                    QTimeZone::NameType mode = QTimeZone::OffsetName);
 
+    static QByteArray aliasToIana(QByteArrayView alias);
     static QByteArray ianaIdToWindowsId(const QByteArray &ianaId);
     static QByteArray windowsIdToDefaultIanaId(const QByteArray &windowsId);
     static QByteArray windowsIdToDefaultIanaId(const QByteArray &windowsId,
@@ -167,6 +153,21 @@ public:
         return QByteArrayLiteral("UTC");
     }
 
+protected:
+    // Zones CLDR data says match a condition.
+    // Use to filter what the backend has available.
+    QList<QByteArrayView> matchingTimeZoneIds(QLocale::Territory territory) const;
+    QList<QByteArrayView> matchingTimeZoneIds(int utcOffset) const;
+
+#if QT_CONFIG(timezone_locale)
+private:
+    // Defined in qtimezonelocale.cpp
+    QString localeName(qint64 atMSecsSinceEpoch, int offsetFromUtc,
+                       QTimeZone::TimeType timeType,
+                       QTimeZone::NameType nameType,
+                       const QLocale &locale) const;
+#endif // L10n helpers.
+
 protected:
     QByteArray m_id;
 };
@@ -191,15 +192,20 @@ public:
     virtual ~QUtcTimeZonePrivate();
 
     // Fall-back for UTC[+-]\d+(:\d+){,2} IDs.
-    static qint64 offsetFromUtcString(const QByteArray &id);
+    static qint64 offsetFromUtcString(QByteArrayView id);
 
     QUtcTimeZonePrivate *clone() const override;
 
     Data data(qint64 forMSecsSinceEpoch) const override;
+    Data data(QTimeZone::TimeType timeType) const override;
+    bool isDataLocale(const QLocale &locale) const override;
 
     QLocale::Territory territory() const override;
     QString comment() const override;
 
+    QString displayName(qint64 atMSecsSinceEpoch,
+                        QTimeZone::NameType nameType,
+                        const QLocale &locale) const override;
     QString displayName(QTimeZone::TimeType timeType,
                         QTimeZone::NameType nameType,
                         const QLocale &locale) const override;
@@ -218,7 +224,6 @@ public:
     void serialize(QDataStream &ds) const override;
 
 private:
-    void init(const QByteArray &zoneId);
     void init(const QByteArray &zoneId, int offsetSeconds, const QString &name,
               const QString &abbreviation, QLocale::Territory territory,
               const QString &comment);
@@ -230,7 +235,10 @@ private:
     int m_offsetFromUtc;
 };
 
-#if QT_CONFIG(icu)
+// TODO: shuffle (almost reverse) order of and rework #if-ery here to use #elif
+// and match the #if-ery in each of QTZ's newBackendTimeZone() cascades for
+// backend selection.
+#if QT_CONFIG(icu) && !defined(Q_OS_UNIX)
 class Q_AUTOTEST_EXPORT QIcuTimeZonePrivate final : public QTimeZonePrivate
 {
 public:
@@ -246,7 +254,6 @@ public:
     using QTimeZonePrivate::displayName;
     QString displayName(QTimeZone::TimeType timeType, QTimeZone::NameType nameType,
                         const QLocale &locale) const override;
-    QString abbreviation(qint64 atMSecsSinceEpoch) const override;
 
     int offsetFromUtc(qint64 atMSecsSinceEpoch) const override;
     int standardTimeOffset(qint64 atMSecsSinceEpoch) const override;
@@ -255,6 +262,7 @@ public:
     bool hasDaylightTime() const override;
     bool isDaylightTime(qint64 atMSecsSinceEpoch) const override;
 
+    using QTimeZonePrivate::data;
     Data data(qint64 forMSecsSinceEpoch) const override;
 
     bool hasTransitions() const override;
@@ -263,6 +271,7 @@ public:
 
     QByteArray systemTimeZoneId() const override;
 
+    bool isTimeZoneIdAvailable(const QByteArray &ianaId) const override;
     QList<QByteArray> availableTimeZoneIds() const override;
     QList<QByteArray> availableTimeZoneIds(QLocale::Territory territory) const override;
     QList<QByteArray> availableTimeZoneIds(int offsetFromUtc) const override;
@@ -272,9 +281,9 @@ private:
 
     UCalendar *m_ucal;
 };
-#endif
+#endif // ICU not on Unix.
 
-#if defined(Q_OS_UNIX) && !defined(Q_OS_DARWIN) && (!defined(Q_OS_ANDROID) || defined(Q_OS_ANDROID_EMBEDDED))
+#if defined(Q_OS_UNIX) && !defined(Q_OS_DARWIN) && !defined(Q_OS_ANDROID)
 struct QTzTransitionTime
 {
     qint64 atMSecsSinceEpoch;
@@ -283,9 +292,9 @@ struct QTzTransitionTime
 Q_DECLARE_TYPEINFO(QTzTransitionTime, Q_PRIMITIVE_TYPE);
 struct QTzTransitionRule
 {
-    int stdOffset;
-    int dstOffset;
-    quint8 abbreviationIndex;
+    int stdOffset = 0;
+    int dstOffset = 0;
+    quint8 abbreviationIndex = 0;
 };
 Q_DECLARE_TYPEINFO(QTzTransitionRule, Q_PRIMITIVE_TYPE);
 constexpr inline bool operator==(const QTzTransitionRule &lhs, const QTzTransitionRule &rhs) noexcept
@@ -302,7 +311,7 @@ struct QTzTimeZoneCacheEntry
     QList<QByteArray> m_abbreviations;
     QByteArray m_posixRule;
     QTzTransitionRule m_preZoneRule;
-    bool m_hasDst;
+    bool m_hasDst = false;
 };
 
 class Q_AUTOTEST_EXPORT QTzTimeZonePrivate final : public QTimeZonePrivate
@@ -320,9 +329,7 @@ public:
     QLocale::Territory territory() const override;
     QString comment() const override;
 
-    QString displayName(qint64 atMSecsSinceEpoch,
-                        QTimeZone::NameType nameType,
-                        const QLocale &locale) const override;
+    using QTimeZonePrivate::displayName;
     QString displayName(QTimeZone::TimeType timeType,
                         QTimeZone::NameType nameType,
                         const QLocale &locale) const override;
@@ -336,6 +343,8 @@ public:
     bool isDaylightTime(qint64 atMSecsSinceEpoch) const override;
 
     Data data(qint64 forMSecsSinceEpoch) const override;
+    Data data(QTimeZone::TimeType timeType) const override;
+    bool isDataLocale(const QLocale &locale) const override;
 
     bool hasTransitions() const override;
     Data nextTransition(qint64 afterMSecsSinceEpoch) const override;
@@ -353,20 +362,12 @@ private:
 
     Data dataForTzTransition(QTzTransitionTime tran) const;
     Data dataFromRule(QTzTransitionRule rule, qint64 msecsSinceEpoch) const;
-#if QT_CONFIG(icu)
-# ifdef __cpp_lib_is_final
-    static_assert(std::is_final<QIcuTimeZonePrivate>::value,
-                  "if QIcuTimeZonePrivate isn't final, we may need to specialize "
-                  "QExplicitlySharedDataPointer::clone() to call QTimeZonePrivate::clone()");
-# endif
-    mutable QExplicitlySharedDataPointer<const QIcuTimeZonePrivate> m_icu;
-#endif
     QTzTimeZoneCacheEntry cached_data;
-    QList<QTzTransitionTime> tranCache() const { return cached_data.m_tranTimes; }
+    const QList<QTzTransitionTime> &tranCache() const { return cached_data.m_tranTimes; }
 };
 #endif // Q_OS_UNIX
 
-#ifdef Q_OS_MAC
+#ifdef Q_OS_DARWIN
 class Q_AUTOTEST_EXPORT QMacTimeZonePrivate final : public QTimeZonePrivate
 {
 public:
@@ -381,6 +382,7 @@ public:
 
     QString comment() const override;
 
+    using QTimeZonePrivate::displayName;
     QString displayName(QTimeZone::TimeType timeType, QTimeZone::NameType nameType,
                         const QLocale &locale) const override;
     QString abbreviation(qint64 atMSecsSinceEpoch) const override;
@@ -392,6 +394,7 @@ public:
     bool hasDaylightTime() const override;
     bool isDaylightTime(qint64 atMSecsSinceEpoch) const override;
 
+    using QTimeZonePrivate::data;
     Data data(qint64 forMSecsSinceEpoch) const override;
 
     bool hasTransitions() const override;
@@ -399,7 +402,7 @@ public:
     Data previousTransition(qint64 beforeMSecsSinceEpoch) const override;
 
     QByteArray systemTimeZoneId() const override;
-
+    bool isTimeZoneIdAvailable(const QByteArray &ianaId) const override;
     QList<QByteArray> availableTimeZoneIds() const override;
 
     NSTimeZone *nsTimeZone() const;
@@ -409,9 +412,9 @@ private:
 
     NSTimeZone *m_nstz;
 };
-#endif // Q_OS_MAC
+#endif // Q_OS_DARWIN
 
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WIN) && !QT_CONFIG(icu)
 class Q_AUTOTEST_EXPORT QWinTimeZonePrivate final : public QTimeZonePrivate
 {
 public:
@@ -434,6 +437,7 @@ public:
 
     QString comment() const override;
 
+    using QTimeZonePrivate::displayName;
     QString displayName(QTimeZone::TimeType timeType, QTimeZone::NameType nameType,
                         const QLocale &locale) const override;
     QString abbreviation(qint64 atMSecsSinceEpoch) const override;
@@ -445,6 +449,7 @@ public:
     bool hasDaylightTime() const override;
     bool isDaylightTime(qint64 atMSecsSinceEpoch) const override;
 
+    using QTimeZonePrivate::data;
     Data data(qint64 forMSecsSinceEpoch) const override;
 
     bool hasTransitions() const override;
@@ -455,10 +460,11 @@ public:
 
     QList<QByteArray> availableTimeZoneIds() const override;
 
-private:
-    void init(const QByteArray &ianaId);
+    // For use within implementation's TransitionTimePair:
     QTimeZonePrivate::Data ruleToData(const QWinTransitionRule &rule, qint64 atMSecsSinceEpoch,
                                       QTimeZone::TimeType type, bool fakeDst = false) const;
+private:
+    void init(const QByteArray &ianaId);
 
     QByteArray m_windowsId;
     QString m_displayName;
@@ -466,9 +472,9 @@ private:
     QString m_daylightName;
     QList<QWinTransitionRule> m_tranRules;
 };
-#endif // Q_OS_WIN
+#endif // Q_OS_WIN && !icu
 
-#if defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_EMBEDDED)
+#ifdef Q_OS_ANDROID
 class QAndroidTimeZonePrivate final : public QTimeZonePrivate
 {
 public:
@@ -481,6 +487,7 @@ public:
 
     QAndroidTimeZonePrivate *clone() const override;
 
+    using QTimeZonePrivate::displayName;
     QString displayName(QTimeZone::TimeType timeType, QTimeZone::NameType nameType,
                         const QLocale &locale) const override;
     QString abbreviation(qint64 atMSecsSinceEpoch) const override;
@@ -492,10 +499,11 @@ public:
     bool hasDaylightTime() const override;
     bool isDaylightTime(qint64 atMSecsSinceEpoch) const override;
 
+    using QTimeZonePrivate::data;
     Data data(qint64 forMSecsSinceEpoch) const override;
 
     QByteArray systemTimeZoneId() const override;
-
+    bool isTimeZoneIdAvailable(const QByteArray &ianaId) const override;
     QList<QByteArray> availableTimeZoneIds() const override;
 
 private:
