@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2022 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qsize.h"
 #include "qdatastream.h"
@@ -302,15 +266,15 @@ QSize QSize::scaled(const QSize &s, Qt::AspectRatioMode mode) const noexcept
 */
 
 /*!
-    \fn bool QSize::operator==(const QSize &s1, const QSize &s2)
+    \fn bool QSize::operator==(const QSize &lhs, const QSize &rhs)
 
-    Returns \c true if \a s1 and \a s2 are equal; otherwise returns \c false.
+    Returns \c true if \a lhs and \a rhs are equal; otherwise returns \c false.
 */
 
 /*!
-    \fn bool QSize::operator!=(const QSize &s1, const QSize &s2)
+    \fn bool QSize::operator!=(const QSize &lhs, const QSize &rhs)
 
-    Returns \c true if \a s1 and \a s2 are different; otherwise returns \c false.
+    Returns \c true if \a lhs and \a rhs are different; otherwise returns \c false.
 */
 
 /*!
@@ -403,6 +367,15 @@ QSize QSize::scaled(const QSize &s, Qt::AspectRatioMode mode) const noexcept
     \sa grownBy()
 */
 
+/*!
+    \fn QSize::toSizeF() const
+    \since 6.4
+
+    Returns this size as a size with floating point accuracy.
+
+    \sa QSizeF::toSize()
+*/
+
 /*****************************************************************************
   QSize stream functions
  *****************************************************************************/
@@ -520,7 +493,7 @@ QDebug operator<<(QDebug dbg, const QSize &s)
     Constructs a size with floating point accuracy from the given \a
     size.
 
-    \sa toSize()
+    \sa toSize(), QSize::toSizeF()
 */
 
 /*!
@@ -596,7 +569,7 @@ QDebug operator<<(QDebug dbg, const QSize &s)
     Note that the coordinates in the returned size will be rounded to
     the nearest integer.
 
-    \sa QSizeF()
+    \sa QSizeF(), QSize::toSizeF()
 */
 
 /*!
@@ -741,9 +714,9 @@ QSizeF QSizeF::scaled(const QSizeF &s, Qt::AspectRatioMode mode) const noexcept
 */
 
 /*!
-    \fn bool QSizeF::operator==(const QSizeF &s1, const QSizeF &s2)
+    \fn bool QSizeF::operator==(const QSizeF &lhs, const QSizeF &rhs)
 
-    Returns \c true if \a s1 and \a s2 are approximately equal; otherwise
+    Returns \c true if \a lhs and \a rhs are approximately equal; otherwise
     returns false.
 
     \warning This function does not check for strict equality; instead,
@@ -753,9 +726,9 @@ QSizeF QSizeF::scaled(const QSizeF &s, Qt::AspectRatioMode mode) const noexcept
 */
 
 /*!
-    \fn bool QSizeF::operator!=(const QSizeF &s1, const QSizeF &s2)
+    \fn bool QSizeF::operator!=(const QSizeF &lhs, const QSizeF &rhs)
 
-    Returns \c true if \a s1 and \a s2 are sufficiently different; otherwise
+    Returns \c true if \a lhs and \a rhs are sufficiently different; otherwise
     returns \c false.
 
     \warning This function does not check for strict inequality; instead,
@@ -835,7 +808,24 @@ QSizeF QSizeF::scaled(const QSizeF &s, Qt::AspectRatioMode mode) const noexcept
     \sa expandedTo(), scale()
 */
 
+/*!
+    \fn bool QSizeF::qFuzzyCompare(const QSizeF &lhs, const QSizeF &rhs)
+    \since 6.8
 
+    Returns \c true if the size \a lhs is approximately equal to the
+    size \a rhs; otherwise returns \c false.
+
+    The sizes are considered approximately equal if their width and
+    height are approximately equal.
+*/
+
+/*!
+    \fn bool QSizeF::qFuzzyIsNull(const QSizeF &size)
+    \since 6.8
+
+    Returns \c true if both width and height of the size \a size
+    are approximately equal to zero.
+*/
 
 /*****************************************************************************
   QSizeF stream functions
