@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Copyright (C) 2004, 2005 Daniel M. Duley., (C) Carsten Haitzler and various contributors., (C) Willem Monsuwe <willem@stack.nl>
+// SPDX-License-Identifier: BSD-2-Clause AND Imlib2
 #include <private/qimagescale_p.h>
 #include <private/qdrawhelper_p.h>
 #include <private/qimage_p.h>
@@ -10,8 +11,9 @@
 #include "qrgbafloat.h"
 
 #if QT_CONFIG(thread) && !defined(Q_OS_WASM)
-#include "qsemaphore.h"
-#include "qthreadpool.h"
+#include <qsemaphore.h>
+#include <qthreadpool.h>
+#include <private/qthreadpool_p.h>
 #endif
 
 QT_BEGIN_NAMESPACE
@@ -273,7 +275,7 @@ static inline void multithread_pixels_function(QImageScaleInfo *isi, int dh, con
 #if QT_CONFIG(thread) && !defined(Q_OS_WASM)
     int segments = (qsizetype(isi->sh) * isi->sw) / (1<<16);
     segments = std::min(segments, dh);
-    QThreadPool *threadPool = QThreadPool::globalInstance();
+    QThreadPool *threadPool = QThreadPoolPrivate::qtGuiInstance();
     if (segments > 1 && threadPool && !threadPool->contains(QThread::currentThread())) {
         QSemaphore semaphore;
         int y = 0;
@@ -1207,7 +1209,7 @@ QImage qSmoothScaleImage(const QImage &src, int dw, int dh)
                              dw, dh, dw, src.bytesPerLine() / 8);
     else
 #endif
-    if (src.hasAlphaChannel())
+    if (src.hasAlphaChannel() || src.format() == QImage::Format_CMYK8888)
         qt_qimageScaleAARGBA(scaleinfo, (unsigned int *)buffer.scanLine(0),
                              dw, dh, dw, src.bytesPerLine() / 4);
     else
