@@ -78,17 +78,14 @@ static const char defaultFixedFontNameC[] = "monospace";
 enum { defaultSystemFontSize = 9 };
 
 #if !defined(QT_NO_DBUS) && !defined(QT_NO_SYSTEMTRAYICON)
-static bool isDBusTrayAvailable() {
-    static bool dbusTrayAvailable = false;
-    static bool dbusTrayAvailableKnown = false;
-    if (!dbusTrayAvailableKnown) {
-        QDBusMenuConnection conn;
-        if (conn.isWatcherRegistered())
-            dbusTrayAvailable = true;
-        dbusTrayAvailableKnown = true;
-        qCDebug(qLcTray) << "D-Bus tray available:" << dbusTrayAvailable;
-    }
-    return dbusTrayAvailable;
+static bool shouldUseDBusTray() {
+    // There's no other tray implementation to fallback to on non-X11
+    // and QDBusTrayIcon can register the icon on the fly after creation
+    if (QGuiApplication::platformName() != "xcb"_L1)
+        return true;
+    const bool result = QDBusMenuConnection().isWatcherRegistered();
+    qCDebug(qLcTray) << "D-Bus tray available:" << result;
+    return result;
 }
 #endif
 
@@ -476,7 +473,7 @@ QPlatformMenuBar *QGenericUnixTheme::createPlatformMenuBar() const
 #if !defined(QT_NO_DBUS) && !defined(QT_NO_SYSTEMTRAYICON)
 QPlatformSystemTrayIcon *QGenericUnixTheme::createPlatformSystemTrayIcon() const
 {
-    if (isDBusTrayAvailable())
+    if (shouldUseDBusTray())
         return new QDBusTrayIcon();
     return nullptr;
 }
@@ -506,6 +503,8 @@ QVariant QGenericUnixTheme::themeHint(ThemeHint hint) const
         return QVariant(mouseCursorTheme());
     case QPlatformTheme::MouseCursorSize:
         return QVariant(mouseCursorSize());
+    case QPlatformTheme::PreferFileIconFromTheme:
+        return true;
     default:
         break;
     }
@@ -541,6 +540,48 @@ class QKdeThemePrivate : public QPlatformThemePrivate
 {
 
 public:
+    enum class KdeSettingType {
+        Root,
+        KDE,
+        Icons,
+        ToolBarIcons,
+        ToolBarStyle,
+        Fonts,
+        Colors,
+    };
+
+    enum class KdeSetting {
+        WidgetStyle,
+        ColorScheme,
+        SingleClick,
+        ShowIconsOnPushButtons,
+        IconTheme,
+        ToolBarIconSize,
+        ToolButtonStyle,
+        WheelScrollLines,
+        DoubleClickInterval,
+        StartDragDistance,
+        StartDragTime,
+        CursorBlinkRate,
+        Font,
+        Fixed,
+        MenuFont,
+        ToolBarFont,
+        ButtonBackground,
+        WindowBackground,
+        ViewForeground,
+        WindowForeground,
+        ViewBackground,
+        SelectionBackground,
+        SelectionForeground,
+        ViewBackgroundAlternate,
+        ButtonForeground,
+        ViewForegroundLink,
+        ViewForegroundVisited,
+        TooltipBackground,
+        TooltipForeground,
+    };
+
     QKdeThemePrivate(const QStringList &kdeDirs, int kdeVersion);
 
     static QString kdeGlobals(const QString &kdeDir, int kdeVersion)
@@ -551,7 +592,9 @@ public:
     }
 
     void refresh();
-    static QVariant readKdeSetting(const QString &key, const QStringList &kdeDirs, int kdeVersion, QHash<QString, QSettings*> &kdeSettings);
+    static QVariant readKdeSetting(KdeSetting s, const QStringList &kdeDirs, int kdeVersion, QHash<QString, QSettings*> &settings);
+    QVariant readKdeSetting(KdeSetting s) const;
+    void clearKdeSettings() const;
     static void readKdeSystemPalette(const QStringList &kdeDirs, int kdeVersion, QHash<QString, QSettings*> &kdeSettings, QPalette *pal);
     static QFont *kdeFont(const QVariant &fontValue);
     static QStringList kdeIconThemeSearchPaths(const QStringList &kdeDirs);
@@ -575,8 +618,9 @@ public:
     Qt::ColorScheme m_colorScheme = Qt::ColorScheme::Unknown;
     void updateColorScheme(const QString &themeName);
 
-#ifndef QT_NO_DBUS
 private:
+    mutable QHash<QString, QSettings *> kdeSettings;
+#ifndef QT_NO_DBUS
     std::unique_ptr<QGenericUnixThemeDBusListener> dbus;
     bool initDbus();
     void settingChangedHandler(QGenericUnixThemeDBusListener::Provider provider,
@@ -620,7 +664,7 @@ bool QKdeThemePrivate::initDbus()
         settingChangedHandler(provider, setting, value);
     };
 
-    return QObject::connect(dbus.get(), &QGenericUnixThemeDBusListener::settingChanged, wrapper);
+    return QObject::connect(dbus.get(), &QGenericUnixThemeDBusListener::settingChanged, dbus.get(), wrapper);
 }
 #endif // QT_NO_DBUS
 
@@ -632,9 +676,151 @@ QKdeThemePrivate::QKdeThemePrivate(const QStringList &kdeDirs, int kdeVersion)
 #endif // QT_NO_DBUS
 }
 
+static constexpr QLatin1StringView settingsPrefix(QKdeThemePrivate::KdeSettingType type)
+{
+    switch (type) {
+    case QKdeThemePrivate::KdeSettingType::Root:
+        return QLatin1StringView();
+    case QKdeThemePrivate::KdeSettingType::KDE:
+        return QLatin1StringView("KDE/");
+    case QKdeThemePrivate::KdeSettingType::Fonts:
+        return QLatin1StringView();
+    case QKdeThemePrivate::KdeSettingType::Colors:
+        return QLatin1StringView("Colors:");
+    case QKdeThemePrivate::KdeSettingType::Icons:
+        return QLatin1StringView("Icons/");
+    case QKdeThemePrivate::KdeSettingType::ToolBarIcons:
+        return QLatin1StringView("ToolbarIcons/");
+    case QKdeThemePrivate::KdeSettingType::ToolBarStyle:
+        return QLatin1StringView("Toolbar style/");
+    }
+    // GCC 8.x does not treat __builtin_unreachable() as constexpr
+#  if !defined(Q_CC_GNU_ONLY) || (Q_CC_GNU >= 900)
+    // NOLINTNEXTLINE(qt-use-unreachable-return): Triggers on Clang, breaking GCC 8
+    Q_UNREACHABLE();
+#  endif
+    return {};
+}
+
+static constexpr QKdeThemePrivate::KdeSettingType settingsType(QKdeThemePrivate::KdeSetting setting)
+{
+#define CASE(s, type) case QKdeThemePrivate::KdeSetting::s:\
+        return QKdeThemePrivate::KdeSettingType::type
+
+    switch (setting) {
+    CASE(WidgetStyle, Root);
+    CASE(ColorScheme, Root);
+    CASE(SingleClick, KDE);
+    CASE(ShowIconsOnPushButtons, KDE);
+    CASE(IconTheme, Icons);
+    CASE(ToolBarIconSize, ToolBarIcons);
+    CASE(ToolButtonStyle, ToolBarStyle);
+    CASE(WheelScrollLines, KDE);
+    CASE(DoubleClickInterval, KDE);
+    CASE(StartDragDistance, KDE);
+    CASE(StartDragTime, KDE);
+    CASE(CursorBlinkRate, KDE);
+    CASE(Font, Root);
+    CASE(Fixed, Root);
+    CASE(MenuFont, Root);
+    CASE(ToolBarFont, Root);
+    CASE(ButtonBackground, Colors);
+    CASE(WindowBackground, Colors);
+    CASE(ViewForeground, Colors);
+    CASE(WindowForeground, Colors);
+    CASE(ViewBackground, Colors);
+    CASE(SelectionBackground, Colors);
+    CASE(SelectionForeground, Colors);
+    CASE(ViewBackgroundAlternate, Colors);
+    CASE(ButtonForeground, Colors);
+    CASE(ViewForegroundLink, Colors);
+    CASE(ViewForegroundVisited, Colors);
+    CASE(TooltipBackground, Colors);
+    CASE(TooltipForeground, Colors);
+    };
+    // GCC 8.x does not treat __builtin_unreachable() as constexpr
+#  if !defined(Q_CC_GNU_ONLY) || (Q_CC_GNU >= 900)
+    // NOLINTNEXTLINE(qt-use-unreachable-return): Triggers on Clang, breaking GCC 8
+    Q_UNREACHABLE();
+#  endif
+    return QKdeThemePrivate::KdeSettingType::Root;
+}
+#undef CASE
+
+static constexpr QLatin1StringView settingsKey(QKdeThemePrivate::KdeSetting setting)
+{
+    switch (setting) {
+    case QKdeThemePrivate::KdeSetting::WidgetStyle:
+        return QLatin1StringView("widgetStyle");
+    case QKdeThemePrivate::KdeSetting::ColorScheme:
+        return QLatin1StringView("ColorScheme");
+    case QKdeThemePrivate::KdeSetting::SingleClick:
+        return QLatin1StringView("SingleClick");
+    case QKdeThemePrivate::KdeSetting::ShowIconsOnPushButtons:
+        return QLatin1StringView("ShowIconsOnPushButtons");
+    case QKdeThemePrivate::KdeSetting::IconTheme:
+        return QLatin1StringView("Theme");
+    case QKdeThemePrivate::KdeSetting::ToolBarIconSize:
+        return QLatin1StringView("Size");
+    case QKdeThemePrivate::KdeSetting::ToolButtonStyle:
+        return QLatin1StringView("ToolButtonStyle");
+    case QKdeThemePrivate::KdeSetting::WheelScrollLines:
+        return QLatin1StringView("WheelScrollLines");
+    case QKdeThemePrivate::KdeSetting::DoubleClickInterval:
+        return QLatin1StringView("DoubleClickInterval");
+    case QKdeThemePrivate::KdeSetting::StartDragDistance:
+        return QLatin1StringView("StartDragDist");
+    case QKdeThemePrivate::KdeSetting::StartDragTime:
+        return QLatin1StringView("StartDragTime");
+    case QKdeThemePrivate::KdeSetting::CursorBlinkRate:
+        return QLatin1StringView("CursorBlinkRate");
+    case QKdeThemePrivate::KdeSetting::Font:
+        return QLatin1StringView("font");
+    case QKdeThemePrivate::KdeSetting::Fixed:
+        return QLatin1StringView("fixed");
+    case QKdeThemePrivate::KdeSetting::MenuFont:
+        return QLatin1StringView("menuFont");
+    case QKdeThemePrivate::KdeSetting::ToolBarFont:
+        return QLatin1StringView("toolBarFont");
+    case QKdeThemePrivate::KdeSetting::ButtonBackground:
+        return QLatin1StringView("Button/BackgroundNormal");
+    case QKdeThemePrivate::KdeSetting::WindowBackground:
+        return QLatin1StringView("Window/BackgroundNormal");
+    case QKdeThemePrivate::KdeSetting::ViewForeground:
+        return QLatin1StringView("View/ForegroundNormal");
+    case QKdeThemePrivate::KdeSetting::WindowForeground:
+        return QLatin1StringView("Window/ForegroundNormal");
+    case QKdeThemePrivate::KdeSetting::ViewBackground:
+        return QLatin1StringView("View/BackgroundNormal");
+    case QKdeThemePrivate::KdeSetting::SelectionBackground:
+        return QLatin1StringView("Selection/BackgroundNormal");
+    case QKdeThemePrivate::KdeSetting::SelectionForeground:
+        return QLatin1StringView("Selection/ForegroundNormal");
+    case QKdeThemePrivate::KdeSetting::ViewBackgroundAlternate:
+        return QLatin1StringView("View/BackgroundAlternate");
+    case QKdeThemePrivate::KdeSetting::ButtonForeground:
+        return QLatin1StringView("Button/ForegroundNormal");
+    case QKdeThemePrivate::KdeSetting::ViewForegroundLink:
+        return QLatin1StringView("View/ForegroundLink");
+    case QKdeThemePrivate::KdeSetting::ViewForegroundVisited:
+        return QLatin1StringView("View/ForegroundVisited");
+    case QKdeThemePrivate::KdeSetting::TooltipBackground:
+        return QLatin1StringView("Tooltip/BackgroundNormal");
+    case QKdeThemePrivate::KdeSetting::TooltipForeground:
+        return QLatin1StringView("Tooltip/ForegroundNormal");
+    };
+    // GCC 8.x does not treat __builtin_unreachable() as constexpr
+#  if !defined(Q_CC_GNU_ONLY) || (Q_CC_GNU >= 900)
+    // NOLINTNEXTLINE(qt-use-unreachable-return): Triggers on Clang, breaking GCC 8
+    Q_UNREACHABLE();
+#  endif
+    return {};
+}
+
 void QKdeThemePrivate::refresh()
 {
     resources.clear();
+    clearKdeSettings();
 
     toolButtonStyle = Qt::ToolButtonTextBesideIcon;
     toolBarIconSize = 0;
@@ -647,45 +833,43 @@ void QKdeThemePrivate::refresh()
     else
         iconFallbackThemeName = iconThemeName = QStringLiteral("oxygen");
 
-    QHash<QString, QSettings*> kdeSettings;
-
     QPalette systemPalette = QPalette();
     readKdeSystemPalette(kdeDirs, kdeVersion, kdeSettings, &systemPalette);
     resources.palettes[QPlatformTheme::SystemPalette] = new QPalette(systemPalette);
     //## TODO tooltip color
 
-    const QVariant styleValue = readKdeSetting(QStringLiteral("widgetStyle"), kdeDirs, kdeVersion, kdeSettings);
+    const QVariant styleValue = readKdeSetting(KdeSetting::WidgetStyle);
     if (styleValue.isValid()) {
         const QString style = styleValue.toString();
         if (style != styleNames.front())
             styleNames.push_front(style);
     }
 
-    const QVariant colorScheme = readKdeSetting(QStringLiteral("ColorScheme"), kdeDirs,
-                                                   kdeVersion, kdeSettings);
+    const QVariant colorScheme = readKdeSetting(KdeSetting::ColorScheme);
 
-    if (colorScheme.isValid())
-        updateColorScheme(colorScheme.toString());
-    else
-        m_colorScheme = Qt::ColorScheme::Unknown;
+    updateColorScheme(colorScheme.toString());
 
-    const QVariant singleClickValue = readKdeSetting(QStringLiteral("KDE/SingleClick"), kdeDirs, kdeVersion, kdeSettings);
+    const QVariant singleClickValue = readKdeSetting(KdeSetting::SingleClick);
     if (singleClickValue.isValid())
         singleClick = singleClickValue.toBool();
+    else if (kdeVersion >= 6) // Plasma 6 defaults to double-click
+        singleClick = false;
+    else // earlier version to single-click
+        singleClick = true;
 
-    const QVariant showIconsOnPushButtonsValue = readKdeSetting(QStringLiteral("KDE/ShowIconsOnPushButtons"), kdeDirs, kdeVersion, kdeSettings);
+    const QVariant showIconsOnPushButtonsValue = readKdeSetting(KdeSetting::ShowIconsOnPushButtons);
     if (showIconsOnPushButtonsValue.isValid())
         showIconsOnPushButtons = showIconsOnPushButtonsValue.toBool();
 
-    const QVariant themeValue = readKdeSetting(QStringLiteral("Icons/Theme"), kdeDirs, kdeVersion, kdeSettings);
+    const QVariant themeValue = readKdeSetting(KdeSetting::IconTheme);
     if (themeValue.isValid())
         iconThemeName = themeValue.toString();
 
-    const QVariant toolBarIconSizeValue = readKdeSetting(QStringLiteral("ToolbarIcons/Size"), kdeDirs, kdeVersion, kdeSettings);
+    const QVariant toolBarIconSizeValue = readKdeSetting(KdeSetting::ToolBarIconSize);
     if (toolBarIconSizeValue.isValid())
         toolBarIconSize = toolBarIconSizeValue.toInt();
 
-    const QVariant toolbarStyleValue = readKdeSetting(QStringLiteral("Toolbar style/ToolButtonStyle"), kdeDirs, kdeVersion, kdeSettings);
+    const QVariant toolbarStyleValue = readKdeSetting(KdeSetting::ToolButtonStyle);
     if (toolbarStyleValue.isValid()) {
         const QString toolBarStyle = toolbarStyleValue.toString();
         if (toolBarStyle == "TextBesideIcon"_L1)
@@ -696,35 +880,35 @@ void QKdeThemePrivate::refresh()
             toolButtonStyle = Qt::ToolButtonTextUnderIcon;
     }
 
-    const QVariant wheelScrollLinesValue = readKdeSetting(QStringLiteral("KDE/WheelScrollLines"), kdeDirs, kdeVersion, kdeSettings);
+    const QVariant wheelScrollLinesValue = readKdeSetting(KdeSetting::WheelScrollLines);
     if (wheelScrollLinesValue.isValid())
         wheelScrollLines = wheelScrollLinesValue.toInt();
 
-    const QVariant doubleClickIntervalValue = readKdeSetting(QStringLiteral("KDE/DoubleClickInterval"), kdeDirs, kdeVersion, kdeSettings);
+    const QVariant doubleClickIntervalValue = readKdeSetting(KdeSetting::DoubleClickInterval);
     if (doubleClickIntervalValue.isValid())
         doubleClickInterval = doubleClickIntervalValue.toInt();
 
-    const QVariant startDragDistValue = readKdeSetting(QStringLiteral("KDE/StartDragDist"), kdeDirs, kdeVersion, kdeSettings);
+    const QVariant startDragDistValue = readKdeSetting(KdeSetting::StartDragDistance);
     if (startDragDistValue.isValid())
         startDragDist = startDragDistValue.toInt();
 
-    const QVariant startDragTimeValue = readKdeSetting(QStringLiteral("KDE/StartDragTime"), kdeDirs, kdeVersion, kdeSettings);
+    const QVariant startDragTimeValue = readKdeSetting(KdeSetting::StartDragTime);
     if (startDragTimeValue.isValid())
         startDragTime = startDragTimeValue.toInt();
 
-    const QVariant cursorBlinkRateValue = readKdeSetting(QStringLiteral("KDE/CursorBlinkRate"), kdeDirs, kdeVersion, kdeSettings);
+    const QVariant cursorBlinkRateValue = readKdeSetting(KdeSetting::CursorBlinkRate);
     if (cursorBlinkRateValue.isValid()) {
         cursorBlinkRate = cursorBlinkRateValue.toInt();
         cursorBlinkRate = cursorBlinkRate > 0 ? qBound(200, cursorBlinkRate, 2000) : 0;
     }
 
     // Read system font, ignore 'smallestReadableFont'
-    if (QFont *systemFont = kdeFont(readKdeSetting(QStringLiteral("font"), kdeDirs, kdeVersion, kdeSettings)))
+    if (QFont *systemFont = kdeFont(readKdeSetting(KdeSetting::Font)))
         resources.fonts[QPlatformTheme::SystemFont] = systemFont;
     else
         resources.fonts[QPlatformTheme::SystemFont] = new QFont(QLatin1StringView(defaultSystemFontNameC), defaultSystemFontSize);
 
-    if (QFont *fixedFont = kdeFont(readKdeSetting(QStringLiteral("fixed"), kdeDirs, kdeVersion, kdeSettings))) {
+    if (QFont *fixedFont = kdeFont(readKdeSetting(KdeSetting::Fixed))) {
         resources.fonts[QPlatformTheme::FixedFont] = fixedFont;
     } else {
         fixedFont = new QFont(QLatin1StringView(defaultFixedFontNameC), defaultSystemFontSize);
@@ -732,12 +916,12 @@ void QKdeThemePrivate::refresh()
         resources.fonts[QPlatformTheme::FixedFont] = fixedFont;
     }
 
-    if (QFont *menuFont = kdeFont(readKdeSetting(QStringLiteral("menuFont"), kdeDirs, kdeVersion, kdeSettings))) {
+    if (QFont *menuFont = kdeFont(readKdeSetting(KdeSetting::MenuFont))) {
         resources.fonts[QPlatformTheme::MenuFont] = menuFont;
         resources.fonts[QPlatformTheme::MenuBarFont] = new QFont(*menuFont);
     }
 
-    if (QFont *toolBarFont = kdeFont(readKdeSetting(QStringLiteral("toolBarFont"), kdeDirs, kdeVersion, kdeSettings)))
+    if (QFont *toolBarFont = kdeFont(readKdeSetting(KdeSetting::ToolBarFont)))
         resources.fonts[QPlatformTheme::ToolButtonFont] = toolBarFont;
 
     QWindowSystemInterface::handleThemeChange();
@@ -747,7 +931,7 @@ void QKdeThemePrivate::refresh()
     qDeleteAll(kdeSettings);
 }
 
-QVariant QKdeThemePrivate::readKdeSetting(const QString &key, const QStringList &kdeDirs, int kdeVersion, QHash<QString, QSettings*> &kdeSettings)
+QVariant QKdeThemePrivate::readKdeSetting(KdeSetting s, const QStringList &kdeDirs, int kdeVersion, QHash<QString, QSettings*> &kdeSettings)
 {
     for (const QString &kdeDir : kdeDirs) {
         QSettings *settings = kdeSettings.value(kdeDir);
@@ -759,6 +943,7 @@ QVariant QKdeThemePrivate::readKdeSetting(const QString &key, const QStringList
             }
         }
         if (settings) {
+            const QString key = settingsPrefix(settingsType(s)) + settingsKey(s);
             const QVariant value = settings->value(key);
             if (value.isValid())
                 return value;
@@ -767,6 +952,16 @@ QVariant QKdeThemePrivate::readKdeSetting(const QString &key, const QStringList
     return QVariant();
 }
 
+QVariant QKdeThemePrivate::readKdeSetting(KdeSetting s) const
+{
+    return readKdeSetting(s, kdeDirs, kdeVersion, kdeSettings);
+}
+
+void QKdeThemePrivate::clearKdeSettings() const
+{
+    kdeSettings.clear();
+}
+
 // Reads the color from the KDE configuration, and store it in the
 // palette with the given color role if found.
 static inline bool kdeColor(QPalette *pal, QPalette::ColorRole role, const QVariant &value)
@@ -782,7 +977,7 @@ static inline bool kdeColor(QPalette *pal, QPalette::ColorRole role, const QVari
 
 void QKdeThemePrivate::readKdeSystemPalette(const QStringList &kdeDirs, int kdeVersion, QHash<QString, QSettings*> &kdeSettings, QPalette *pal)
 {
-    if (!kdeColor(pal, QPalette::Button, readKdeSetting(QStringLiteral("Colors:Button/BackgroundNormal"), kdeDirs, kdeVersion, kdeSettings))) {
+    if (!kdeColor(pal, QPalette::Button, readKdeSetting(KdeSetting::ButtonBackground, kdeDirs, kdeVersion, kdeSettings))) {
         // kcolorscheme.cpp: SetDefaultColors
         const QColor defaultWindowBackground(214, 210, 208);
         const QColor defaultButtonBackground(223, 220, 217);
@@ -790,18 +985,18 @@ void QKdeThemePrivate::readKdeSystemPalette(const QStringList &kdeDirs, int kdeV
         return;
     }
 
-    kdeColor(pal, QPalette::Window, readKdeSetting(QStringLiteral("Colors:Window/BackgroundNormal"), kdeDirs, kdeVersion, kdeSettings));
-    kdeColor(pal, QPalette::Text, readKdeSetting(QStringLiteral("Colors:View/ForegroundNormal"), kdeDirs, kdeVersion, kdeSettings));
-    kdeColor(pal, QPalette::WindowText, readKdeSetting(QStringLiteral("Colors:Window/ForegroundNormal"), kdeDirs, kdeVersion, kdeSettings));
-    kdeColor(pal, QPalette::Base, readKdeSetting(QStringLiteral("Colors:View/BackgroundNormal"), kdeDirs, kdeVersion, kdeSettings));
-    kdeColor(pal, QPalette::Highlight, readKdeSetting(QStringLiteral("Colors:Selection/BackgroundNormal"), kdeDirs, kdeVersion, kdeSettings));
-    kdeColor(pal, QPalette::HighlightedText, readKdeSetting(QStringLiteral("Colors:Selection/ForegroundNormal"), kdeDirs, kdeVersion, kdeSettings));
-    kdeColor(pal, QPalette::AlternateBase, readKdeSetting(QStringLiteral("Colors:View/BackgroundAlternate"), kdeDirs, kdeVersion, kdeSettings));
-    kdeColor(pal, QPalette::ButtonText, readKdeSetting(QStringLiteral("Colors:Button/ForegroundNormal"), kdeDirs, kdeVersion, kdeSettings));
-    kdeColor(pal, QPalette::Link, readKdeSetting(QStringLiteral("Colors:View/ForegroundLink"), kdeDirs, kdeVersion, kdeSettings));
-    kdeColor(pal, QPalette::LinkVisited, readKdeSetting(QStringLiteral("Colors:View/ForegroundVisited"), kdeDirs, kdeVersion, kdeSettings));
-    kdeColor(pal, QPalette::ToolTipBase, readKdeSetting(QStringLiteral("Colors:Tooltip/BackgroundNormal"), kdeDirs, kdeVersion, kdeSettings));
-    kdeColor(pal, QPalette::ToolTipText, readKdeSetting(QStringLiteral("Colors:Tooltip/ForegroundNormal"), kdeDirs, kdeVersion, kdeSettings));
+    kdeColor(pal, QPalette::Window, readKdeSetting(KdeSetting::WindowBackground, kdeDirs, kdeVersion, kdeSettings));
+    kdeColor(pal, QPalette::Text, readKdeSetting(KdeSetting::ViewForeground, kdeDirs, kdeVersion, kdeSettings));
+    kdeColor(pal, QPalette::WindowText, readKdeSetting(KdeSetting::WindowForeground, kdeDirs, kdeVersion, kdeSettings));
+    kdeColor(pal, QPalette::Base, readKdeSetting(KdeSetting::ViewBackground, kdeDirs, kdeVersion, kdeSettings));
+    kdeColor(pal, QPalette::Highlight, readKdeSetting(KdeSetting::SelectionBackground, kdeDirs, kdeVersion, kdeSettings));
+    kdeColor(pal, QPalette::HighlightedText, readKdeSetting(KdeSetting::SelectionForeground, kdeDirs, kdeVersion, kdeSettings));
+    kdeColor(pal, QPalette::AlternateBase, readKdeSetting(KdeSetting::ViewBackgroundAlternate, kdeDirs, kdeVersion, kdeSettings));
+    kdeColor(pal, QPalette::ButtonText, readKdeSetting(KdeSetting::ButtonForeground, kdeDirs, kdeVersion, kdeSettings));
+    kdeColor(pal, QPalette::Link, readKdeSetting(KdeSetting::ViewForegroundLink, kdeDirs, kdeVersion, kdeSettings));
+    kdeColor(pal, QPalette::LinkVisited, readKdeSetting(KdeSetting::ViewForegroundVisited, kdeDirs, kdeVersion, kdeSettings));
+    kdeColor(pal, QPalette::ToolTipBase, readKdeSetting(KdeSetting::TooltipBackground, kdeDirs, kdeVersion, kdeSettings));
+    kdeColor(pal, QPalette::ToolTipText, readKdeSetting(KdeSetting::TooltipForeground, kdeDirs, kdeVersion, kdeSettings));
 
     // The above code sets _all_ color roles to "normal" colors. In KDE, the disabled
     // color roles are calculated by applying various effects described in kdeglobals.
@@ -934,6 +1129,8 @@ QVariant QKdeTheme::themeHint(QPlatformTheme::ThemeHint hint) const
         return QVariant(mouseCursorTheme());
     case QPlatformTheme::MouseCursorSize:
         return QVariant(mouseCursorSize());
+    case QPlatformTheme::PreferFileIconFromTheme:
+        return true;
     default:
         break;
     }
@@ -957,13 +1154,13 @@ Qt::ColorScheme QKdeTheme::colorScheme() const
 
 /*!
    \internal
-   \brief QKdeTheme::setColorScheme - guess and set appearance for unix themes.
-   KDE themes do not have an appearance property.
-   The key words "dark" or "light" should be part of the theme name.
+   \brief QKdeTheme::updateColorScheme - guess and set a color scheme for unix themes.
+   KDE themes do not have a color scheme property.
+   The key words "dark" or "light" are usually part of the theme name.
    This is, however, not a mandatory convention.
 
-   If \param themeName contains a key word, the respective appearance is set.
-   If it doesn't, the appearance is heuristically determined by comparing text and base color
+   If \param themeName contains a valid key word, the respective color scheme is set.
+   If it doesn't, the color scheme is heuristically determined by comparing text and base color
    of the system palette.
  */
 void QKdeThemePrivate::updateColorScheme(const QString &themeName)
@@ -991,7 +1188,6 @@ void QKdeThemePrivate::updateColorScheme(const QString &themeName)
     m_colorScheme = Qt::ColorScheme::Unknown;
 }
 
-
 const QPalette *QKdeTheme::palette(Palette type) const
 {
     Q_D(const QKdeTheme);
@@ -1071,7 +1267,7 @@ QPlatformMenuBar *QKdeTheme::createPlatformMenuBar() const
 #if !defined(QT_NO_DBUS) && !defined(QT_NO_SYSTEMTRAYICON)
 QPlatformSystemTrayIcon *QKdeTheme::createPlatformSystemTrayIcon() const
 {
-    if (isDBusTrayAvailable())
+    if (shouldUseDBusTray())
         return new QDBusTrayIcon();
     return nullptr;
 }
@@ -1153,7 +1349,7 @@ bool QGnomeThemePrivate::initDbus()
             updateColorScheme(value);
     };
 
-    return QObject::connect(dbus.get(), &QGenericUnixThemeDBusListener::settingChanged, wrapper);
+    return QObject::connect(dbus.get(), &QGenericUnixThemeDBusListener::settingChanged, dbus.get(), wrapper);
 }
 
 void QGnomeThemePrivate::updateColorScheme(const QString &themeName)
@@ -1212,6 +1408,8 @@ QVariant QGnomeTheme::themeHint(QPlatformTheme::ThemeHint hint) const
         return QVariant(mouseCursorTheme());
     case QPlatformTheme::MouseCursorSize:
         return QVariant(mouseCursorSize());
+    case QPlatformTheme::PreferFileIconFromTheme:
+        return true;
     default:
         break;
     }
@@ -1266,7 +1464,7 @@ Qt::ColorScheme QGnomeTheme::colorScheme() const
 #if !defined(QT_NO_DBUS) && !defined(QT_NO_SYSTEMTRAYICON)
 QPlatformSystemTrayIcon *QGnomeTheme::createPlatformSystemTrayIcon() const
 {
-    if (isDBusTrayAvailable())
+    if (shouldUseDBusTray())
         return new QDBusTrayIcon();
     return nullptr;
 }
@@ -1317,6 +1515,7 @@ QStringList QGenericUnixTheme::themeNames()
         QList<QByteArray> gtkBasedEnvironments;
         gtkBasedEnvironments << "GNOME"
                              << "X-CINNAMON"
+                             << "PANTHEON"
                              << "UNITY"
                              << "MATE"
                              << "XFCE"
