@@ -31,8 +31,13 @@ extern void qt_format_text(const QFont& font, const QRectF &_r,
     \ingroup shared
 
     QFontMetrics functions calculate the size of characters and
-    strings for a given font. There are three ways you can create a
-    QFontMetrics object:
+    strings for a given font. The class is an integer-based version
+    of QFontMetricsF and will round all numbers to the nearest
+    integer. This means its results will be inaccurate for any font
+    with fractional metrics. In most cases QFontMetricsF should be
+    used instead.
+
+    There are three ways you can create a QFontMetrics object:
 
     \list 1
     \li Calling the QFontMetrics constructor with a QFont creates a
@@ -75,15 +80,23 @@ extern void qt_format_text(const QFont& font, const QRectF &_r,
     inFont(). You can also treat the character as a string, and use
     the string functions on it.
 
-    The string functions include horizontalAdvance(), to return the width of a
-    string in pixels (or points, for a printer), boundingRect(), to
-    return a rectangle large enough to contain the rendered string,
+    The string functions include horizontalAdvance(), to return the advance
+    width of a string in pixels (or points, for a printer), boundingRect(),
+    to return a rectangle large enough to contain the rendered string,
     and size(), to return the size of that rectangle.
 
+    \note The advance width can be different from the width of the actual
+    rendered text. It refers to the distance from the origin of the string to
+    where you would append additional characters. As text may have overhang
+    (in the case of an italic font for instance) or padding between
+    characters, the advance width can be either smaller or larger than the
+    actual rendering of the text. This is called the right bearing of the
+    text.
+
     Example:
     \snippet code/src_gui_text_qfontmetrics.cpp 0
 
-    \sa QFont, QFontInfo, QFontDatabase, {Character Map Example}
+    \sa QFont, QFontInfo, QFontDatabase
 */
 
 /*!
@@ -187,9 +200,7 @@ QFontMetrics &QFontMetrics::operator=(const QFontMetrics &fm)
 /*!
     \fn void QFontMetrics::swap(QFontMetrics &other)
     \since 5.0
-
-    Swaps this font metrics instance with \a other. This function is
-    very fast and never fails.
+    \memberswap{font metrics instance}
 */
 
 /*!
@@ -473,10 +484,13 @@ int QFontMetrics::rightBearing(QChar ch) const
     return qRound(rb);
 }
 
+static constexpr QLatin1Char s_variableLengthStringSeparator('\x9c');
+
 /*!
     Returns the horizontal advance in pixels of the first \a len characters of \a
     text. If \a len is negative (the default), the entire string is
-    used.
+    used. The entire length of \a text is analysed even if \a len is substantially
+    shorter.
 
     This is the distance appropriate for drawing a subsequent character
     after \a text.
@@ -487,16 +501,18 @@ int QFontMetrics::rightBearing(QChar ch) const
 */
 int QFontMetrics::horizontalAdvance(const QString &text, int len) const
 {
-    int pos = text.indexOf(QLatin1Char('\x9c'));
+    int pos = (len >= 0)
+            ? QStringView(text).left(len).indexOf(s_variableLengthStringSeparator)
+            : text.indexOf(s_variableLengthStringSeparator);
     if (pos != -1) {
-        len = (len < 0) ? pos : qMin(pos, len);
+        len = pos;
     } else if (len < 0) {
         len = text.size();
     }
     if (len == 0)
         return 0;
 
-    QStackTextEngine layout(text, QFont(d.data()));
+    Q_DECL_UNINITIALIZED QStackTextEngine layout(text, QFont(d.data()));
     return qRound(layout.width(0, len));
 }
 
@@ -512,17 +528,17 @@ int QFontMetrics::horizontalAdvance(const QString &text, int len) const
 */
 int QFontMetrics::horizontalAdvance(const QString &text, const QTextOption &option) const
 {
-    int pos = text.indexOf(QLatin1Char('\x9c'));
+    int pos = text.indexOf(s_variableLengthStringSeparator);
     int len = -1;
     if (pos != -1) {
-        len = (len < 0) ? pos : qMin(pos, len);
-    } else if (len < 0) {
+        len = pos;
+    } else {
         len = text.size();
     }
     if (len == 0)
         return 0;
 
-    QStackTextEngine layout(text, QFont(d.data()));
+    Q_DECL_UNINITIALIZED QStackTextEngine layout(text, QFont(d.data()));
     layout.option = option;
     return qRound(layout.width(0, len));
 }
@@ -607,7 +623,7 @@ QRect QFontMetrics::boundingRect(const QString &text) const
     if (text.size() == 0)
         return QRect();
 
-    QStackTextEngine layout(text, QFont(d.data()));
+    Q_DECL_UNINITIALIZED QStackTextEngine layout(text, QFont(d.data()));
     layout.itemize();
     glyph_metrics_t gm = layout.boundingBox(0, text.size());
     return QRect(qRound(gm.x), qRound(gm.y), qRound(gm.width), qRound(gm.height));
@@ -642,7 +658,7 @@ QRect QFontMetrics::boundingRect(const QString &text, const QTextOption &option)
     if (text.size() == 0)
         return QRect();
 
-    QStackTextEngine layout(text, QFont(d.data()));
+    Q_DECL_UNINITIALIZED QStackTextEngine layout(text, QFont(d.data()));
     layout.option = option;
     layout.itemize();
     glyph_metrics_t gm = layout.boundingBox(0, text.size());
@@ -810,7 +826,7 @@ QRect QFontMetrics::tightBoundingRect(const QString &text) const
     if (text.size() == 0)
         return QRect();
 
-    QStackTextEngine layout(text, QFont(d.data()));
+    Q_DECL_UNINITIALIZED QStackTextEngine layout(text, QFont(d.data()));
     layout.itemize();
     glyph_metrics_t gm = layout.tightBoundingBox(0, text.size());
     return QRect(qRound(gm.x), qRound(gm.y), qRound(gm.width), qRound(gm.height));
@@ -842,7 +858,7 @@ QRect QFontMetrics::tightBoundingRect(const QString &text, const QTextOption &op
     if (text.size() == 0)
         return QRect();
 
-    QStackTextEngine layout(text, QFont(d.data()));
+    Q_DECL_UNINITIALIZED QStackTextEngine layout(text, QFont(d.data()));
     layout.option = option;
     layout.itemize();
     glyph_metrics_t gm = layout.tightBoundingBox(0, text.size());
@@ -876,17 +892,17 @@ QString QFontMetrics::elidedText(const QString &text, Qt::TextElideMode mode, in
     QString _text = text;
     if (!(flags & Qt::TextLongestVariant)) {
         int posA = 0;
-        int posB = _text.indexOf(QLatin1Char('\x9c'));
+        int posB = _text.indexOf(s_variableLengthStringSeparator);
         while (posB >= 0) {
             QString portion = _text.mid(posA, posB - posA);
             if (size(flags, portion).width() <= width)
                 return portion;
             posA = posB + 1;
-            posB = _text.indexOf(QLatin1Char('\x9c'), posA);
+            posB = _text.indexOf(s_variableLengthStringSeparator, posA);
         }
         _text = _text.mid(posA);
     }
-    QStackTextEngine engine(_text, QFont(d.data()));
+    Q_DECL_UNINITIALIZED QStackTextEngine engine(_text, QFont(d.data()));
     return engine.elidedText(mode, width, flags);
 }
 
@@ -1025,9 +1041,7 @@ QFontMetricsF &QFontMetricsF::operator=(const QFontMetrics &other)
 /*!
     \fn void QFontMetricsF::swap(QFontMetricsF &other)
     \since 5.0
-
-    Swaps this font metrics instance with \a other. This function is
-    very fast and never fails.
+    \memberswap{font metrics instance}
 */
 
 
@@ -1387,7 +1401,8 @@ qreal QFontMetricsF::rightBearing(QChar ch) const
 /*!
     Returns the horizontal advance in pixels of the first \a length characters of \a
     text. If \a length is negative (the default), the entire string is
-    used.
+    used. The entire length of \a text is analysed even if \a length is substantially
+    shorter.
 
     The advance is the distance appropriate for drawing a subsequent
     character after \a text.
@@ -1398,16 +1413,18 @@ qreal QFontMetricsF::rightBearing(QChar ch) const
 */
 qreal QFontMetricsF::horizontalAdvance(const QString &text, int length) const
 {
-    int pos = text.indexOf(QLatin1Char('\x9c'));
+    int pos = (length >= 0)
+            ? QStringView(text).left(length).indexOf(s_variableLengthStringSeparator)
+            : text.indexOf(s_variableLengthStringSeparator);
     if (pos != -1)
-        length = (length < 0) ? pos : qMin(pos, length);
+        length = pos;
     else if (length < 0)
         length = text.size();
 
     if (length == 0)
         return 0;
 
-    QStackTextEngine layout(text, QFont(d.data()));
+    Q_DECL_UNINITIALIZED QStackTextEngine layout(text, QFont(d.data()));
     layout.itemize();
     return layout.width(0, length).toReal();
 }
@@ -1424,17 +1441,17 @@ qreal QFontMetricsF::horizontalAdvance(const QString &text, int length) const
 */
 qreal QFontMetricsF::horizontalAdvance(const QString &text, const QTextOption &option) const
 {
-    int pos = text.indexOf(QLatin1Char('\x9c'));
+    int pos = text.indexOf(s_variableLengthStringSeparator);
     int length = -1;
     if (pos != -1)
-        length = (length < 0) ? pos : qMin(pos, length);
-    else if (length < 0)
+        length = pos;
+    else
         length = text.size();
 
     if (length == 0)
         return 0;
 
-    QStackTextEngine layout(text, QFont(d.data()));
+    Q_DECL_UNINITIALIZED QStackTextEngine layout(text, QFont(d.data()));
     layout.option = option;
     layout.itemize();
     return layout.width(0, length).toReal();
@@ -1520,7 +1537,7 @@ QRectF QFontMetricsF::boundingRect(const QString &text) const
     if (len == 0)
         return QRectF();
 
-    QStackTextEngine layout(text, QFont(d.data()));
+    Q_DECL_UNINITIALIZED QStackTextEngine layout(text, QFont(d.data()));
     layout.itemize();
     glyph_metrics_t gm = layout.boundingBox(0, len);
     return QRectF(gm.x.toReal(), gm.y.toReal(),
@@ -1554,7 +1571,7 @@ QRectF QFontMetricsF::boundingRect(const QString &text, const QTextOption &optio
     if (text.size() == 0)
         return QRectF();
 
-    QStackTextEngine layout(text, QFont(d.data()));
+    Q_DECL_UNINITIALIZED QStackTextEngine layout(text, QFont(d.data()));
     layout.option = option;
     layout.itemize();
     glyph_metrics_t gm = layout.boundingBox(0, text.size());
@@ -1728,7 +1745,7 @@ QRectF QFontMetricsF::tightBoundingRect(const QString &text) const
     if (text.size() == 0)
         return QRectF();
 
-    QStackTextEngine layout(text, QFont(d.data()));
+    Q_DECL_UNINITIALIZED QStackTextEngine layout(text, QFont(d.data()));
     layout.itemize();
     glyph_metrics_t gm = layout.tightBoundingBox(0, text.size());
     return QRectF(gm.x.toReal(), gm.y.toReal(), gm.width.toReal(), gm.height.toReal());
@@ -1760,7 +1777,7 @@ QRectF QFontMetricsF::tightBoundingRect(const QString &text, const QTextOption &
     if (text.size() == 0)
         return QRectF();
 
-    QStackTextEngine layout(text, QFont(d.data()));
+    Q_DECL_UNINITIALIZED QStackTextEngine layout(text, QFont(d.data()));
     layout.option = option;
     layout.itemize();
     glyph_metrics_t gm = layout.tightBoundingBox(0, text.size());
@@ -1793,17 +1810,17 @@ QString QFontMetricsF::elidedText(const QString &text, Qt::TextElideMode mode, q
     QString _text = text;
     if (!(flags & Qt::TextLongestVariant)) {
         int posA = 0;
-        int posB = _text.indexOf(QLatin1Char('\x9c'));
+        int posB = _text.indexOf(s_variableLengthStringSeparator);
         while (posB >= 0) {
             QString portion = _text.mid(posA, posB - posA);
             if (size(flags, portion).width() <= width)
                 return portion;
             posA = posB + 1;
-            posB = _text.indexOf(QLatin1Char('\x9c'), posA);
+            posB = _text.indexOf(s_variableLengthStringSeparator, posA);
         }
         _text = _text.mid(posA);
     }
-    QStackTextEngine engine(_text, QFont(d.data()));
+    Q_DECL_UNINITIALIZED QStackTextEngine engine(_text, QFont(d.data()));
     return engine.elidedText(mode, QFixed::fromReal(width), flags);
 }
 
