@@ -12,34 +12,81 @@
 
 QT_BEGIN_NAMESPACE
 
+Q_DECLARE_JNI_CLASS(Locale, "java/util/Locale")
+Q_DECLARE_JNI_CLASS(Resources, "android/content/res/Resources")
+Q_DECLARE_JNI_CLASS(Configuration, "android/content/res/Configuration")
+Q_DECLARE_JNI_CLASS(LocaleList, "android/os/LocaleList")
+Q_DECLARE_JNI_CLASS(DateFormat, "android/text/format/DateFormat")
+
+using namespace QtJniTypes;
+
 QAndroidSystemLocale::QAndroidSystemLocale() : m_locale(QLocale::C)
 {
 }
 
 void QAndroidSystemLocale::getLocaleFromJava() const
 {
+    const Locale javaLocaleObject = []{
+        const QJniObject javaContext = QtAndroidPrivate::context();
+        if (javaContext.isValid()) {
+            const QJniObject resources = javaContext.callMethod<Resources>("getResources");
+            const QJniObject configuration = resources.callMethod<Configuration>("getConfiguration");
+            return configuration.getField<Locale>("locale");
+        } else {
+            return Locale::callStaticMethod<Locale>("getDefault");
+        }
+    }();
+
+    const QString languageCode = javaLocaleObject.callMethod<QString>("getLanguage");
+    const QString countryCode = javaLocaleObject.callMethod<QString>("getCountry");
+
+    const bool is24HourFormat = DateFormat::callStaticMethod<bool>("is24HourFormat",
+                                    QNativeInterface::QAndroidApplication::context());
+
     QWriteLocker locker(&m_lock);
-
-    QJniObject javaLocaleObject;
-    QJniObject javaActivity(QtAndroid::activity());
-    if (!javaActivity.isValid())
-        javaActivity = QtAndroid::service();
-    if (javaActivity.isValid()) {
-        QJniObject resources = javaActivity.callObjectMethod("getResources", "()Landroid/content/res/Resources;");
-        QJniObject configuration = resources.callObjectMethod("getConfiguration", "()Landroid/content/res/Configuration;");
-
-        javaLocaleObject = configuration.getObjectField("locale", "Ljava/util/Locale;");
-    } else {
-        javaLocaleObject = QJniObject::callStaticObjectMethod("java/util/Locale", "getDefault", "()Ljava/util/Locale;");
-    }
-
-    QString languageCode = javaLocaleObject.callObjectMethod("getLanguage", "()Ljava/lang/String;").toString();
-    QString countryCode = javaLocaleObject.callObjectMethod("getCountry", "()Ljava/lang/String;").toString();
-
     m_locale = QLocale(languageCode + u'_' + countryCode);
+    m_24hFormat = is24HourFormat;
 }
 
-QVariant QAndroidSystemLocale::query(QueryType type, QVariant in) const
+QString QAndroidSystemLocale::convertTo24hFormat(const QString &format) const
+{
+    if (!m_24hFormat)
+        return format;
+
+    QString format24(format);
+    bool inQuoted = false;
+    for (qsizetype i = 0; i < format24.size(); ++i) {
+        if (format24[i] == QLatin1Char('\'')) {
+            inQuoted = !inQuoted;
+            continue;
+        }
+        if (inQuoted)
+            continue;
+
+        // remove AM/PM markerg from format string
+        const auto c = format24[i].toUpper();
+        if (c == QLatin1Char('A') || c == QLatin1Char('P'))
+            format24.remove(i--, 1);
+    }
+
+    return format24.trimmed();
+}
+
+QString QAndroidSystemLocale::timeToString(const QTime &time, QLocale::FormatType type) const
+{
+    if (m_24hFormat)
+        return m_locale.toString(time, convertTo24hFormat(m_locale.timeFormat(type)));
+    return m_locale.toString(time, type);
+}
+
+QString QAndroidSystemLocale::dateTimeToString(const QDateTime &dt, QLocale::FormatType type) const
+{
+    if (m_24hFormat)
+        return m_locale.toString(dt, convertTo24hFormat(m_locale.dateTimeFormat(type)));
+    return m_locale.toString(dt, type);
+}
+
+QVariant QAndroidSystemLocale::query(QueryType type, QVariant &&in) const
 {
     if (type == LocaleChanged) {
         getLocaleFromJava();
@@ -62,9 +109,9 @@ QVariant QAndroidSystemLocale::query(QueryType type, QVariant in) const
     case DateFormatShort:
         return m_locale.dateFormat(QLocale::ShortFormat);
     case TimeFormatLong:
-        return m_locale.timeFormat(QLocale::LongFormat);
+        return convertTo24hFormat(m_locale.timeFormat(QLocale::LongFormat));
     case TimeFormatShort:
-        return m_locale.timeFormat(QLocale::ShortFormat);
+        return convertTo24hFormat(m_locale.timeFormat(QLocale::ShortFormat));
     case DayNameLong:
         return m_locale.dayName(in.toInt(), QLocale::LongFormat);
     case DayNameShort:
@@ -94,17 +141,17 @@ QVariant QAndroidSystemLocale::query(QueryType type, QVariant in) const
     case DateToStringShort:
         return m_locale.toString(in.toDate(), QLocale::ShortFormat);
     case TimeToStringLong:
-        return m_locale.toString(in.toTime(), QLocale::LongFormat);
+        return timeToString(in.toTime(), QLocale::LongFormat);
     case TimeToStringShort:
-        return m_locale.toString(in.toTime(), QLocale::ShortFormat);
+        return timeToString(in.toTime(), QLocale::ShortFormat);
     case DateTimeFormatLong:
-        return m_locale.dateTimeFormat(QLocale::LongFormat);
+        return convertTo24hFormat(m_locale.dateTimeFormat(QLocale::LongFormat));
     case DateTimeFormatShort:
-        return m_locale.dateTimeFormat(QLocale::ShortFormat);
+        return convertTo24hFormat(m_locale.dateTimeFormat(QLocale::ShortFormat));
     case DateTimeToStringLong:
-        return m_locale.toString(in.toDateTime(), QLocale::LongFormat);
+        return dateTimeToString(in.toDateTime(), QLocale::LongFormat);
     case DateTimeToStringShort:
-        return m_locale.toString(in.toDateTime(), QLocale::ShortFormat);
+        return dateTimeToString(in.toDateTime(), QLocale::ShortFormat);
     case PositiveSign:
         return m_locale.positiveSign();
     case AMText:
@@ -142,12 +189,9 @@ QVariant QAndroidSystemLocale::query(QueryType type, QVariant in) const
         Q_ASSERT_X(false, Q_FUNC_INFO, "This can't happen.");
     case UILanguages: {
         if (QtAndroidPrivate::androidSdkVersion() >= 24) {
-            QJniObject localeListObject =
-                QJniObject::callStaticObjectMethod("android/os/LocaleList", "getDefault",
-                                                   "()Landroid/os/LocaleList;");
+            LocaleList localeListObject = LocaleList::callStaticMethod<LocaleList>("getDefault");
             if (localeListObject.isValid()) {
-                QString lang = localeListObject.callObjectMethod("toLanguageTags",
-                                                                 "()Ljava/lang/String;").toString();
+                QString lang = localeListObject.callMethod<QString>("toLanguageTags");
                 // Some devices return with it enclosed in []'s so check if both exists before
                 // removing to ensure it is formatted correctly
                 if (lang.startsWith(QChar('[')) && lang.endsWith(QChar(']')))
