@@ -15,7 +15,9 @@
 #include <IOKit/graphics/IOGraphicsLib.h>
 
 #include <QtGui/private/qwindow_p.h>
+#include <QtGui/private/qhighdpiscaling_p.h>
 
+#include <QtCore/private/qcore_mac_p.h>
 #include <QtCore/private/qeventdispatcher_cf_p.h>
 
 QT_BEGIN_NAMESPACE
@@ -195,38 +197,6 @@ QCocoaScreen::~QCocoaScreen()
          dispatch_release(m_displayLinkSource);
 }
 
-static QString displayName(CGDirectDisplayID displayID)
-{
-    QIOType<io_iterator_t> iterator;
-    if (IOServiceGetMatchingServices(kIOMasterPortDefault,
-        IOServiceMatching("IODisplayConnect"), &iterator))
-        return QString();
-
-    QIOType<io_service_t> display;
-    while ((display = IOIteratorNext(iterator)) != 0)
-    {
-        NSDictionary *info = [(__bridge NSDictionary*)IODisplayCreateInfoDictionary(
-            display, kIODisplayOnlyPreferredName) autorelease];
-
-        if ([[info objectForKey:@kDisplayVendorID] unsignedIntValue] != CGDisplayVendorNumber(displayID))
-            continue;
-
-        if ([[info objectForKey:@kDisplayProductID] unsignedIntValue] != CGDisplayModelNumber(displayID))
-            continue;
-
-        if ([[info objectForKey:@kDisplaySerialNumber] unsignedIntValue] != CGDisplaySerialNumber(displayID))
-            continue;
-
-        NSDictionary *localizedNames = [info objectForKey:@kDisplayProductName];
-        if (![localizedNames count])
-            break; // Correct screen, but no name in dictionary
-
-        return QString::fromNSString([localizedNames objectForKey:[[localizedNames allKeys] objectAtIndex:0]]);
-    }
-
-    return QString();
-}
-
 void QCocoaScreen::update(CGDirectDisplayID displayId)
 {
     if (displayId != m_displayId) {
@@ -246,6 +216,7 @@ void QCocoaScreen::update(CGDirectDisplayID displayId)
     const QRect previousGeometry = m_geometry;
     const QRect previousAvailableGeometry = m_availableGeometry;
     const qreal previousRefreshRate = m_refreshRate;
+    const double previousRotation = m_rotation;
 
     // The reference screen for the geometry is always the primary screen
     QRectF primaryScreenGeometry = QRectF::fromCGRect(CGDisplayBounds(CGMainDisplayID()));
@@ -270,14 +241,14 @@ void QCocoaScreen::update(CGDirectDisplayID displayId)
     QCFType<CGDisplayModeRef> displayMode = CGDisplayCopyDisplayMode(m_displayId);
     float refresh = CGDisplayModeGetRefreshRate(displayMode);
     m_refreshRate = refresh > 0 ? refresh : 60.0;
-
-    if (@available(macOS 10.15, *))
-        m_name = QString::fromNSString(nsScreen.localizedName);
-    else
-        m_name = displayName(m_displayId);
+    m_rotation = CGDisplayRotation(displayId);
+    m_name = QString::fromNSString(nsScreen.localizedName);
 
     const bool didChangeGeometry = m_geometry != previousGeometry || m_availableGeometry != previousAvailableGeometry;
 
+    if (m_rotation != previousRotation)
+        QWindowSystemInterface::handleScreenOrientationChange(screen(), orientation());
+
     if (didChangeGeometry)
         QWindowSystemInterface::handleScreenGeometryChange(screen(), geometry(), availableGeometry());
     if (m_refreshRate != previousRefreshRate)
@@ -288,24 +259,33 @@ void QCocoaScreen::update(CGDirectDisplayID displayId)
 
 Q_LOGGING_CATEGORY(lcQpaScreenUpdates, "qt.qpa.screen.updates", QtCriticalMsg);
 
-void QCocoaScreen::requestUpdate()
+bool QCocoaScreen::requestUpdate()
 {
     Q_ASSERT(m_displayId);
 
     if (!isOnline()) {
         qCDebug(lcQpaScreenUpdates) << this << "is not online. Ignoring update request";
-        return;
+        return false;
     }
 
     if (!m_displayLink) {
-        CVDisplayLinkCreateWithCGDisplay(m_displayId, &m_displayLink);
+        qCDebug(lcQpaScreenUpdates) << "Creating display link for" << this;
+        if (CVDisplayLinkCreateWithCGDisplay(m_displayId, &m_displayLink) != kCVReturnSuccess) {
+            qCWarning(lcQpaScreenUpdates) << "Failed to create display link for" << this;
+            return false;
+        }
+        if (auto displayId = CVDisplayLinkGetCurrentCGDisplay(m_displayLink); displayId != m_displayId) {
+            qCWarning(lcQpaScreenUpdates) << "Unexpected display" << displayId << "for display link";
+            CVDisplayLinkRelease(m_displayLink);
+            m_displayLink = nullptr;
+            return false;
+        }
         CVDisplayLinkSetOutputCallback(m_displayLink, [](CVDisplayLinkRef, const CVTimeStamp*,
             const CVTimeStamp*, CVOptionFlags, CVOptionFlags*, void* displayLinkContext) -> int {
                 // FIXME: It would be nice if update requests would include timing info
                 static_cast<QCocoaScreen*>(displayLinkContext)->deliverUpdateRequests();
                 return kCVReturnSuccess;
         }, this);
-        qCDebug(lcQpaScreenUpdates) << "Display link created for" << this;
 
         // During live window resizing -[NSWindow _resizeWithEvent:] will spin a local event loop
         // in event-tracking mode, dequeuing only the mouse drag events needed to update the window's
@@ -360,6 +340,8 @@ void QCocoaScreen::requestUpdate()
         qCDebug(lcQpaScreenUpdates) << "Starting display link for" << this;
         CVDisplayLinkStart(m_displayLink);
     }
+
+    return true;
 }
 
 // Helper to allow building up debug output in multiple steps
@@ -439,22 +421,26 @@ void QCocoaScreen::deliverUpdateRequests()
         auto windows = QGuiApplication::allWindows();
         for (int i = 0; i < windows.size(); ++i) {
             QWindow *window = windows.at(i);
-            auto *platformWindow = static_cast<QCocoaWindow*>(window->handle());
+            if (window->screen() != screen())
+                continue;
+
+            QPointer<QCocoaWindow> platformWindow = static_cast<QCocoaWindow*>(window->handle());
             if (!platformWindow)
                 continue;
 
             if (!platformWindow->hasPendingUpdateRequest())
                 continue;
 
-            if (window->screen() != screen())
-                continue;
-
             // Skip windows that are not doing update requests via display link
             if (!platformWindow->updatesWithDisplayLink())
                 continue;
 
             platformWindow->deliverUpdateRequest();
 
+            // platform window can be destroyed in deliverUpdateRequest()
+            if (!platformWindow)
+                continue;
+
             // Another update request was triggered, keep the display link running
             if (platformWindow->hasPendingUpdateRequest())
                 pauseUpdates = false;
@@ -490,41 +476,56 @@ QPlatformScreen::SubpixelAntialiasingType QCocoaScreen::subpixelAntialiasingType
     return type;
 }
 
+Qt::ScreenOrientation QCocoaScreen::orientation() const
+{
+    if (m_rotation == 0)
+        return Qt::LandscapeOrientation;
+    if (m_rotation == 90)
+        return Qt::PortraitOrientation;
+    if (m_rotation == 180)
+        return Qt::InvertedLandscapeOrientation;
+    if (m_rotation == 270)
+        return Qt::InvertedPortraitOrientation;
+    return QPlatformScreen::orientation();
+}
+
 QWindow *QCocoaScreen::topLevelAt(const QPoint &point) const
 {
-    NSPoint screenPoint = mapToNative(point);
+    __block QWindow *window = nullptr;
+    [NSApp enumerateWindowsWithOptions:NSWindowListOrderedFrontToBack
+        usingBlock:^(NSWindow *nsWindow, BOOL *stop) {
+            if (!nsWindow)
+                return;
 
-    // Search (hit test) for the top-level window. [NSWidow windowNumberAtPoint:
-    // belowWindowWithWindowNumber] may return windows that are not interesting
-    // to Qt. The search iterates until a suitable window or no window is found.
-    NSInteger topWindowNumber = 0;
-    QWindow *window = nullptr;
-    do {
-        // Get the top-most window, below any previously rejected window.
-        topWindowNumber = [NSWindow windowNumberAtPoint:screenPoint
-                                    belowWindowWithWindowNumber:topWindowNumber];
+            // Continue the search if the window does not belong to Qt
+            if (![nsWindow conformsToProtocol:@protocol(QNSWindowProtocol)])
+                return;
 
-        // Continue the search if the window does not belong to this process.
-        NSWindow *nsWindow = [NSApp windowWithWindowNumber:topWindowNumber];
-        if (!nsWindow)
-            continue;
+            QCocoaWindow *cocoaWindow = qnsview_cast(nsWindow.contentView).platformWindow;
+            if (!cocoaWindow)
+                return;
 
-        // Continue the search if the window does not belong to Qt.
-        if (![nsWindow conformsToProtocol:@protocol(QNSWindowProtocol)])
-            continue;
+            QWindow *w = cocoaWindow->window();
+            if (!w->isVisible())
+                return;
 
-        QCocoaWindow *cocoaWindow = qnsview_cast(nsWindow.contentView).platformWindow;
-        if (!cocoaWindow)
-            continue;
-        window = cocoaWindow->window();
+            auto nativeGeometry = QHighDpi::toNativePixels(w->geometry(), w);
+            if (!nativeGeometry.contains(point))
+                return;
 
-        // Continue the search if the window is not a top-level window.
-        if (!window->isTopLevel())
-             continue;
+            QRegion mask = QHighDpi::toNativeLocalPosition(w->mask(), w);
+            if (!mask.isEmpty() && !mask.contains(point - nativeGeometry.topLeft()))
+                return;
 
-        // Stop searching. The current window is the correct window.
-        break;
-    } while (topWindowNumber > 0);
+            window = w;
+
+            // Continue the search if the window is not a top-level window
+            if (!window->isTopLevel())
+                return;
+
+            *stop = true;
+        }
+    ];
 
     return window;
 }
