@@ -1,6 +1,9 @@
 // Copyright (C) 2018 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
+#ifndef QWASMOPENGLCONTEXT_H
+#define QWASMOPENGLCONTEXT_H
+
 #include <qpa/qplatformopenglcontext.h>
 
 #include <emscripten.h>
@@ -8,11 +11,13 @@
 
 QT_BEGIN_NAMESPACE
 
+class QOpenGLContext;
 class QPlatformScreen;
+class QPlatformSurface;
 class QWasmOpenGLContext : public QPlatformOpenGLContext
 {
 public:
-    QWasmOpenGLContext(const QSurfaceFormat &format);
+    explicit QWasmOpenGLContext(QOpenGLContext *context);
     ~QWasmOpenGLContext();
 
     QSurfaceFormat format() const override;
@@ -25,14 +30,25 @@ public:
     QFunctionPointer getProcAddress(const char *procName) override;
 
 private:
-    static bool isOpenGLVersionSupported(QSurfaceFormat format);
-    bool maybeCreateEmscriptenContext(QPlatformSurface *surface);
-    static EMSCRIPTEN_WEBGL_CONTEXT_HANDLE createEmscriptenContext(const QString &canvasId, QSurfaceFormat format);
+    struct QOpenGLContextData
+    {
+        QPlatformSurface *surface = nullptr;
+        EMSCRIPTEN_WEBGL_CONTEXT_HANDLE handle = 0;
+    };
 
-    QSurfaceFormat m_requestedFormat;
-    QPlatformScreen *m_screen = nullptr;
-    EMSCRIPTEN_WEBGL_CONTEXT_HANDLE m_context = 0;
+    static bool isOpenGLVersionSupported(QSurfaceFormat format);
+    EMSCRIPTEN_WEBGL_CONTEXT_HANDLE obtainEmscriptenContext(QPlatformSurface *surface);
+    static EMSCRIPTEN_WEBGL_CONTEXT_HANDLE
+    createEmscriptenContext(const std::string &canvasSelector, QSurfaceFormat format);
+
+    static void destroyWebGLContext(EMSCRIPTEN_WEBGL_CONTEXT_HANDLE contextHandle);
+
+    QSurfaceFormat m_actualFormat;
+    QOpenGLContext *m_qGlContext;
+    QOpenGLContextData m_ownedWebGLContext;
+    EMSCRIPTEN_WEBGL_CONTEXT_HANDLE m_usedWebGLContextHandle = 0;
 };
 
 QT_END_NAMESPACE
 
+#endif // QWASMOPENGLCONTEXT_H
