@@ -5,7 +5,6 @@
 #include "qwindowscontext.h"
 #include "qwindowswindow.h"
 #include "qwindowsintegration.h"
-#include "qwindowsmousehandler.h"
 
 #include <QtCore/qdebug.h>
 #include <QtCore/qobject.h>
@@ -676,7 +675,7 @@ int QWindowsInputContext::reconvertString(RECONVERTSTRING *reconv)
     reconv->dwTargetStrOffset = reconv->dwCompStrOffset;
     auto *pastReconv = reinterpret_cast<ushort *>(reconv + 1);
     std::copy(surroundingText.utf16(), surroundingText.utf16() + surroundingText.size(),
-              QT_MAKE_UNCHECKED_ARRAY_ITERATOR(pastReconv));
+              pastReconv);
     return memSize;
 }
 
