@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtTest module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <QtTest/qtestassert.h>
 
@@ -60,6 +24,7 @@
 #include <QtCore/qbytearray.h>
 #include <QtCore/qelapsedtimer.h>
 #include <QtCore/qlist.h>
+#include <QtCore/qmutex.h>
 #include <QtCore/qvariant.h>
 #if QT_CONFIG(regularexpression)
 #include <QtCore/QRegularExpression>
@@ -69,8 +34,16 @@
 #include <string.h>
 #include <limits.h>
 
+#include <QtCore/q20algorithm.h>
+#include <atomic>
+#include <cstdio>
+#include <memory>
+#include <vector>
+
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 static void saveCoverageTool(const char * appname, bool testfailed, bool installedTestCoverage)
 {
 #ifdef __COVERAGESCANNER__
@@ -96,10 +69,71 @@ static void saveCoverageTool(const char * appname, bool testfailed, bool install
 #endif
 }
 
-static QElapsedTimer elapsedFunctionTime;
-static QElapsedTimer elapsedTotalTime;
+Q_CONSTINIT static QElapsedTimer elapsedFunctionTime;
+Q_CONSTINIT static QElapsedTimer elapsedTotalTime;
 
-#define FOREACH_TEST_LOGGER for (QAbstractTestLogger *logger : *QTest::loggers())
+namespace {
+class LoggerRegistry
+{
+    using LoggersContainer = std::vector<std::shared_ptr<QAbstractTestLogger>>;
+    using SharedLoggersContainer = std::shared_ptr<LoggersContainer>;
+
+public:
+    void addLogger(std::unique_ptr<QAbstractTestLogger> logger)
+    {
+        // read/update/clone
+        const SharedLoggersContainer currentLoggers = load();
+        SharedLoggersContainer newLoggers = currentLoggers
+                ? std::make_shared<LoggersContainer>(*currentLoggers)
+                : std::make_shared<LoggersContainer>();
+        newLoggers->emplace_back(std::move(logger));
+        store(std::move(newLoggers));
+    }
+
+    void clear() { store(SharedLoggersContainer{}); }
+
+    auto allLoggers() const
+    {
+        struct LoggersRange
+        {
+            const SharedLoggersContainer loggers;
+
+            auto begin() const
+            {
+                return loggers ? loggers->cbegin() : LoggersContainer::const_iterator{};
+            }
+            auto end() const
+            {
+                return loggers ? loggers->cend() : LoggersContainer::const_iterator{};
+            }
+            bool isEmpty() const { return loggers ? loggers->empty() : true; }
+        };
+
+        return LoggersRange{ load() };
+    }
+
+private:
+#ifdef __cpp_lib_atomic_shared_ptr
+    SharedLoggersContainer load() const { return loggers.load(std::memory_order_relaxed); }
+    void store(SharedLoggersContainer newLoggers)
+    {
+        loggers.store(std::move(newLoggers), std::memory_order_relaxed);
+    }
+    std::atomic<SharedLoggersContainer> loggers;
+#else
+    SharedLoggersContainer load() const
+    {
+        return std::atomic_load_explicit(&loggers, std::memory_order_relaxed);
+    }
+    void store(SharedLoggersContainer newLoggers)
+    {
+        std::atomic_store_explicit(&loggers, std::move(newLoggers), std::memory_order_relaxed);
+    }
+    SharedLoggersContainer loggers;
+#endif
+};
+
+} // namespace
 
 namespace QTest {
 
@@ -107,6 +141,7 @@ namespace QTest {
     int passes = 0;
     int skips = 0;
     int blacklists = 0;
+    enum { Unresolved, Passed, Skipped, Suppressed, Failed } currentTestState;
 
     struct IgnoreResultList
     {
@@ -143,8 +178,8 @@ namespace QTest {
             // ignore an optional whitespace at the end of str
             // (the space was added automatically by ~QDebug() until Qt 5.3,
             //  so autotests still might expect it)
-            if (expected.endsWith(QLatin1Char(' ')))
-                return actual == QStringView{expected}.left(expected.length() - 1);
+            if (expected.endsWith(u' '))
+                return actual == QStringView{expected}.left(expected.size() - 1);
 
             return false;
         }
@@ -167,8 +202,11 @@ namespace QTest {
     };
 
     static IgnoreResultList *ignoreResultList = nullptr;
+    Q_CONSTINIT static QBasicMutex mutex;
 
-    Q_GLOBAL_STATIC(QList<QAbstractTestLogger *>, loggers)
+    static std::vector<QVariant> failOnWarningList;
+
+    Q_GLOBAL_STATIC(LoggerRegistry, loggers)
 
     static int verbosity = 0;
     static int maxWarnings = 2002;
@@ -178,6 +216,8 @@ namespace QTest {
 
     static bool handleIgnoredMessage(QtMsgType type, const QString &message)
     {
+        const QMutexLocker mutexLocker(&QTest::mutex);
+
         if (!ignoreResultList)
             return false;
         IgnoreResultList *last = nullptr;
@@ -187,10 +227,8 @@ namespace QTest {
                 // remove the item from the list
                 if (last)
                     last->next = list->next;
-                else if (list->next)
-                    ignoreResultList = list->next;
                 else
-                    ignoreResultList = nullptr;
+                    ignoreResultList = list->next;
 
                 delete list;
                 return true;
@@ -202,14 +240,44 @@ namespace QTest {
         return false;
     }
 
+    static bool handleFailOnWarning(const QMessageLogContext &context, const QString &message)
+    {
+        // failOnWarning can be called multiple times per test function, so let
+        // each call cause a failure if required.
+        for (const auto &pattern : failOnWarningList) {
+            if (pattern.metaType() == QMetaType::fromType<QString>()) {
+                if (message != pattern.toString())
+                    continue;
+            }
+#if QT_CONFIG(regularexpression)
+            else if (pattern.metaType() == QMetaType::fromType<QRegularExpression>()) {
+                if (!message.contains(pattern.toRegularExpression()))
+                    continue;
+            }
+#endif
+
+            const size_t maxMsgLen = 1024;
+            char msg[maxMsgLen] = {'\0'};
+            std::snprintf(msg, maxMsgLen, "Received a warning that resulted in a failure:\n%s",
+                          qPrintable(message));
+            QTestResult::addFailure(msg, context.file, context.line);
+            return true;
+        }
+        return false;
+    }
+
     static void messageHandler(QtMsgType type, const QMessageLogContext & context, const QString &message)
     {
         static QBasicAtomicInt counter = Q_BASIC_ATOMIC_INITIALIZER(QTest::maxWarnings);
 
-        if (QTestLog::loggerCount() == 0) {
-            // if this goes wrong, something is seriously broken.
-            qInstallMessageHandler(oldMessageHandler);
-            QTEST_ASSERT(QTestLog::loggerCount() != 0);
+        auto loggerCapture = loggers->allLoggers();
+
+        if (loggerCapture.isEmpty()) {
+            // the message handler may be called from a worker thread, after the main thread stopped
+            // logging. Forwarding to original message handler to avoid swallowing the message
+            Q_ASSERT(oldMessageHandler);
+            oldMessageHandler(type, context, message);
+            return;
         }
 
         if (handleIgnoredMessage(type, message)) {
@@ -217,24 +285,28 @@ namespace QTest {
             return;
         }
 
+        if (type == QtWarningMsg && handleFailOnWarning(context, message))
+            return;
+
         if (type != QtFatalMsg) {
             if (counter.loadRelaxed() <= 0)
                 return;
 
             if (!counter.deref()) {
-                FOREACH_TEST_LOGGER {
+                for (auto &logger : loggerCapture)
                     logger->addMessage(QAbstractTestLogger::Warn,
-                        QStringLiteral("Maximum amount of warnings exceeded. Use -maxwarnings to override."));
-                }
+                                       QStringLiteral("Maximum amount of warnings exceeded. Use "
+                                                      "-maxwarnings to override."));
+
                 return;
             }
         }
 
-        FOREACH_TEST_LOGGER
+        for (auto &logger : loggerCapture)
             logger->addMessage(type, context, message);
 
         if (type == QtFatalMsg) {
-             /* Right now, we're inside the custom message handler and we're
+            /* Right now, we're inside the custom message handler and we're
              * being qt_message_output in qglobal.cpp. After we return from
              * this function, it will proceed with calling exit() and abort()
              * and hence crash. Therefore, we call these logging functions such
@@ -254,7 +326,7 @@ void QTestLog::enterTestFunction(const char* function)
 
     QTEST_ASSERT(function);
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->enterTestFunction(function);
 }
 
@@ -262,12 +334,13 @@ void QTestLog::enterTestData(QTestData *data)
 {
     QTEST_ASSERT(data);
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->enterTestData(data);
 }
 
 int QTestLog::unhandledIgnoreMessages()
 {
+    const QMutexLocker mutexLocker(&QTest::mutex);
     int i = 0;
     QTest::IgnoreResultList *list = QTest::ignoreResultList;
     while (list) {
@@ -282,23 +355,25 @@ void QTestLog::leaveTestFunction()
     if (printAvailableTags)
         return;
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->leaveTestFunction();
 }
 
 void QTestLog::printUnhandledIgnoreMessages()
 {
+    const QMutexLocker mutexLocker(&QTest::mutex);
     QString message;
     QTest::IgnoreResultList *list = QTest::ignoreResultList;
     while (list) {
         if (list->pattern.userType() == QMetaType::QString) {
-            message = QStringLiteral("Did not receive message: \"") + list->pattern.toString() + QLatin1Char('"');
+            message = "Did not receive message: \"%1\""_L1.arg(list->pattern.toString());
         } else {
 #if QT_CONFIG(regularexpression)
-            message = QStringLiteral("Did not receive any message matching: \"") + list->pattern.toRegularExpression().pattern() + QLatin1Char('"');
+            message = "Did not receive any message matching: \"%1\""_L1.arg(
+                    list->pattern.toRegularExpression().pattern());
 #endif
         }
-        FOREACH_TEST_LOGGER
+        for (auto &logger : QTest::loggers->allLoggers())
             logger->addMessage(QAbstractTestLogger::Info, message);
 
         list = list->next;
@@ -307,19 +382,34 @@ void QTestLog::printUnhandledIgnoreMessages()
 
 void QTestLog::clearIgnoreMessages()
 {
+    const QMutexLocker mutexLocker(&QTest::mutex);
     QTest::IgnoreResultList::clearList(QTest::ignoreResultList);
 }
 
+void QTestLog::clearFailOnWarnings()
+{
+    QTest::failOnWarningList.clear();
+}
+
+void QTestLog::clearCurrentTestState()
+{
+    clearIgnoreMessages();
+    clearFailOnWarnings();
+    QTest::currentTestState = QTest::Unresolved;
+}
+
 void QTestLog::addPass(const char *msg)
 {
     if (printAvailableTags)
         return;
 
     QTEST_ASSERT(msg);
+    Q_ASSERT(QTest::currentTestState == QTest::Unresolved);
 
     ++QTest::passes;
+    QTest::currentTestState = QTest::Passed;
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::Pass, msg);
 }
 
@@ -327,9 +417,19 @@ void QTestLog::addFail(const char *msg, const char *file, int line)
 {
     QTEST_ASSERT(msg);
 
-    ++QTest::fails;
+    if (QTest::currentTestState == QTest::Unresolved) {
+        ++QTest::fails;
+    } else {
+        // After an XPASS/Continue, or fail or skip in a function the test
+        // calls, we can subsequently fail.
+        Q_ASSERT(QTest::currentTestState == QTest::Failed
+                 || QTest::currentTestState == QTest::Skipped);
+    }
+    // It is up to particular loggers to decide whether to report such
+    // subsequent failures; they may carry useful information.
 
-    FOREACH_TEST_LOGGER
+    QTest::currentTestState = QTest::Failed;
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::Fail, msg, file, line);
 }
 
@@ -339,7 +439,7 @@ void QTestLog::addXFail(const char *msg, const char *file, int line)
 
     // Will be counted in addPass() if we get there.
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::XFail, msg, file, line);
 }
 
@@ -347,19 +447,29 @@ void QTestLog::addXPass(const char *msg, const char *file, int line)
 {
     QTEST_ASSERT(msg);
 
-    ++QTest::fails;
+    if (QTest::currentTestState == QTest::Unresolved) {
+        ++QTest::fails;
+    } else {
+        // After an XPASS/Continue, we can subsequently XPASS again.
+        // Likewise after a fail or skip in a function called by the test.
+        Q_ASSERT(QTest::currentTestState == QTest::Failed
+                 || QTest::currentTestState == QTest::Skipped);
+    }
 
-    FOREACH_TEST_LOGGER
+    QTest::currentTestState = QTest::Failed;
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::XPass, msg, file, line);
 }
 
 void QTestLog::addBPass(const char *msg)
 {
     QTEST_ASSERT(msg);
+    Q_ASSERT(QTest::currentTestState == QTest::Unresolved);
 
-    ++QTest::blacklists;
+    ++QTest::blacklists; // Not passes ?
+    QTest::currentTestState = QTest::Suppressed;
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::BlacklistedPass, msg);
 }
 
@@ -367,9 +477,17 @@ void QTestLog::addBFail(const char *msg, const char *file, int line)
 {
     QTEST_ASSERT(msg);
 
-    ++QTest::blacklists;
+    if (QTest::currentTestState == QTest::Unresolved) {
+        ++QTest::blacklists;
+    } else {
+        // After a BXPASS/Continue, we can subsequently fail.
+        // Likewise after a fail or skip in a function called by a test.
+        Q_ASSERT(QTest::currentTestState == QTest::Suppressed
+                 || QTest::currentTestState == QTest::Skipped);
+    }
 
-    FOREACH_TEST_LOGGER
+    QTest::currentTestState = QTest::Suppressed;
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::BlacklistedFail, msg, file, line);
 }
 
@@ -377,9 +495,17 @@ void QTestLog::addBXPass(const char *msg, const char *file, int line)
 {
     QTEST_ASSERT(msg);
 
-    ++QTest::blacklists;
+    if (QTest::currentTestState == QTest::Unresolved) {
+        ++QTest::blacklists;
+    } else {
+        // After a BXPASS/Continue, we may BXPASS again.
+        // Likewise after a fail or skip in a function called by a test.
+        Q_ASSERT(QTest::currentTestState == QTest::Suppressed
+                 || QTest::currentTestState == QTest::Skipped);
+    }
 
-    FOREACH_TEST_LOGGER
+    QTest::currentTestState = QTest::Suppressed;
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::BlacklistedXPass, msg, file, line);
 }
 
@@ -389,7 +515,7 @@ void QTestLog::addBXFail(const char *msg, const char *file, int line)
 
     // Will be counted in addBPass() if we get there.
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::BlacklistedXFail, msg, file, line);
 }
 
@@ -397,23 +523,34 @@ void QTestLog::addSkip(const char *msg, const char *file, int line)
 {
     QTEST_ASSERT(msg);
 
-    ++QTest::skips;
+    if (QTest::currentTestState == QTest::Unresolved) {
+        ++QTest::skips;
+        QTest::currentTestState = QTest::Skipped;
+    } else {
+        // After an B?XPASS/Continue, we might subsequently skip.
+        // Likewise after a skip in a function called by a test.
+        Q_ASSERT(QTest::currentTestState == QTest::Suppressed
+                 || QTest::currentTestState == QTest::Failed
+                 || QTest::currentTestState == QTest::Skipped);
+    }
+    // It is up to particular loggers to decide whether to report such
+    // subsequent skips; they may carry useful information.
 
-    FOREACH_TEST_LOGGER
-        logger->addMessage(QAbstractTestLogger::Skip, QString::fromUtf8(msg), file, line);
+    for (auto &logger : QTest::loggers->allLoggers())
+        logger->addIncident(QAbstractTestLogger::Skip, msg, file, line);
 }
 
-void QTestLog::addBenchmarkResult(const QBenchmarkResult &result)
+void QTestLog::addBenchmarkResults(const QList<QBenchmarkResult> &results)
 {
-    FOREACH_TEST_LOGGER
-        logger->addBenchmarkResult(result);
+    for (auto &logger : QTest::loggers->allLoggers())
+        logger->addBenchmarkResults(results);
 }
 
 void QTestLog::startLogging()
 {
     elapsedTotalTime.start();
     elapsedFunctionTime.start();
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->startLogging();
     QTest::oldMessageHandler = qInstallMessageHandler(QTest::messageHandler);
 }
@@ -421,11 +558,10 @@ void QTestLog::startLogging()
 void QTestLog::stopLogging()
 {
     qInstallMessageHandler(QTest::oldMessageHandler);
-    FOREACH_TEST_LOGGER {
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->stopLogging();
-        delete logger;
-    }
-    QTest::loggers()->clear();
+
+    QTest::loggers->clear();
     saveCoverageTool(QTestResult::currentAppName(), failCount() != 0, QTestLog::installedTestCoverage());
 }
 
@@ -470,7 +606,7 @@ void QTestLog::addLogger(LogMode mode, const char *filename)
     }
 
     QTEST_ASSERT(logger);
-    addLogger(logger);
+    addLogger(std::unique_ptr<QAbstractTestLogger>{ logger });
 }
 
 /*!
@@ -478,35 +614,45 @@ void QTestLog::addLogger(LogMode mode, const char *filename)
 
     Adds a new logger to the set of loggers that will be used
     to report incidents and messages during testing.
-
-    The function takes ownership of the logger.
 */
-void QTestLog::addLogger(QAbstractTestLogger *logger)
+void QTestLog::addLogger(std::unique_ptr<QAbstractTestLogger> logger)
 {
     QTEST_ASSERT(logger);
-    QTest::loggers()->append(logger);
+    QTest::loggers()->addLogger(std::move(logger));
 }
 
-int QTestLog::loggerCount()
+bool QTestLog::hasLoggers()
 {
-    return QTest::loggers()->size();
+    return !QTest::loggers()->allLoggers().isEmpty();
+}
+
+/*!
+    \internal
+
+    Returns true if all loggers support repeated test runs
+*/
+bool QTestLog::isRepeatSupported()
+{
+    for (auto &logger : QTest::loggers->allLoggers())
+        if (!logger->isRepeatSupported())
+            return false;
+
+    return true;
 }
 
 bool QTestLog::loggerUsingStdout()
 {
-    FOREACH_TEST_LOGGER {
-        if (logger->isLoggingToStdout())
-            return true;
-    }
-
-    return false;
+    auto loggersCapture = QTest::loggers->allLoggers();
+    return q20::ranges::any_of(loggersCapture.begin(), loggersCapture.end(), [](auto &logger) {
+        return logger->isLoggingToStdout();
+    });
 }
 
 void QTestLog::warn(const char *msg, const char *file, int line)
 {
     QTEST_ASSERT(msg);
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addMessage(QAbstractTestLogger::Warn, QString::fromUtf8(msg), file, line);
 }
 
@@ -514,7 +660,7 @@ void QTestLog::info(const char *msg, const char *file, int line)
 {
     QTEST_ASSERT(msg);
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addMessage(QAbstractTestLogger::Info, QString::fromUtf8(msg), file, line);
 }
 
@@ -532,6 +678,7 @@ void QTestLog::ignoreMessage(QtMsgType type, const char *msg)
 {
     QTEST_ASSERT(msg);
 
+    const QMutexLocker mutexLocker(&QTest::mutex);
     QTest::IgnoreResultList::append(QTest::ignoreResultList, type, QString::fromUtf8(msg));
 }
 
@@ -540,10 +687,30 @@ void QTestLog::ignoreMessage(QtMsgType type, const QRegularExpression &expressio
 {
     QTEST_ASSERT(expression.isValid());
 
+    const QMutexLocker mutexLocker(&QTest::mutex);
     QTest::IgnoreResultList::append(QTest::ignoreResultList, type, QVariant(expression));
 }
 #endif // QT_CONFIG(regularexpression)
 
+void QTestLog::failOnWarning()
+{
+    QTest::failOnWarningList.push_back({});
+}
+
+void QTestLog::failOnWarning(const char *msg)
+{
+    QTest::failOnWarningList.push_back(QString::fromUtf8(msg));
+}
+
+#if QT_CONFIG(regularexpression)
+void QTestLog::failOnWarning(const QRegularExpression &expression)
+{
+    QTEST_ASSERT(expression.isValid());
+
+    QTest::failOnWarningList.push_back(QVariant::fromValue(expression));
+}
+#endif // QT_CONFIG(regularexpression)
+
 void QTestLog::setMaxWarnings(int m)
 {
     QTest::maxWarnings = m <= 0 ? INT_MAX : m + 2;
