@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWidgets module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #ifndef QABSTRACTITEMVIEW_P_H
 #define QABSTRACTITEMVIEW_P_H
@@ -52,17 +16,22 @@
 //
 
 #include <QtWidgets/private/qtwidgetsglobal_p.h>
+#include "qabstractitemview.h"
 #include "private/qabstractscrollarea_p.h"
 #include "private/qabstractitemmodel_p.h"
 #include "QtWidgets/qapplication.h"
 #include "QtGui/qevent.h"
 #include "QtCore/qmimedata.h"
 #include "QtGui/qpainter.h"
-#include "QtCore/qpair.h"
 #include "QtGui/qregion.h"
+
 #include "QtCore/qdebug.h"
 #include "QtCore/qbasictimer.h"
 #include "QtCore/qelapsedtimer.h"
+#include <QtCore/qpointer.h>
+
+
+#include <array>
 
 QT_REQUIRE_CONFIG(itemviews);
 
@@ -99,19 +68,20 @@ public:
 
     void init();
 
-    virtual void _q_rowsRemoved(const QModelIndex &parent, int start, int end);
-    virtual void _q_rowsInserted(const QModelIndex &parent, int start, int end);
-    virtual void _q_columnsAboutToBeRemoved(const QModelIndex &parent, int start, int end);
-    virtual void _q_columnsRemoved(const QModelIndex &parent, int start, int end);
-    virtual void _q_columnsInserted(const QModelIndex &parent, int start, int end);
-    virtual void _q_modelDestroyed();
-    virtual void _q_layoutChanged();
-    virtual void _q_rowsMoved(const QModelIndex &source, int sourceStart, int sourceEnd, const QModelIndex &destination, int destinationStart);
-    virtual void _q_columnsMoved(const QModelIndex &source, int sourceStart, int sourceEnd, const QModelIndex &destination, int destinationStart);
+    virtual void rowsRemoved(const QModelIndex &parent, int start, int end);
+    virtual void rowsInserted(const QModelIndex &parent, int start, int end);
+    virtual void columnsAboutToBeRemoved(const QModelIndex &parent, int start, int end);
+    virtual void columnsRemoved(const QModelIndex &parent, int start, int end);
+    virtual void columnsInserted(const QModelIndex &parent, int start, int end);
+    virtual void modelDestroyed();
+    virtual void layoutChanged();
+    virtual void rowsMoved(const QModelIndex &source, int sourceStart, int sourceEnd, const QModelIndex &destination, int destinationStart);
+    virtual void columnsMoved(const QModelIndex &source, int sourceStart, int sourceEnd, const QModelIndex &destination, int destinationStart);
     virtual QRect intersectedRect(const QRect rect, const QModelIndex &topLeft, const QModelIndex &bottomRight) const;
 
-    void _q_headerDataChanged() { doDelayedItemsLayout(); }
-    void _q_scrollerStateChanged();
+    void headerDataChanged() { doDelayedItemsLayout(); }
+    void scrollerStateChanged();
+    void delegateSizeHintChanged(const QModelIndex &index);
 
     void fetchMore();
 
@@ -194,10 +164,11 @@ public:
     inline void paintDropIndicator(QPainter *painter)
     {
         if (showDropIndicator && state == QAbstractItemView::DraggingState
+            && !dropIndicatorRect.isNull()
 #ifndef QT_NO_CURSOR
             && viewport->cursor().shape() != Qt::ForbiddenCursor
 #endif
-            ) {
+        ) {
             QStyleOption opt;
             opt.initFrom(q_func());
             opt.rect = dropIndicatorRect;
@@ -213,8 +184,8 @@ public:
     inline void releaseEditor(QWidget *editor, const QModelIndex &index = QModelIndex()) const {
         if (editor) {
             Q_Q(const QAbstractItemView);
-            QObject::disconnect(editor, SIGNAL(destroyed(QObject*)),
-                                q_func(), SLOT(editorDestroyed(QObject*)));
+            QObject::disconnect(editor, &QWidget::destroyed,
+                                q, &QAbstractItemView::editorDestroyed);
             editor->removeEventFilter(itemDelegate);
             editor->hide();
             QAbstractItemDelegate *delegate = q->itemDelegateForIndex(index);
@@ -287,12 +258,14 @@ public:
     inline bool isIndexEnabled(const QModelIndex &index) const {
         return (model->flags(index) & Qt::ItemIsEnabled);
     }
+#if QT_CONFIG(draganddrop)
     inline bool isIndexDropEnabled(const QModelIndex &index) const {
         return (model->flags(index) & Qt::ItemIsDropEnabled);
     }
     inline bool isIndexDragEnabled(const QModelIndex &index) const {
         return (model->flags(index) & Qt::ItemIsDragEnabled);
     }
+#endif
 
     virtual bool selectionAllowed(const QModelIndex &index) const {
         // in some views we want to go ahead with selections, even if the index is invalid
@@ -339,13 +312,16 @@ public:
         return static_cast<QAbstractItemModelPrivate *>(model->d_ptr.data())->persistent.indexes.contains(index);
     }
 
+#if QT_CONFIG(draganddrop)
     QModelIndexList selectedDraggableIndexes() const;
+    void maybeStartDrag(QPoint eventPoint);
+#endif
 
     void doDelayedReset()
     {
         //we delay the reset of the timer because some views (QTableView)
         //with headers can't handle the fact that the model has been destroyed
-        //all _q_modelDestroyed slots must have been called
+        //all modelDestroyed() slots must have been called
         if (!delayedReset.isActive())
             delayedReset.start(0, q_func());
     }
@@ -375,6 +351,7 @@ public:
     QPersistentModelIndex currentSelectionStartIndex;
     Qt::KeyboardModifiers pressedModifiers;
     QPoint pressedPosition;
+    QPoint draggedPosition;
     bool pressedAlreadySelected;
     bool releaseFromDoubleClick;
 
@@ -445,7 +422,18 @@ public:
     bool verticalScrollModeSet;
     bool horizontalScrollModeSet;
 
+    virtual QRect visualRect(const QModelIndex &index) const { return q_func()->visualRect(index); }
+
+    std::array<QMetaObject::Connection, 14> modelConnections;
+    std::array<QMetaObject::Connection, 4> scrollbarConnections;
+#if QT_CONFIG(gestures) && QT_CONFIG(scroller)
+    QMetaObject::Connection scollerConnection;
+#endif
+
 private:
+    void connectDelegate(QAbstractItemDelegate *delegate);
+    void disconnectDelegate(QAbstractItemDelegate *delegate);
+    void disconnectAll();
     inline QAbstractItemDelegate *delegateForIndex(const QModelIndex &index) const {
         QMap<int, QPointer<QAbstractItemDelegate> >::ConstIterator it;
 
