@@ -1,45 +1,8 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWidgets module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qtablewidget.h"
 
-#include <qitemdelegate.h>
 #include <qpainter.h>
 #include <private/qtablewidget_p.h>
 
@@ -63,12 +26,12 @@ QTableModel::~QTableModel()
 
 bool QTableModel::insertRows(int row, int count, const QModelIndex &)
 {
-    if (count < 1 || row < 0 || row > verticalHeaderItems.count())
+    if (count < 1 || row < 0 || row > verticalHeaderItems.size())
         return false;
 
     beginInsertRows(QModelIndex(), row, row + count - 1);
-    int rc = verticalHeaderItems.count();
-    int cc = horizontalHeaderItems.count();
+    int rc = verticalHeaderItems.size();
+    int cc = horizontalHeaderItems.size();
     verticalHeaderItems.insert(row, count, 0);
     if (rc == 0)
         tableItems.resize(cc * count);
@@ -80,12 +43,12 @@ bool QTableModel::insertRows(int row, int count, const QModelIndex &)
 
 bool QTableModel::insertColumns(int column, int count, const QModelIndex &)
 {
-    if (count < 1 || column < 0 || column > horizontalHeaderItems.count())
+    if (count < 1 || column < 0 || column > horizontalHeaderItems.size())
         return false;
 
     beginInsertColumns(QModelIndex(), column, column + count - 1);
-    int rc = verticalHeaderItems.count();
-    int cc = horizontalHeaderItems.count();
+    int rc = verticalHeaderItems.size();
+    int cc = horizontalHeaderItems.size();
     horizontalHeaderItems.insert(column, count, 0);
     if (cc == 0)
         tableItems.resize(rc * count);
@@ -98,7 +61,7 @@ bool QTableModel::insertColumns(int column, int count, const QModelIndex &)
 
 bool QTableModel::removeRows(int row, int count, const QModelIndex &)
 {
-    if (count < 1 || row < 0 || row + count > verticalHeaderItems.count())
+    if (count < 1 || row < 0 || row + count > verticalHeaderItems.size())
         return false;
 
     beginRemoveRows(QModelIndex(), row, row + count - 1);
@@ -125,7 +88,7 @@ bool QTableModel::removeRows(int row, int count, const QModelIndex &)
 
 bool QTableModel::removeColumns(int column, int count, const QModelIndex &)
 {
-    if (count < 1 || column < 0 || column + count >  horizontalHeaderItems.count())
+    if (count < 1 || column < 0 || column + count >  horizontalHeaderItems.size())
         return false;
 
     beginRemoveColumns(QModelIndex(), column, column + count - 1);
@@ -151,10 +114,50 @@ bool QTableModel::removeColumns(int column, int count, const QModelIndex &)
     return true;
 }
 
+bool QTableModel::moveRows(const QModelIndex &sourceParent, int sourceRow, int count, const QModelIndex &destinationParent, int destinationChild)
+{
+    if (sourceRow < 0
+        || sourceRow + count - 1 >= rowCount(sourceParent)
+        || destinationChild < 0
+        || destinationChild > rowCount(destinationParent)
+        || sourceRow == destinationChild
+        || sourceRow == destinationChild - 1
+        || count <= 0
+        || sourceParent.isValid()
+        || destinationParent.isValid()) {
+        return false;
+    }
+    if (!beginMoveRows(sourceParent, sourceRow, sourceRow + count - 1, destinationParent, destinationChild))
+        return false;
+
+    // Table items
+    int numItems = count * columnCount();
+    int fromIndex = tableIndex(sourceRow, 0);
+    int destinationIndex = tableIndex(destinationChild, 0);
+    if (destinationChild < sourceRow)
+        fromIndex += numItems - 1;
+    else
+        destinationIndex--;
+    while (numItems--)
+        tableItems.move(fromIndex, destinationIndex);
+
+    // Header items
+    int fromRow = sourceRow;
+    if (destinationChild < sourceRow)
+        fromRow += count - 1;
+    else
+        destinationChild--;
+    while (count--)
+        verticalHeaderItems.move(fromRow, destinationChild);
+
+    endMoveRows();
+    return true;
+}
+
 void QTableModel::setItem(int row, int column, QTableWidgetItem *item)
 {
     int i = tableIndex(row, column);
-    if (i < 0 || i >= tableItems.count())
+    if (i < 0 || i >= tableItems.size())
         return;
     QTableWidgetItem *oldItem = tableItems.at(i);
     if (item == oldItem)
@@ -177,39 +180,20 @@ void QTableModel::setItem(int row, int column, QTableWidgetItem *item)
         // sorted insertion
         Qt::SortOrder order = view->horizontalHeader()->sortIndicatorOrder();
         QList<QTableWidgetItem *> colItems = columnItems(column);
-        if (row < colItems.count())
+        if (row < colItems.size())
             colItems.remove(row);
         int sortedRow;
         if (item == nullptr) {
             // move to after all non-0 (sortable) items
-            sortedRow = colItems.count();
+            sortedRow = colItems.size();
         } else {
             QList<QTableWidgetItem *>::iterator it;
             it = sortedInsertionIterator(colItems.begin(), colItems.end(), order, item);
             sortedRow = qMax((int)(it - colItems.begin()), 0);
         }
         if (sortedRow != row) {
-            emit layoutAboutToBeChanged({}, QAbstractItemModel::VerticalSortHint);
-            // move the items @ row to sortedRow
-            int cc = columnCount();
-            QList<QTableWidgetItem *> rowItems(cc);
-            for (int j = 0; j < cc; ++j)
-                rowItems[j] = tableItems.at(tableIndex(row, j));
-            tableItems.remove(tableIndex(row, 0), cc);
-            tableItems.insert(tableIndex(sortedRow, 0), cc, 0);
-            for (int j = 0; j < cc; ++j)
-                tableItems[tableIndex(sortedRow, j)] = rowItems.at(j);
-            QTableWidgetItem *header = verticalHeaderItems.at(row);
-            verticalHeaderItems.remove(row);
-            verticalHeaderItems.insert(sortedRow, header);
-            // update persistent indexes
-            QModelIndexList oldPersistentIndexes = persistentIndexList();
-            QModelIndexList newPersistentIndexes = oldPersistentIndexes;
-            updateRowIndexes(newPersistentIndexes, row, sortedRow);
-            changePersistentIndexList(oldPersistentIndexes,
-                                      newPersistentIndexes);
-
-            emit layoutChanged({}, QAbstractItemModel::VerticalSortHint);
+            const int destinationChild = sortedRow > row ? sortedRow + 1 : sortedRow;
+            moveRows(QModelIndex(), row, 1, QModelIndex(), destinationChild);
             return;
         }
     }
@@ -270,7 +254,7 @@ void QTableModel::removeItem(QTableWidgetItem *item)
 
 void QTableModel::setHorizontalHeaderItem(int section, QTableWidgetItem *item)
 {
-    if (section < 0 || section >= horizontalHeaderItems.count())
+    if (section < 0 || section >= horizontalHeaderItems.size())
         return;
     QTableWidgetItem *oldItem = horizontalHeaderItems.at(section);
     if (item == oldItem)
@@ -284,7 +268,7 @@ void QTableModel::setHorizontalHeaderItem(int section, QTableWidgetItem *item)
 
     if (item) {
         item->view = view;
-        item->itemFlags = Qt::ItemFlags(int(item->itemFlags)|ItemIsHeaderItem);
+        item->d->headerItem = true;
     }
     horizontalHeaderItems[section] = item;
     emit headerDataChanged(Qt::Horizontal, section, section);
@@ -292,7 +276,7 @@ void QTableModel::setHorizontalHeaderItem(int section, QTableWidgetItem *item)
 
 void QTableModel::setVerticalHeaderItem(int section, QTableWidgetItem *item)
 {
-    if (section < 0 || section >= verticalHeaderItems.count())
+    if (section < 0 || section >= verticalHeaderItems.size())
         return;
     QTableWidgetItem *oldItem = verticalHeaderItems.at(section);
     if (item == oldItem)
@@ -306,7 +290,7 @@ void QTableModel::setVerticalHeaderItem(int section, QTableWidgetItem *item)
 
     if (item) {
         item->view = view;
-        item->itemFlags = Qt::ItemFlags(int(item->itemFlags)|ItemIsHeaderItem);
+        item->d->headerItem = true;
     }
     verticalHeaderItems[section] = item;
     emit headerDataChanged(Qt::Vertical, section, section);
@@ -314,12 +298,12 @@ void QTableModel::setVerticalHeaderItem(int section, QTableWidgetItem *item)
 
 QTableWidgetItem *QTableModel::takeHorizontalHeaderItem(int section)
 {
-    if (section < 0 || section >= horizontalHeaderItems.count())
+    if (section < 0 || section >= horizontalHeaderItems.size())
         return nullptr;
     QTableWidgetItem *itm = horizontalHeaderItems.at(section);
     if (itm) {
         itm->view = nullptr;
-        itm->itemFlags &= ~ItemIsHeaderItem;
+        itm->d->headerItem = false;
         horizontalHeaderItems[section] = 0;
     }
     return itm;
@@ -327,12 +311,12 @@ QTableWidgetItem *QTableModel::takeHorizontalHeaderItem(int section)
 
 QTableWidgetItem *QTableModel::takeVerticalHeaderItem(int section)
 {
-    if (section < 0 || section >= verticalHeaderItems.count())
+    if (section < 0 || section >= verticalHeaderItems.size())
         return nullptr;
     QTableWidgetItem *itm = verticalHeaderItems.at(section);
     if (itm) {
         itm->view = nullptr;
-        itm->itemFlags &= ~ItemIsHeaderItem;
+        itm->d->headerItem = false;
         verticalHeaderItems[section] = 0;
     }
     return itm;
@@ -354,7 +338,7 @@ QModelIndex QTableModel::index(const QTableWidgetItem *item) const
         return QModelIndex();
     int i = -1;
     const int id = item->d->id;
-    if (id >= 0 && id < tableItems.count() && tableItems.at(id) == item) {
+    if (id >= 0 && id < tableItems.size() && tableItems.at(id) == item) {
         i = id;
     } else { // we need to search for the item
         i = tableItems.indexOf(const_cast<QTableWidgetItem*>(item));
@@ -368,7 +352,7 @@ QModelIndex QTableModel::index(const QTableWidgetItem *item) const
 
 void QTableModel::setRowCount(int rows)
 {
-    int rc = verticalHeaderItems.count();
+    int rc = verticalHeaderItems.size();
     if (rows < 0 || rc == rows)
         return;
     if (rc < rows)
@@ -379,7 +363,7 @@ void QTableModel::setRowCount(int rows)
 
 void QTableModel::setColumnCount(int columns)
 {
-    int cc = horizontalHeaderItems.count();
+    int cc = horizontalHeaderItems.size();
     if (columns < 0 || cc == columns)
         return;
     if (cc < columns)
@@ -390,12 +374,12 @@ void QTableModel::setColumnCount(int columns)
 
 int QTableModel::rowCount(const QModelIndex &parent) const
 {
-    return parent.isValid() ? 0 : verticalHeaderItems.count();
+    return parent.isValid() ? 0 : verticalHeaderItems.size();
 }
 
 int QTableModel::columnCount(const QModelIndex &parent) const
 {
-    return parent.isValid() ? 0 : horizontalHeaderItems.count();
+    return parent.isValid() ? 0 : horizontalHeaderItems.size();
 }
 
 QVariant QTableModel::data(const QModelIndex &index, int role) const
@@ -436,7 +420,7 @@ QMap<int, QVariant> QTableModel::itemData(const QModelIndex &index) const
     QMap<int, QVariant> roles;
     QTableWidgetItem *itm = item(index);
     if (itm) {
-        for (int i = 0; i < itm->values.count(); ++i) {
+        for (int i = 0; i < itm->values.size(); ++i) {
             roles.insert(itm->values.at(i).role,
                          itm->values.at(i).value);
         }
@@ -512,15 +496,16 @@ Qt::ItemFlags QTableModel::flags(const QModelIndex &index) const
 
 void QTableModel::sort(int column, Qt::SortOrder order)
 {
-    QList<QPair<QTableWidgetItem *, int>> sortable;
+    QList<std::pair<QTableWidgetItem *, int>> sortable;
     QList<int> unsortable;
+    const int numRows = rowCount();
 
-    sortable.reserve(rowCount());
-    unsortable.reserve(rowCount());
+    sortable.reserve(numRows);
+    unsortable.reserve(numRows);
 
-    for (int row = 0; row < rowCount(); ++row) {
+    for (int row = 0; row < numRows; ++row) {
         if (QTableWidgetItem *itm = item(row, column))
-            sortable.append(QPair<QTableWidgetItem*,int>(itm, row));
+            sortable.emplace_back(itm, row);
         else
             unsortable.append(row);
     }
@@ -528,17 +513,16 @@ void QTableModel::sort(int column, Qt::SortOrder order)
     const auto compare = (order == Qt::AscendingOrder ? &itemLessThan : &itemGreaterThan);
     std::stable_sort(sortable.begin(), sortable.end(), compare);
 
-    QList<QTableWidgetItem *> sorted_table(tableItems.count());
+    QList<QTableWidgetItem *> sorted_table(tableItems.size());
     QModelIndexList from;
     QModelIndexList to;
-    const int numRows = rowCount();
     const int numColumns = columnCount();
     from.reserve(numRows * numColumns);
     to.reserve(numRows * numColumns);
     for (int i = 0; i < numRows; ++i) {
-        int r = (i < sortable.count()
+        int r = (i < sortable.size()
                  ? sortable.at(i).second
-                 : unsortable.at(i - sortable.count()));
+                 : unsortable.at(i - sortable.size()));
         for (int c = 0; c < numColumns; ++c) {
             sorted_table[tableIndex(i, c)] = item(r, c);
             from.append(createIndex(r, c));
@@ -565,7 +549,7 @@ void QTableModel::ensureSorted(int column, Qt::SortOrder order,
                                int start, int end)
 {
     int count = end - start + 1;
-    QList<QPair<QTableWidgetItem *, int>> sorting;
+    QList<std::pair<QTableWidgetItem *, int>> sorting;
     sorting.reserve(count);
     for (int row = start; row <= end; ++row) {
         QTableWidgetItem *itm = item(row, column);
@@ -574,7 +558,7 @@ void QTableModel::ensureSorted(int column, Qt::SortOrder order,
             // at the end of the table when it is sorted)
             break;
         }
-        sorting.append(QPair<QTableWidgetItem*,int>(itm, row));
+        sorting.emplace_back(itm, row);
     }
 
     const auto compare = (order == Qt::AscendingOrder ? &itemLessThan : &itemGreaterThan);
@@ -586,7 +570,7 @@ void QTableModel::ensureSorted(int column, Qt::SortOrder order,
     QList<QTableWidgetItem *>::iterator vit = colItems.begin();
     qsizetype distanceFromBegin = 0;
     bool changed = false;
-    for (int i = 0; i < sorting.count(); ++i) {
+    for (int i = 0; i < sorting.size(); ++i) {
         distanceFromBegin = std::distance(colItems.begin(), vit);
         int oldRow = sorting.at(i).second;
         QTableWidgetItem *item = colItems.at(oldRow);
@@ -619,7 +603,7 @@ void QTableModel::ensureSorted(int column, Qt::SortOrder order,
             // update persistent indexes
             updateRowIndexes(newPersistentIndexes, oldRow, newRow);
             // the index of the remaining rows may have changed
-            for (int j = i + 1; j < sorting.count(); ++j) {
+            for (int j = i + 1; j < sorting.size(); ++j) {
                 int otherRow = sorting.at(j).second;
                 if (oldRow < otherRow && newRow >= otherRow)
                     --sorting[j].second;
@@ -702,14 +686,14 @@ QTableModel::sortedInsertionIterator(const QList<QTableWidgetItem *>::iterator &
     return std::lower_bound(begin, end, item, QTableModelGreaterThan());
 }
 
-bool QTableModel::itemLessThan(const QPair<QTableWidgetItem*,int> &left,
-                               const QPair<QTableWidgetItem*,int> &right)
+bool QTableModel::itemLessThan(const std::pair<QTableWidgetItem*,int> &left,
+                               const std::pair<QTableWidgetItem*,int> &right)
 {
     return *(left.first) < *(right.first);
 }
 
-bool QTableModel::itemGreaterThan(const QPair<QTableWidgetItem*,int> &left,
-                                  const QPair<QTableWidgetItem*,int> &right)
+bool QTableModel::itemGreaterThan(const std::pair<QTableWidgetItem*,int> &left,
+                                  const std::pair<QTableWidgetItem*,int> &right)
 {
     return (*(right.first) < *(left .first));
 }
@@ -720,9 +704,9 @@ QVariant QTableModel::headerData(int section, Qt::Orientation orientation, int r
         return QVariant();
 
     QTableWidgetItem *itm = nullptr;
-    if (orientation == Qt::Horizontal && section < horizontalHeaderItems.count())
+    if (orientation == Qt::Horizontal && section < horizontalHeaderItems.size())
         itm = horizontalHeaderItems.at(section);
-    else if (orientation == Qt::Vertical && section < verticalHeaderItems.count())
+    else if (orientation == Qt::Vertical && section < verticalHeaderItems.size())
         itm = verticalHeaderItems.at(section);
     else
         return QVariant(); // section is out of bounds
@@ -757,20 +741,20 @@ bool QTableModel::setHeaderData(int section, Qt::Orientation orientation,
 bool QTableModel::isValid(const QModelIndex &index) const
 {
     return (index.isValid()
-            && index.row() < verticalHeaderItems.count()
-            && index.column() < horizontalHeaderItems.count());
+            && index.row() < verticalHeaderItems.size()
+            && index.column() < horizontalHeaderItems.size());
 }
 
 void QTableModel::clear()
 {
-    for (int j = 0; j < verticalHeaderItems.count(); ++j) {
+    for (int j = 0; j < verticalHeaderItems.size(); ++j) {
         if (verticalHeaderItems.at(j)) {
             verticalHeaderItems.at(j)->view = nullptr;
             delete verticalHeaderItems.at(j);
             verticalHeaderItems[j] = 0;
         }
     }
-    for (int k = 0; k < horizontalHeaderItems.count(); ++k) {
+    for (int k = 0; k < horizontalHeaderItems.size(); ++k) {
         if (horizontalHeaderItems.at(k)) {
             horizontalHeaderItems.at(k)->view = nullptr;
             delete horizontalHeaderItems.at(k);
@@ -783,7 +767,7 @@ void QTableModel::clear()
 void QTableModel::clearContents()
 {
     beginResetModel();
-    for (int i = 0; i < tableItems.count(); ++i) {
+    for (int i = 0; i < tableItems.size(); ++i) {
         if (tableItems.at(i)) {
             tableItems.at(i)->view = nullptr;
             delete tableItems.at(i);
@@ -797,7 +781,7 @@ void QTableModel::itemChanged(QTableWidgetItem *item, const QList<int> &roles)
 {
     if (!item)
         return;
-    if (item->flags() & ItemIsHeaderItem) {
+    if (item->d->headerItem) {
         int row = verticalHeaderItems.indexOf(item);
         if (row >= 0) {
             emit headerDataChanged(Qt::Vertical, row, row);
@@ -845,7 +829,7 @@ QMimeData *QTableModel::internalMimeData()  const
 QMimeData *QTableModel::mimeData(const QModelIndexList &indexes) const
 {
     QList<QTableWidgetItem*> items;
-    const int indexesCount = indexes.count();
+    const int indexesCount = indexes.size();
     items.reserve(indexesCount);
     for (int i = 0; i < indexesCount; ++i)
         items << item(indexes.at(i));
@@ -865,9 +849,12 @@ bool QTableModel::dropMimeData(const QMimeData *data, Qt::DropAction action,
     if (index.isValid()) {
         row = index.row();
         column = index.column();
-    }else if (row == -1 || column == -1) {  // The user dropped outside the table.
+    } else if (row == -1 || column == -1) { // The user dropped outside the table.
         row = rowCount();
         column = 0;
+    } else { // The user dropped between two rows
+        // This means inserting a row, which only makes sense at column 0
+        column = 0;
     }
 
     QTableWidget *view = qobject_cast<QTableWidget*>(QObject::parent());
@@ -1289,14 +1276,29 @@ void QTableWidgetItem::setFlags(Qt::ItemFlags aflags)
 */
 
 /*!
+    \if defined(qt7)
+
+    \fn Qt::Alignment QTableWidgetItem::textAlignment() const
+
+    Returns the text alignment for the list item.
+
+    \else
+
     \fn int QTableWidgetItem::textAlignment() const
 
     Returns the text alignment for the item's text.
 
+    \note This function returns an int for historical reasons. It will
+    be corrected to return Qt::Alignment in Qt 7.
+
     \sa Qt::Alignment
+
+    \endif
 */
 
 /*!
+    \obsolete [6.4] Use the overload that takes a Qt::Alignment argument.
+
     \fn void QTableWidgetItem::setTextAlignment(int alignment)
 
     Sets the text alignment for the item's text to the \a alignment
@@ -1305,6 +1307,20 @@ void QTableWidgetItem::setFlags(Qt::ItemFlags aflags)
     \sa Qt::Alignment
 */
 
+/*!
+    \since 6.4
+
+    \fn void QTableWidgetItem::setTextAlignment(Qt::Alignment alignment)
+
+    Sets the text alignment for the item's text to the \a alignment
+    specified.
+*/
+
+/*!
+  \fn void QTableWidgetItem::setTextAlignment(Qt::AlignmentFlag alignment)
+  \internal
+*/
+
 /*!
     Constructs a table item of the specified \a type that does not belong
     to any table.
@@ -1387,7 +1403,7 @@ void QTableWidgetItem::setData(int role, const QVariant &value)
 {
     bool found = false;
     role = (role == Qt::EditRole ? Qt::DisplayRole : role);
-    for (int i = 0; i < values.count(); ++i) {
+    for (int i = 0; i < values.size(); ++i) {
         if (values.at(i).role == role) {
             if (values[i].value == value)
                 return;
@@ -1534,7 +1550,7 @@ QTableWidgetItem &QTableWidgetItem::operator=(const QTableWidgetItem &other)
     \ingroup model-view
     \inmodule QtWidgets
 
-    \image windows-tableview.png
+    \image fusion-tableview.png
 
     Table widgets provide standard table display facilities for applications.
     The items in a QTableWidget are provided by QTableWidgetItem.
@@ -1599,28 +1615,41 @@ QTableWidgetItem &QTableWidgetItem::operator=(const QTableWidgetItem &other)
 void QTableWidgetPrivate::setup()
 {
     Q_Q(QTableWidget);
-    // view signals
-    QObject::connect(q, SIGNAL(pressed(QModelIndex)), q, SLOT(_q_emitItemPressed(QModelIndex)));
-    QObject::connect(q, SIGNAL(clicked(QModelIndex)), q, SLOT(_q_emitItemClicked(QModelIndex)));
-    QObject::connect(q, SIGNAL(doubleClicked(QModelIndex)),
-                     q, SLOT(_q_emitItemDoubleClicked(QModelIndex)));
-    QObject::connect(q, SIGNAL(activated(QModelIndex)), q, SLOT(_q_emitItemActivated(QModelIndex)));
-    QObject::connect(q, SIGNAL(entered(QModelIndex)), q, SLOT(_q_emitItemEntered(QModelIndex)));
-    // model signals
-    QObject::connect(model, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-                     q, SLOT(_q_emitItemChanged(QModelIndex)));
-    // selection signals
-    QObject::connect(q->selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),
-                     q, SLOT(_q_emitCurrentItemChanged(QModelIndex,QModelIndex)));
-    QObject::connect(q->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
-                     q, SIGNAL(itemSelectionChanged()));
-    // sorting
-    QObject::connect(model, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-                     q, SLOT(_q_dataChanged(QModelIndex,QModelIndex)));
-    QObject::connect(model, SIGNAL(columnsRemoved(QModelIndex,int,int)), q, SLOT(_q_sort()));
+    connections = {
+        // view signals
+        QObjectPrivate::connect(q, &QTableWidget::pressed,
+                                this, &QTableWidgetPrivate::emitItemPressed),
+        QObjectPrivate::connect(q, &QTableWidget::clicked,
+                                this, &QTableWidgetPrivate::emitItemClicked),
+        QObjectPrivate::connect(q, &QTableWidget::doubleClicked,
+                                this, &QTableWidgetPrivate::emitItemDoubleClicked),
+        QObjectPrivate::connect(q, &QTableWidget::activated,
+                                this, &QTableWidgetPrivate::emitItemActivated),
+        QObjectPrivate::connect(q, &QTableWidget::entered,
+                                this, &QTableWidgetPrivate::emitItemEntered),
+        // model signals
+        QObjectPrivate::connect(model, &QAbstractItemModel::dataChanged,
+                                this, &QTableWidgetPrivate::emitItemChanged),
+        // selection signals
+        QObjectPrivate::connect(q->selectionModel(), &QItemSelectionModel::currentChanged,
+                                this, &QTableWidgetPrivate::emitCurrentItemChanged),
+        QObject::connect(q->selectionModel(), &QItemSelectionModel::selectionChanged,
+                         q, &QTableWidget::itemSelectionChanged),
+        // sorting
+        QObjectPrivate::connect(model, &QAbstractItemModel::dataChanged,
+                                this, &QTableWidgetPrivate::dataChanged),
+        QObjectPrivate::connect(model, &QAbstractItemModel::columnsRemoved,
+                                this, &QTableWidgetPrivate::sort)
+    };
 }
 
-void QTableWidgetPrivate::_q_emitItemPressed(const QModelIndex &index)
+void QTableWidgetPrivate::clearConnections()
+{
+    for (const QMetaObject::Connection &connection : connections)
+        QObject::disconnect(connection);
+}
+
+void QTableWidgetPrivate::emitItemPressed(const QModelIndex &index)
 {
     Q_Q(QTableWidget);
     if (QTableWidgetItem *item = tableModel()->item(index))
@@ -1628,7 +1657,7 @@ void QTableWidgetPrivate::_q_emitItemPressed(const QModelIndex &index)
     emit q->cellPressed(index.row(), index.column());
 }
 
-void QTableWidgetPrivate::_q_emitItemClicked(const QModelIndex &index)
+void QTableWidgetPrivate::emitItemClicked(const QModelIndex &index)
 {
     Q_Q(QTableWidget);
     if (QTableWidgetItem *item = tableModel()->item(index))
@@ -1636,7 +1665,7 @@ void QTableWidgetPrivate::_q_emitItemClicked(const QModelIndex &index)
     emit q->cellClicked(index.row(), index.column());
 }
 
-void QTableWidgetPrivate::_q_emitItemDoubleClicked(const QModelIndex &index)
+void QTableWidgetPrivate::emitItemDoubleClicked(const QModelIndex &index)
 {
     Q_Q(QTableWidget);
     if (QTableWidgetItem *item = tableModel()->item(index))
@@ -1644,7 +1673,7 @@ void QTableWidgetPrivate::_q_emitItemDoubleClicked(const QModelIndex &index)
     emit q->cellDoubleClicked(index.row(), index.column());
 }
 
-void QTableWidgetPrivate::_q_emitItemActivated(const QModelIndex &index)
+void QTableWidgetPrivate::emitItemActivated(const QModelIndex &index)
 {
     Q_Q(QTableWidget);
     if (QTableWidgetItem *item = tableModel()->item(index))
@@ -1652,7 +1681,7 @@ void QTableWidgetPrivate::_q_emitItemActivated(const QModelIndex &index)
     emit q->cellActivated(index.row(), index.column());
 }
 
-void QTableWidgetPrivate::_q_emitItemEntered(const QModelIndex &index)
+void QTableWidgetPrivate::emitItemEntered(const QModelIndex &index)
 {
     Q_Q(QTableWidget);
     if (QTableWidgetItem *item = tableModel()->item(index))
@@ -1660,7 +1689,7 @@ void QTableWidgetPrivate::_q_emitItemEntered(const QModelIndex &index)
     emit q->cellEntered(index.row(), index.column());
 }
 
-void QTableWidgetPrivate::_q_emitItemChanged(const QModelIndex &index)
+void QTableWidgetPrivate::emitItemChanged(const QModelIndex &index)
 {
     Q_Q(QTableWidget);
     if (QTableWidgetItem *item = tableModel()->item(index))
@@ -1668,7 +1697,7 @@ void QTableWidgetPrivate::_q_emitItemChanged(const QModelIndex &index)
     emit q->cellChanged(index.row(), index.column());
 }
 
-void QTableWidgetPrivate::_q_emitCurrentItemChanged(const QModelIndex &current,
+void QTableWidgetPrivate::emitCurrentItemChanged(const QModelIndex &current,
                                                  const QModelIndex &previous)
 {
     Q_Q(QTableWidget);
@@ -1679,7 +1708,7 @@ void QTableWidgetPrivate::_q_emitCurrentItemChanged(const QModelIndex &current,
     emit q->currentCellChanged(current.row(), current.column(), previous.row(), previous.column());
 }
 
-void QTableWidgetPrivate::_q_sort()
+void QTableWidgetPrivate::sort()
 {
     if (sortingEnabled) {
         int column = horizontalHeader->sortIndicatorSection();
@@ -1688,8 +1717,8 @@ void QTableWidgetPrivate::_q_sort()
     }
 }
 
-void QTableWidgetPrivate::_q_dataChanged(const QModelIndex &topLeft,
-                                         const QModelIndex &bottomRight)
+void QTableWidgetPrivate::dataChanged(const QModelIndex &topLeft,
+                                      const QModelIndex &bottomRight)
 {
     if (sortingEnabled && topLeft.isValid() && bottomRight.isValid()) {
         int column = horizontalHeader->sortIndicatorSection();
@@ -1887,6 +1916,8 @@ QTableWidget::QTableWidget(int rows, int columns, QWidget *parent)
 */
 QTableWidget::~QTableWidget()
 {
+    Q_D(QTableWidget);
+    d->clearConnections();
 }
 
 /*!
@@ -2094,7 +2125,7 @@ void QTableWidget::setVerticalHeaderLabels(const QStringList &labels)
     Q_D(QTableWidget);
     QTableModel *model = d->tableModel();
     QTableWidgetItem *item = nullptr;
-    for (int i = 0; i < model->rowCount() && i < labels.count(); ++i) {
+    for (int i = 0; i < model->rowCount() && i < labels.size(); ++i) {
         item = model->verticalHeaderItem(i);
         if (!item) {
             item = model->createItem();
@@ -2112,7 +2143,7 @@ void QTableWidget::setHorizontalHeaderLabels(const QStringList &labels)
     Q_D(QTableWidget);
     QTableModel *model = d->tableModel();
     QTableWidgetItem *item = nullptr;
-    for (int i = 0; i < model->columnCount() && i < labels.count(); ++i) {
+    for (int i = 0; i < model->columnCount() && i < labels.size(); ++i) {
         item = model->horizontalHeaderItem(i);
         if (!item) {
             item = model->createItem();
@@ -2351,7 +2382,7 @@ QList<QTableWidgetSelectionRange> QTableWidget::selectedRanges() const
 {
     const QList<QItemSelectionRange> ranges = selectionModel()->selection();
     QList<QTableWidgetSelectionRange> result;
-    const int rangesCount = ranges.count();
+    const int rangesCount = ranges.size();
     result.reserve(rangesCount);
     for (int i = 0; i < rangesCount; ++i)
         result.append({ranges.at(i).top(),
@@ -2589,7 +2620,7 @@ QMimeData *QTableWidget::mimeData(const QList<QTableWidgetItem *> &items) const
 
     // if non empty, it's called from the model's own mimeData
     if (cachedIndexes.isEmpty()) {
-        cachedIndexes.reserve(items.count());
+        cachedIndexes.reserve(items.size());
         for (QTableWidgetItem *item : items)
             cachedIndexes << indexFromItem(item);
 
@@ -2694,7 +2725,8 @@ void QTableWidget::dropEvent(QDropEvent *event) {
         int col = -1;
         int row = -1;
         // check whether a subclass has already accepted the event, ie. moved the data
-        if (!event->isAccepted() && d->dropOn(event, &row, &col, &topIndex)) {
+        if (!event->isAccepted() && d->dropOn(event, &row, &col, &topIndex) && row == -1 && col == -1) {
+            // Drop onto item
             const QModelIndexList indexes = selectedIndexes();
             int top = INT_MAX;
             int left = INT_MAX;
@@ -2702,9 +2734,8 @@ void QTableWidget::dropEvent(QDropEvent *event) {
                 top = qMin(index.row(), top);
                 left = qMin(index.column(), left);
             }
-
             QList<QTableWidgetItem *> taken;
-            const int indexesCount = indexes.count();
+            const int indexesCount = indexes.size();
             taken.reserve(indexesCount);
             for (const auto &index : indexes)
                 taken.append(takeItem(index.row(), index.column()));
@@ -2717,7 +2748,7 @@ void QTableWidget::dropEvent(QDropEvent *event) {
 
             event->accept();
         }
-        // either we or a subclass accepted the move event, so assume that the data was
+        // either we or a subclass accepted the drop event, so assume that the data was
         // moved and that QAbstractItemView shouldn't remove the source when QDrag::exec returns
         if (event->isAccepted())
             d->dropEventMoved = true;
