@@ -189,7 +189,7 @@ void QCommandLinkButtonPrivate::init()
     q->setIconSize(QSize(20, 20));
     QStyleOptionButton opt;
     q->initStyleOption(&opt);
-    q->setIcon(q->style()->standardIcon(QStyle::SP_CommandLink, &opt));
+    q->setIcon(q->style()->standardIcon(QStyle::SP_CommandLink, &opt, q));
 }
 
 // Calculates the height of the description text based on widget width
@@ -276,6 +276,12 @@ QCommandLinkButton::~QCommandLinkButton()
 /*! \reimp */
 bool QCommandLinkButton::event(QEvent *e)
 {
+    if (e->type() == QEvent::StyleChange) {
+        QStyleOptionButton opt;
+        initStyleOption(&opt);
+        setIcon(style()->standardIcon(QStyle::SP_CommandLink, &opt, this));
+    }
+
     return QPushButton::event(e);
 }
 
@@ -315,14 +321,12 @@ void QCommandLinkButton::paintEvent(QPaintEvent *)
 {
     Q_D(QCommandLinkButton);
     QStylePainter p(this);
-    p.save();
 
     QStyleOptionButton option;
     initStyleOption(&option);
 
     option.text = QString();
     option.icon = QIcon(); //we draw this ourselves
-    QSize pixmapSize = icon().actualSize(iconSize());
 
     const int vOffset = isDown()
         ? style()->pixelMetric(QStyle::PM_ButtonShiftVertical, &option, this) : 0;
@@ -331,10 +335,14 @@ void QCommandLinkButton::paintEvent(QPaintEvent *)
 
     //Draw icon
     p.drawControl(QStyle::CE_PushButton, option);
-    if (!icon().isNull())
-        p.drawPixmap(d->leftMargin() + hOffset, d->topMargin() + vOffset,
-        icon().pixmap(pixmapSize, isEnabled() ? QIcon::Normal : QIcon::Disabled,
-                                  isChecked() ? QIcon::On : QIcon::Off));
+    if (!icon().isNull()) {
+        const auto size = icon().actualSize(iconSize());
+        const auto mode = isEnabled() ? QIcon::Normal : QIcon::Disabled;
+        const auto state = isChecked() ? QIcon::On : QIcon::Off;
+        const auto rect = QRect(d->leftMargin() + hOffset, d->topMargin() + vOffset,
+                                size.width(), size.height());
+        icon().paint(&p, rect, Qt::AlignCenter, mode, state);
+    }
 
     //Draw title
     QColor textColor = palette().buttonText().color();
@@ -360,7 +368,6 @@ void QCommandLinkButton::paintEvent(QPaintEvent *)
     p.setFont(d->descriptionFont());
     p.drawItemText(d->descriptionRect().translated(hOffset, vOffset), textflags,
                     option.palette, isEnabled(), description(), QPalette::ButtonText);
-    p.restore();
 }
 
 void QCommandLinkButton::setDescription(const QString &description)
