@@ -1,6 +1,6 @@
 // Copyright (C) 2020 The Qt Company Ltd.
 // Copyright (C) 2021 Intel Corporation.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 #include <QSignalSpy>
@@ -12,10 +12,14 @@
 #include <QScopeGuard>
 #include "theplugin/plugininterface.h"
 
+#include <QtCore/private/qsimd_p.h>
+
 #if defined(QT_BUILD_INTERNAL) && defined(Q_OF_MACH_O)
 #  include <QtCore/private/qmachparser_p.h>
 #endif
 
+using namespace Qt::StringLiterals;
+
 // Helper macros to let us know if some suffixes are valid
 #define bundle_VALID    false
 #define dylib_VALID     false
@@ -31,10 +35,10 @@
 # define bundle_VALID   true
 # define dylib_VALID    true
 # define so_VALID       true
-# ifdef QT_NO_DEBUG
-#  define SUFFIX         ".dylib"
-# else
+# if QT_CONFIG(debug) && !QT_CONFIG(framework)
 #  define SUFFIX         "_debug.dylib"
+# else
+#  define SUFFIX         ".dylib"
 # endif
 # define PREFIX         "lib"
 
@@ -128,7 +132,7 @@ static std::unique_ptr<QTemporaryFile> patchElf(const QString &source, ElfPatche
         const char *basename = QTest::currentDataTag();
         if (!basename)
             basename = QTest::currentTestFunction();
-        tmplib.reset(new QTemporaryFile(basename + QString(".XXXXXX" SUFFIX)));
+        tmplib.reset(new QTemporaryFile(QDir::currentPath() + u'/' + basename + u".XXXXXX" SUFFIX ""_s));
         QVERIFY2(tmplib->open(), qPrintable(tmplib->errorString()));
 
         // sanity-check
@@ -197,6 +201,7 @@ private slots:
     void loadCorruptElfOldPlugin();
 #  endif
 #endif
+    void archSpecificVersion();
     void loadMachO_data();
     void loadMachO();
     void relativePath();
@@ -206,6 +211,7 @@ private slots:
     void preloadedPlugin_data();
     void preloadedPlugin();
     void staticPlugins();
+    void reregisteredStaticPlugins();
 };
 
 Q_IMPORT_PLUGIN(StaticPlugin)
@@ -277,7 +283,9 @@ void tst_QPluginLoader::errorString()
     QVERIFY(!unloaded);
     }
 
-#if !defined(Q_OS_WIN) && !defined(Q_OS_MAC) && !defined(Q_OS_HPUX)
+// A bug in QNX causes the test to crash on exit after attempting to load
+// a shared library with undefined symbols (tracked as QTBUG-114682).
+#if !defined(Q_OS_WIN) && !defined(Q_OS_DARWIN) && !defined(Q_OS_HPUX) && !defined(Q_OS_QNX)
     {
     QPluginLoader loader( sys_qualifiedLibraryName("almostplugin"));     //a plugin with unresolved symbols
     loader.setLoadHints(QLibrary::ResolveAllSymbolsHint);
@@ -337,16 +345,34 @@ void tst_QPluginLoader::loadHints()
     QCOMPARE(loader.loadHints(), QLibrary::PreventUnloadHint);   //Do not crash
     loader.setLoadHints(QLibrary::ResolveAllSymbolsHint);
     QCOMPARE(loader.loadHints(), QLibrary::ResolveAllSymbolsHint);
+    // We can clear load hints when file name is not set.
+    loader.setLoadHints(QLibrary::LoadHints{});
+    QCOMPARE(loader.loadHints(), QLibrary::LoadHints{});
+    // Set the hints again
+    loader.setLoadHints(QLibrary::ResolveAllSymbolsHint);
+    QCOMPARE(loader.loadHints(), QLibrary::ResolveAllSymbolsHint);
     loader.setFileName( sys_qualifiedLibraryName("theplugin"));     //a plugin
     QCOMPARE(loader.loadHints(), QLibrary::ResolveAllSymbolsHint);
 
+    QPluginLoader loader4;
+    QCOMPARE(loader4.loadHints(), QLibrary::PreventUnloadHint);
+    loader4.setLoadHints(QLibrary::LoadHints{});
+    QCOMPARE(loader4.loadHints(), QLibrary::LoadHints{});
+    loader4.setFileName(sys_qualifiedLibraryName("theplugin"));
+    // Hints are merged with hints from the previous loader.
+    QCOMPARE(loader4.loadHints(), QLibrary::ResolveAllSymbolsHint);
+    // We cannot clear load hints after associating the loader with a file.
+    loader.setLoadHints(QLibrary::LoadHints{});
+    QCOMPARE(loader.loadHints(), QLibrary::ResolveAllSymbolsHint);
+
     QPluginLoader loader2;
     QCOMPARE(loader2.loadHints(), QLibrary::PreventUnloadHint);
     loader2.setFileName(sys_qualifiedLibraryName("theplugin"));
-    QCOMPARE(loader2.loadHints(), QLibrary::PreventUnloadHint);
+    // Hints are merged with hints from previous loaders.
+    QCOMPARE(loader2.loadHints(), QLibrary::PreventUnloadHint | QLibrary::ResolveAllSymbolsHint);
 
     QPluginLoader loader3(sys_qualifiedLibraryName("theplugin"));
-    QCOMPARE(loader3.loadHints(), QLibrary::PreventUnloadHint);
+    QCOMPARE(loader3.loadHints(), QLibrary::PreventUnloadHint | QLibrary::ResolveAllSymbolsHint);
 }
 
 void tst_QPluginLoader::deleteinstanceOnUnload()
@@ -451,7 +477,7 @@ static void loadCorruptElfCommonRows()
         memcpy(h, &o, sizeof(o));
     });
     newRow("invalid-word-size", "file is for a different word size", [](H h) {
-        h->e_ident[EI_CLASS] = ELFCLASSNONE;;
+        h->e_ident[EI_CLASS] = ELFCLASSNONE;
     });
     newRow("unknown-word-size", "file is for a different word size", [](H h) {
         h->e_ident[EI_CLASS] |= 0x40;
@@ -825,6 +851,28 @@ void tst_QPluginLoader::loadCorruptElfOldPlugin()
 #  endif // Qt 7
 #endif // Q_OF_ELF
 
+void tst_QPluginLoader::archSpecificVersion()
+{
+#if !defined(QT_SHARED)
+    QSKIP("This test requires Qt to create shared libraries.");
+#endif
+    QPluginLoader loader(sys_qualifiedLibraryName("theplugin"));
+    QVERIFY2(loader.load(), qPrintable(loader.errorString()));
+
+    QString expectedArch;
+#if defined(Q_PROCESSOR_X86_64) && defined(Q_OS_UNIX) && !defined(Q_OS_DARWIN)
+    // On Unix systems (other than Darwin, which has fat binaries),
+    // QPluginLoader will load a separate file for x86-64-v3 systems.
+    if (qCpuHasFeature(ArchHaswell))
+        expectedArch = "x86-64-v3";
+#endif
+
+    PluginInterface* theplugin = qobject_cast<PluginInterface*>(loader.instance());
+    QVERIFY(theplugin);
+    QCOMPARE(theplugin->architectureName(), expectedArch);
+    QVERIFY(loader.unload());
+}
+
 void tst_QPluginLoader::loadMachO_data()
 {
 #if defined(QT_BUILD_INTERNAL) && defined(Q_OF_MACH_O)
@@ -837,26 +885,23 @@ void tst_QPluginLoader::loadMachO_data()
 
 #  ifdef Q_PROCESSOR_X86_64
     QTest::newRow("machtest/good.x86_64.dylib") << true;
-    QTest::newRow("machtest/good.i386.dylib") << false;
+    QTest::newRow("machtest/good.arm64.dylib") << false;
     QTest::newRow("machtest/good.fat.no-x86_64.dylib") << false;
-    QTest::newRow("machtest/good.fat.no-i386.dylib") << true;
-#  elif defined(Q_PROCESSOR_X86_32)
-    QTest::newRow("machtest/good.i386.dylib") << true;
+    QTest::newRow("machtest/good.fat.no-arm64.dylib") << true;
+#  elif defined(Q_PROCESSOR_ARM)
+    QTest::newRow("machtest/good.arm64.dylib") << true;
     QTest::newRow("machtest/good.x86_64.dylib") << false;
-    QTest::newRow("machtest/good.fat.no-i386.dylib") << false;
+    QTest::newRow("machtest/good.fat.no-arm64.dylib") << false;
     QTest::newRow("machtest/good.fat.no-x86_64.dylib") << true;
 #  endif
-#  ifndef Q_PROCESSOR_POWER_64
-    QTest::newRow("machtest/good.ppc64.dylib") << false;
-#  endif
 
     QTest::newRow("machtest/good.fat.all.dylib") << true;
     QTest::newRow("machtest/good.fat.stub-x86_64.dylib") << false;
-    QTest::newRow("machtest/good.fat.stub-i386.dylib") << false;
+    QTest::newRow("machtest/good.fat.stub-arm64.dylib") << false;
 
     QDir d(QFINDTESTDATA("machtest"));
-    QStringList badlist = d.entryList(QStringList() << "bad*.dylib");
-    foreach (const QString &bad, badlist)
+    const QStringList badlist = d.entryList(QStringList() << "bad*.dylib");
+    for (const QString &bad : badlist)
         QTest::newRow(qPrintable("machtest/" + bad)) << false;
 #endif
 }
@@ -880,12 +925,7 @@ void tst_QPluginLoader::loadMachO()
     }
 
     QVERIFY(r.pos > 0);
-    QVERIFY(size_t(r.length) >= sizeof(void*));
     QVERIFY(r.pos + r.length < data.size());
-    QCOMPARE(r.pos & (sizeof(void*) - 1), 0UL);
-
-    void *value = *(void**)(data.constData() + r.pos);
-    QCOMPARE(value, sizeof(void*) > 4 ? (void*)(0xc0ffeec0ffeeL) : (void*)0xc0ffee);
 
     // now that we know it's valid, let's try to make it invalid
     ulong offeredlen = r.pos;
@@ -1080,19 +1120,18 @@ void tst_QPluginLoader::staticPlugins()
     const QObjectList instances = QPluginLoader::staticInstances();
     QVERIFY(instances.size());
 
-    bool found = false;
-    for (QObject *obj : instances) {
-        found = obj->metaObject()->className() == QLatin1String("StaticPlugin");
-        if (found)
-            break;
-    }
-    QVERIFY(found);
+    // ensure the our plugin only shows up once
+    int foundCount = std::count_if(instances.begin(), instances.end(), [](QObject *obj) {
+            return obj->metaObject()->className() == QLatin1String("StaticPlugin");
+    });
+    QCOMPARE(foundCount, 1);
 
     const auto plugins = QPluginLoader::staticPlugins();
     QCOMPARE(plugins.size(), instances.size());
 
     // find the metadata
     QJsonObject metaData;
+    bool found = false;
     for (const auto &p : plugins) {
         metaData = p.metaData();
         found = metaData.value("className").toString() == QLatin1String("StaticPlugin");
@@ -1108,6 +1147,18 @@ void tst_QPluginLoader::staticPlugins()
     QCOMPARE(metaData.value("URI").toString(), "qt.test.pluginloader.staticplugin");
 }
 
+void tst_QPluginLoader::reregisteredStaticPlugins()
+{
+    // the Q_IMPORT_PLUGIN macro will have already done this
+    qRegisterStaticPluginFunction(qt_static_plugin_StaticPlugin());
+    staticPlugins();
+    if (QTest::currentTestFailed())
+        return;
+
+    qRegisterStaticPluginFunction(qt_static_plugin_StaticPlugin());
+    staticPlugins();
+}
+
 
 QTEST_MAIN(tst_QPluginLoader)
 #include "tst_qpluginloader.moc"
