@@ -1,5 +1,7 @@
 // Copyright (C) 2020 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com, author Marc Mutz <marc.mutz@kdab.com>
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+
+#include "arrays_of_unknown_bounds.h"
 
 #include <QStringView>
 #include <QStringTokenizer>
@@ -7,6 +9,11 @@
 #include <QChar>
 #include <QVarLengthArray>
 #include <QList>
+#if QT_CONFIG(cpp_winrt)
+#  include <private/qt_winrtbase_p.h>
+#endif
+#include <private/qxmlstream_p.h>
+
 
 #include <QTest>
 
@@ -40,6 +47,9 @@ static_assert(!CanConvert<QByteArray>::value);
 static_assert(!CanConvert<QChar>::value);
 
 static_assert(CanConvert<QChar[123]>::value);
+#ifndef Q_OS_INTEGRITY // ¯\_(ツ)_/¯
+static_assert(CanConvert<QChar[]>::value);
+#endif
 
 static_assert(CanConvert<      QString >::value);
 static_assert(CanConvert<const QString >::value);
@@ -53,6 +63,9 @@ static_assert(CanConvert<const QString&>::value);
 static_assert(!CanConvert<ushort>::value);
 
 static_assert(CanConvert<ushort[123]>::value);
+#ifndef Q_OS_INTEGRITY // ¯\_(ツ)_/¯
+static_assert(CanConvert<ushort[]>::value);
+#endif
 
 static_assert(CanConvert<      ushort*>::value);
 static_assert(CanConvert<const ushort*>::value);
@@ -70,6 +83,11 @@ static_assert(!CanConvert<std::list<ushort>>::value);
 
 static_assert(!CanConvert<char16_t>::value);
 
+static_assert(CanConvert<char16_t[123]>::value);
+#ifndef Q_OS_INTEGRITY // ¯\_(ツ)_/¯
+static_assert(CanConvert<char16_t[]>::value);
+#endif
+
 static_assert(CanConvert<      char16_t*>::value);
 static_assert(CanConvert<const char16_t*>::value);
 
@@ -90,6 +108,8 @@ static_assert(CanConvert<std::array<char16_t, 123>>::value);
 static_assert(!CanConvert<std::deque<char16_t>>::value);
 static_assert(!CanConvert<std::list<char16_t>>::value);
 
+static_assert(CanConvert<QtPrivate::XmlStringRef>::value);
+
 //
 // wchar_t
 //
@@ -104,6 +124,11 @@ constexpr bool CanConvertFromWCharT =
 
 static_assert(!CanConvert<wchar_t>::value);
 
+static_assert(CanConvert<wchar_t[123]>::value == CanConvertFromWCharT);
+#ifndef Q_OS_INTEGRITY // ¯\_(ツ)_/¯
+static_assert(CanConvert<wchar_t[]>::value    == CanConvertFromWCharT);
+#endif
+
 static_assert(CanConvert<      wchar_t*>::value == CanConvertFromWCharT);
 static_assert(CanConvert<const wchar_t*>::value == CanConvertFromWCharT);
 
@@ -124,6 +149,19 @@ static_assert(CanConvert<std::array<wchar_t, 123>>::value == CanConvertFromWChar
 static_assert(!CanConvert<std::deque<wchar_t>>::value);
 static_assert(!CanConvert<std::list<wchar_t>>::value);
 
+#if QT_CONFIG(cpp_winrt)
+
+//
+// winrt::hstring (QTBUG-111886)
+//
+
+static_assert(CanConvert<      winrt::hstring >::value);
+static_assert(CanConvert<const winrt::hstring >::value);
+static_assert(CanConvert<      winrt::hstring&>::value);
+static_assert(CanConvert<const winrt::hstring&>::value);
+
+#endif // QT_CONFIG(cpp_winrt)
+
 class tst_QStringView : public QObject
 {
     Q_OBJECT
@@ -165,6 +203,16 @@ private Q_SLOTS:
 #endif
     }
 
+    void fromChar16TArrayWithUnknownSize() { from_u16array_of_unknown_size<QStringView>(); }
+    void fromWCharTArrayWithUnknownSize()
+    {
+#ifdef Q_OS_WIN
+        from_warray_of_unknown_size<QStringView>();
+#else
+        QSKIP("This is a Windows-only test");
+#endif
+    }
+
     void fromQCharRange() const
     {
         const QChar str[] = { 'H', 'e', 'l', 'l', 'o', ',', ' ', 'W', 'o', 'r', 'l', 'd', '!' };
@@ -238,6 +286,8 @@ private Q_SLOTS:
     void tokenize_data() const;
     void tokenize() const;
 
+    void std_stringview_conversion();
+
 private:
     template <typename String>
     void conversion_tests(String arg) const;
@@ -337,6 +387,30 @@ void tst_QStringView::constExpr() const
         static_assert(sv3.isEmpty());
         static_assert(sv3.size() == 0);
     }
+#if !defined(Q_CC_GNU_ONLY) || !defined(QT_SANITIZE_UNDEFINED)
+    // Below checks are disabled because of a compilation issue with GCC and
+    // -fsanitize=undefined. See https://gcc.gnu.org/bugzilla/show_bug.cgi?id=71962.
+    {
+        static constexpr char16_t hello[] = u"Hello";
+        constexpr QStringView sv(hello);
+        static_assert(sv.size() == 5);
+        static_assert(!sv.empty());
+        static_assert(!sv.isEmpty());
+        static_assert(!sv.isNull());
+        static_assert(*sv.utf16() == 'H');
+        static_assert(sv[0]      == QLatin1Char('H'));
+        static_assert(sv.at(0)   == QLatin1Char('H'));
+        static_assert(sv.front() == QLatin1Char('H'));
+        static_assert(sv.first() == QLatin1Char('H'));
+        static_assert(sv[4]      == QLatin1Char('o'));
+        static_assert(sv.at(4)   == QLatin1Char('o'));
+        static_assert(sv.back()  == QLatin1Char('o'));
+        static_assert(sv.last()  == QLatin1Char('o'));
+
+        constexpr auto sv2 = QStringView::fromArray(hello);
+        QCOMPARE_EQ(sv, sv2.chopped(1));
+    }
+#endif // -fsanitize=undefined
 }
 
 void tst_QStringView::basics() const
@@ -408,7 +482,7 @@ void tst_QStringView::fromArray() const
 {
     static constexpr char16_t hello[] = u"Hello\0abc\0\0.";
 
-    constexpr QStringView sv = QStringView::fromArray(hello);
+    const QStringView sv = QStringView::fromArray(hello);
     QCOMPARE(sv.size(), 13);
     QVERIFY(!sv.empty());
     QVERIFY(!sv.isEmpty());
@@ -447,6 +521,10 @@ void tst_QStringView::at() const
 
 void tst_QStringView::arg() const
 {
+    // nullness checks
+    QCOMPARE(QStringView().arg(QStringView()), "");
+    QCOMPARE(QStringView(u"%1").arg(QStringView()), "");
+
 #define CHECK1(pattern, arg1, expected) \
     do { \
         auto p = QStringView(u"" pattern); \
@@ -819,6 +897,11 @@ static void test(QString) = delete;
 static void test(QStringView) {}
 }
 
+extern const QChar qcharArrayOfUnknownSize[];
+extern const char16_t char16ArrayOfUnknownSize[];
+[[maybe_unused]]
+extern const wchar_t wchartArrayOfUnknownSize[];
+
 // Compile-time only test: overload resolution prefers QStringView over QString
 void tst_QStringView::overloadResolution()
 {
@@ -827,6 +910,7 @@ void tst_QStringView::overloadResolution()
         QStringViewOverloadResolution::test(qcharArray);
         QChar *qcharPointer = qcharArray;
         QStringViewOverloadResolution::test(qcharPointer);
+        QStringViewOverloadResolution::test(qcharArrayOfUnknownSize);
     }
 
     {
@@ -841,6 +925,7 @@ void tst_QStringView::overloadResolution()
         wchar_t wchartArray[42] = {};
         QStringViewOverloadResolution::test(wchartArray);
         QStringViewOverloadResolution::test(L"test");
+        QStringViewOverloadResolution::test(wchartArrayOfUnknownSize);
     }
 #endif
 
@@ -849,6 +934,7 @@ void tst_QStringView::overloadResolution()
         QStringViewOverloadResolution::test(char16Array);
         char16_t *char16Pointer = char16Array;
         QStringViewOverloadResolution::test(char16Pointer);
+        QStringViewOverloadResolution::test(char16ArrayOfUnknownSize);
     }
 
     {
@@ -859,5 +945,34 @@ void tst_QStringView::overloadResolution()
     }
 }
 
+const QChar qcharArrayOfUnknownSize[] = {u'a', u'b', u'c', u'\0', u'd', u'e', u'f'};
+const char16_t char16ArrayOfUnknownSize[] = u"abc\0def";
+const wchar_t wchartArrayOfUnknownSize[] = L"abc\0def";
+
+void tst_QStringView::std_stringview_conversion()
+{
+    static_assert(std::is_convertible_v<QStringView, std::u16string_view>);
+
+    QStringView s;
+    std::u16string_view sv(s);
+    QCOMPARE(sv, std::u16string_view());
+
+    s = u"";
+    sv = s;
+    QCOMPARE(s.size(), 0);
+    QCOMPARE(sv.size(), size_t(0));
+    QCOMPARE(sv, std::u16string_view());
+
+    s = u"Hello";
+    sv = s;
+    QCOMPARE(sv, std::u16string_view(u"Hello"));
+
+    s = QStringView::fromArray(u"Hello\0world");
+    sv = s;
+    QCOMPARE(s.size(), 12);
+    QCOMPARE(sv.size(), size_t(12));
+    QCOMPARE(sv, std::u16string_view(u"Hello\0world\0", 12));
+}
+
 QTEST_APPLESS_MAIN(tst_QStringView)
 #include "tst_qstringview.moc"
