@@ -1,16 +1,22 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
 #include <QStandardItemModel>
 #include <QTreeView>
+#include <QMap>
 #include <QSignalSpy>
 #include <QAbstractItemModelTester>
 
 #include <private/qabstractitemmodel_p.h>
+#include <private/qpropertytesthelper_p.h>
 #include <private/qtreeview_p.h>
 
+#include <algorithm>
+
+using namespace Qt::StringLiterals;
+
 class tst_QStandardItemModel : public QObject
 {
     Q_OBJECT
@@ -62,6 +68,7 @@ private slots:
     void insertRow();
     void insertRows();
     void insertRowsItems();
+    void insertRowsWithChildItems();
     void insertRowInHierarcy();
     void insertColumn_data();
     void insertColumn();
@@ -87,6 +94,7 @@ private slots:
     void indexFromItem();
     void itemFromIndex();
     void getSetItemPrototype();
+    void getSetItemData_data();
     void getSetItemData();
     void setHeaderLabels_data();
     void setHeaderLabels();
@@ -113,6 +121,7 @@ private slots:
     void taskQTBUG_45114_setItemData();
     void setItemPersistentIndex();
     void signalsOnTakeItem();
+    void takeChild();
     void createPersistentOnLayoutAboutToBeChanged();
 private:
     QStandardItemModel *m_model = nullptr;
@@ -288,6 +297,39 @@ void tst_QStandardItemModel::insertRowsItems()
     }
 }
 
+void tst_QStandardItemModel::insertRowsWithChildItems()
+{
+    QStandardItemModel model;
+    auto *top = new QStandardItem("top");
+    auto *mid = new QStandardItem("mid");
+    auto *btm = new QStandardItem("bottom");
+
+    model.appendRow(top);
+    mid->appendRow(btm);
+
+    QCOMPARE(top->model(), &model);
+    QCOMPARE(mid->model(), nullptr);
+    QCOMPARE(btm->model(), nullptr);
+
+    top->appendRow(mid);
+    QCOMPARE(top->model(), &model);
+    QCOMPARE(mid->model(), &model);
+    QCOMPARE(btm->model(), &model);
+
+    auto mid2 = top->takeChild(0);
+    top->removeRow(0);
+    QCOMPARE(mid, mid2);
+    QCOMPARE(top->model(), &model);
+    QCOMPARE(mid->model(), nullptr);
+    QCOMPARE(btm->model(), nullptr);
+
+    top->appendRows({mid}); // other codepath than appendRow() above
+    QCOMPARE(top->model(), &model);
+    QCOMPARE(mid->model(), &model);
+    QCOMPARE(btm->model(), &model);
+}
+
+
 void tst_QStandardItemModel::insertRowInHierarcy()
 {
     QVERIFY(m_model->insertRows(0, 1, QModelIndex()));
@@ -720,7 +762,7 @@ void tst_QStandardItemModel::data()
     const QMap<int, QVariant> itmData = m_model->itemData(m_model->index(0, 0));
     QCOMPARE(itmData.value(Qt::DisplayRole), QLatin1String("initialitem"));
     QCOMPARE(itmData.value(Qt::ToolTipRole), QLatin1String("tooltip"));
-    QVERIFY(!itmData.contains(Qt::UserRole - 1));
+    QVERIFY(!itmData.contains(Qt::UserRole - 1)); // Qt::UserRole - 1 is used to store flags
     QVERIFY(m_model->itemData(QModelIndex()).isEmpty());
 }
 
@@ -896,6 +938,9 @@ void tst_QStandardItemModel::sortRoleBindings()
     sortRoleObserver.setBinding([&] { return model.sortRole(); });
     model.setSortRole(Qt::EditRole);
     QCOMPARE(sortRoleObserver, Qt::EditRole);
+
+    QTestPrivate::testReadWritePropertyBasics(model, static_cast<int>(Qt::DisplayRole),
+                                              static_cast<int>(Qt::EditRole), "sortRole");
 }
 
 void tst_QStandardItemModel::findItems()
@@ -1025,33 +1070,49 @@ void tst_QStandardItemModel::getSetItemPrototype()
     QCOMPARE(model.itemPrototype(), nullptr);
 }
 
+using RoleMap = QMap<int, QVariant>;
+using RoleList = QList<int>;
+
+static RoleMap getSetItemDataRoleMap(int textRole)
+{
+    return {{textRole, "text"_L1},
+            {Qt::StatusTipRole, "statusTip"_L1},
+            {Qt::ToolTipRole, "toolTip"_L1},
+            {Qt::WhatsThisRole, "whatsThis"_L1},
+            {Qt::SizeHintRole, QSize{64, 48}},
+            {Qt::FontRole, QFont{}},
+            {Qt::TextAlignmentRole, int(Qt::AlignLeft|Qt::AlignVCenter)},
+            {Qt::BackgroundRole, QColor(Qt::blue)},
+            {Qt::ForegroundRole, QColor(Qt::green)},
+            {Qt::CheckStateRole, int(Qt::PartiallyChecked)},
+            {Qt::AccessibleTextRole, "accessibleText"_L1},
+            {Qt::AccessibleDescriptionRole, "accessibleDescription"_L1}};
+}
+
+void tst_QStandardItemModel::getSetItemData_data()
+{
+    QTest::addColumn<RoleMap>("itemData");
+    QTest::addColumn<RoleMap>("expectedItemData");
+    QTest::addColumn<RoleList>("expectedRoles");
+
+    // QTBUG-112326: verify that text data set using Qt::EditRole is mapped to
+    // Qt::DisplayRole and both roles are in the changed signal
+    const RoleMap expectedItemData =  getSetItemDataRoleMap(Qt::DisplayRole);
+    RoleList expectedRoles = expectedItemData.keys() << Qt::EditRole;
+    std::sort(expectedRoles.begin(), expectedRoles.end());
+
+    QTest::newRow("DisplayRole") << expectedItemData
+                                 << expectedItemData << expectedRoles;
+
+    QTest::newRow("EditRole") << getSetItemDataRoleMap(Qt::EditRole)
+                              << expectedItemData << expectedRoles;
+}
+
 void tst_QStandardItemModel::getSetItemData()
 {
-    QMap<int, QVariant> roles;
-    QLatin1String text("text");
-    roles.insert(Qt::DisplayRole, text);
-    QLatin1String statusTip("statusTip");
-    roles.insert(Qt::StatusTipRole, statusTip);
-    QLatin1String toolTip("toolTip");
-    roles.insert(Qt::ToolTipRole, toolTip);
-    QLatin1String whatsThis("whatsThis");
-    roles.insert(Qt::WhatsThisRole, whatsThis);
-    QSize sizeHint(64, 48);
-    roles.insert(Qt::SizeHintRole, sizeHint);
-    QFont font;
-    roles.insert(Qt::FontRole, font);
-    Qt::Alignment textAlignment(Qt::AlignLeft|Qt::AlignVCenter);
-    roles.insert(Qt::TextAlignmentRole, int(textAlignment));
-    QColor backgroundColor(Qt::blue);
-    roles.insert(Qt::BackgroundRole, backgroundColor);
-    QColor textColor(Qt::green);
-    roles.insert(Qt::ForegroundRole, textColor);
-    Qt::CheckState checkState(Qt::PartiallyChecked);
-    roles.insert(Qt::CheckStateRole, int(checkState));
-    QLatin1String accessibleText("accessibleText");
-    roles.insert(Qt::AccessibleTextRole, accessibleText);
-    QLatin1String accessibleDescription("accessibleDescription");
-    roles.insert(Qt::AccessibleDescriptionRole, accessibleDescription);
+    QFETCH(RoleMap, itemData);
+    QFETCH(RoleMap, expectedItemData);
+    QFETCH(RoleList, expectedRoles);
 
     QStandardItemModel model;
     model.insertRows(0, 1);
@@ -1060,11 +1121,17 @@ void tst_QStandardItemModel::getSetItemData()
 
     QSignalSpy modelDataChangedSpy(
          &model, &QStandardItemModel::dataChanged);
-    QVERIFY(model.setItemData(idx, roles));
+    QVERIFY(model.setItemData(idx, itemData));
     QCOMPARE(modelDataChangedSpy.size(), 1);
-    QVERIFY(model.setItemData(idx, roles));
+    const QVariantList &args = modelDataChangedSpy.constFirst();
+    QCOMPARE(args.size(), 3);
+    auto roleList = args.at(2).value<QList<int> >();
+    std::sort(roleList.begin(), roleList.end());
+    QCOMPARE(roleList, expectedRoles);
+
+    QVERIFY(model.setItemData(idx, itemData));
     QCOMPARE(modelDataChangedSpy.size(), 1); //it was already changed once
-    QCOMPARE(model.itemData(idx), roles);
+    QCOMPARE(model.itemData(idx), expectedItemData);
 }
 
 void tst_QStandardItemModel::setHeaderLabels_data()
@@ -1581,15 +1648,21 @@ void tst_QStandardItemModel::removeRowsAndColumns()
 
     QList<QStandardItem *> row_taken = model.takeRow(6);
     QCOMPARE(row_taken.size(), col_list.size());
-    for (int c = 0; c < col_list.size(); c++)
-        QCOMPARE(row_taken[c]->text() , row_list[6] + QLatin1Char('x') + col_list[c]);
+    for (qsizetype c = 0; c < row_taken.size(); c++) {
+        auto item = row_taken.at(c);
+        QCOMPARE(item->text() , row_list[6] + QLatin1Char('x') + col_list[c]);
+        delete item;
+    }
     row_list.remove(6);
     VERIFY_MODEL
 
     QList<QStandardItem *> col_taken = model.takeColumn(10);
     QCOMPARE(col_taken.size(), row_list.size());
-    for (int r = 0; r < row_list.size(); r++)
-        QCOMPARE(col_taken[r]->text() , row_list[r] + QLatin1Char('x') + col_list[10]);
+    for (qsizetype r = 0; r < col_taken.size(); r++) {
+        auto item = col_taken.at(r);
+        QCOMPARE(item->text() , row_list[r] + QLatin1Char('x') + col_list[10]);
+        delete item;
+    }
     col_list.remove(10);
     VERIFY_MODEL
 }
@@ -1759,6 +1832,7 @@ void tst_QStandardItemModel::signalsOnTakeItem() // QTBUG-89145
     QCOMPARE(takenItem->model(), nullptr);
     QCOMPARE(takenItem->child(0, 0)->model(), nullptr);
     QCOMPARE(m.index(1, 0).data(), QVariant());
+    delete takenItem;
 }
 
 void tst_QStandardItemModel::createPersistentOnLayoutAboutToBeChanged() // QTBUG-93466
@@ -1796,5 +1870,36 @@ void tst_QStandardItemModel::createPersistentOnLayoutAboutToBeChanged() // QTBUG
     QCOMPARE(layoutChangedSpy.size(), 1);
 }
 
+void tst_QStandardItemModel::takeChild()  // QTBUG-117900
+{
+  {
+      // with model
+      QStandardItemModel model1;
+      QStandardItemModel model2;
+      QStandardItem base1("base1");
+      model1.setItem(0, 0, &base1);
+      QStandardItem base2("base2");
+      model2.setItem(0, 0, &base2);
+      auto item = new QStandardItem("item1");
+      item->appendRow(new QStandardItem("child"));
+      base1.appendRow(item);
+      base2.appendRow(base1.takeChild(0, 0));
+      QCOMPARE(base1.child(0, 0), nullptr);
+      QCOMPARE(base2.child(0, 0), item);
+  }
+  {
+      // without model
+      QStandardItem base1("base1");
+      QStandardItem base2("base2");
+      auto item = new QStandardItem("item1");
+      item->appendRow(new QStandardItem("child"));
+      base1.appendRow(item);
+      base2.appendRow(base1.takeChild(0, 0));
+      QCOMPARE(base1.child(0, 0), nullptr);
+      QCOMPARE(base2.child(0, 0), item);
+  }
+}
+
+
 QTEST_MAIN(tst_QStandardItemModel)
 #include "tst_qstandarditemmodel.moc"
