@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
@@ -11,7 +11,6 @@
 #include <qdebug.h>
 #include <qfiledialog.h>
 #include <qabstractitemdelegate.h>
-#include <qitemdelegate.h>
 #include <qlistview.h>
 #include <qcombobox.h>
 #include <qpushbutton.h>
@@ -26,6 +25,7 @@
 #include <qlayout.h>
 #include <qmenu.h>
 #include <qrandom.h>
+#include <qpointer.h>
 #include "../../../../../src/widgets/dialogs/qsidebar_p.h"
 #include "../../../../../src/gui/itemmodels/qfilesystemmodel_p.h"
 #include "../../../../../src/widgets/dialogs/qfiledialog_p.h"
@@ -37,6 +37,8 @@
 
 #include "../../../../shared/filesystem.h"
 
+#include <QtWidgets/private/qapplication_p.h>
+
 #if defined QT_BUILD_INTERNAL
 QT_BEGIN_NAMESPACE
 Q_GUI_EXPORT bool qt_test_isFetchedRoot();
@@ -103,6 +105,7 @@ private slots:
     void QTBUG4419_lineEditSelectAll();
     void QTBUG6558_showDirsOnly();
     void QTBUG4842_selectFilterWithHideNameFilterDetails();
+    void noCrashWhenParentIsDeleted();
     void dontShowCompleterOnRoot();
     void nameFilterParsing_data();
     void nameFilterParsing();
@@ -680,13 +683,13 @@ void tst_QFileDialog2::completionOnLevelAfterRoot()
 #if defined(Q_OS_WIN)
     fd.setDirectory("C:/");
     QDir current = fd.directory();
-    QStringList entryList = current.entryList(QStringList(), QDir::Dirs);
+    const QStringList entryList = current.entryList(QStringList(), QDir::Dirs);
     // Find a suitable test dir under c:-root:
     // - At least 6 characters long
     // - Ascii, letters only
     // - No another dir with same start
     QString testDir;
-    foreach (const QString &entry, entryList) {
+    for (const QString &entry : entryList) {
         if (entry.size() > 5 && QString(entry.toLatin1()).compare(entry) == 0) {
             bool invalid = false;
             for (int i = 0; i < 5; i++) {
@@ -696,7 +699,7 @@ void tst_QFileDialog2::completionOnLevelAfterRoot()
                 }
             }
             if (!invalid) {
-                foreach (const QString &check, entryList) {
+                for (const QString &check : entryList) {
                     if (check.startsWith(entry.left(5), Qt::CaseInsensitive) && check != entry) {
                         invalid = true;
                         break;
@@ -946,9 +949,9 @@ void tst_QFileDialog2::task239706_editableFilterCombo()
     d.show();
     QVERIFY(QTest::qWaitForWindowExposed(&d));
 
-    QList<QComboBox *> comboList = d.findChildren<QComboBox *>();
+    const QList<QComboBox *> comboList = d.findChildren<QComboBox *>();
     QComboBox *filterCombo = nullptr;
-    foreach (QComboBox *combo, comboList) {
+    for (QComboBox *combo : comboList) {
         if (combo->objectName() == QString("fileTypeCombo")) {
             filterCombo = combo;
             break;
@@ -1109,7 +1112,6 @@ void tst_QFileDialog2::task254490_selectFileMultipleTimes()
     QTemporaryFile *t;
     t = new QTemporaryFile;
     QVERIFY2(t->open(), qPrintable(t->errorString()));
-    t->open();
     QFileDialog fd(0, "TestFileDialog");
 
     fd.setDirectory(tempPath);
@@ -1252,7 +1254,7 @@ void tst_QFileDialog2::QTBUG6558_showDirsOnly()
 
     //Create a file
     QFile tempFile(dirPath + "/plop.txt");
-    tempFile.open(QIODevice::WriteOnly | QIODevice::Text);
+    QVERIFY(tempFile.open(QIODevice::WriteOnly | QIODevice::Text));
     QTextStream out(&tempFile);
     out << "The magic number is: " << 49 << "\n";
     tempFile.close();
@@ -1265,7 +1267,6 @@ void tst_QFileDialog2::QTBUG6558_showDirsOnly()
     fd.setOption(QFileDialog::ShowDirsOnly, true);
     fd.show();
 
-    QApplication::setActiveWindow(&fd);
     QVERIFY(QTest::qWaitForWindowActive(&fd));
     QCOMPARE(fd.isVisible(), true);
     QCOMPARE(QApplication::activeWindow(), static_cast<QWidget*>(&fd));
@@ -1309,7 +1310,6 @@ void tst_QFileDialog2::QTBUG4842_selectFilterWithHideNameFilterDetails()
     fd.selectNameFilter(chosenFilterString);
     fd.show();
 
-    QApplication::setActiveWindow(&fd);
     QVERIFY(QTest::qWaitForWindowActive(&fd));
     QCOMPARE(fd.isVisible(), true);
     QCOMPARE(QApplication::activeWindow(), static_cast<QWidget*>(&fd));
@@ -1325,7 +1325,6 @@ void tst_QFileDialog2::QTBUG4842_selectFilterWithHideNameFilterDetails()
     fd2.selectNameFilter(chosenFilterString);
     fd2.show();
 
-    QApplication::setActiveWindow(&fd2);
     QVERIFY(QTest::qWaitForWindowActive(&fd2));
     QCOMPARE(fd2.isVisible(), true);
     QCOMPARE(QApplication::activeWindow(), static_cast<QWidget*>(&fd2));
@@ -1336,6 +1335,53 @@ void tst_QFileDialog2::QTBUG4842_selectFilterWithHideNameFilterDetails()
 
 }
 
+void tst_QFileDialog2::noCrashWhenParentIsDeleted()
+{
+    {
+        QPointer<QWidget> mainWindow = new QWidget();
+        QTimer::singleShot(1000, mainWindow, [mainWindow]
+                           { if (mainWindow.get()) mainWindow->deleteLater(); });
+        const QUrl url = QFileDialog::getOpenFileUrl(mainWindow.get(),
+                                                     QStringLiteral("getOpenFileUrl"));
+        QVERIFY(url.isEmpty());
+        QVERIFY(!url.isValid());
+        QVERIFY(!mainWindow.get());
+    }
+
+    {
+        QPointer<QWidget> mainWindow = new QWidget();
+        QTimer::singleShot(1000, mainWindow, [mainWindow]
+                           { if (mainWindow.get()) mainWindow->deleteLater(); });
+        const QUrl url = QFileDialog::getSaveFileUrl(mainWindow.get(),
+                                                     QStringLiteral("getSaveFileUrl"));
+        QVERIFY(url.isEmpty());
+        QVERIFY(!url.isValid());
+        QVERIFY(!mainWindow.get());
+    }
+
+    {
+        QPointer<QWidget> mainWindow = new QWidget();
+        QTimer::singleShot(1000, mainWindow, [mainWindow]
+                           { if (mainWindow.get()) mainWindow->deleteLater(); });
+        const QUrl url
+                = QFileDialog::getExistingDirectoryUrl(mainWindow.get(),
+                                                       QStringLiteral("getExistingDirectoryUrl"));
+        QVERIFY(url.isEmpty());
+        QVERIFY(!url.isValid());
+        QVERIFY(!mainWindow.get());
+    }
+
+    {
+        QPointer<QWidget> mainWindow = new QWidget();
+        QTimer::singleShot(1000, mainWindow, [mainWindow]
+                           { if (mainWindow.get()) mainWindow->deleteLater(); });
+        const QList<QUrl> url = QFileDialog::getOpenFileUrls(mainWindow.get(),
+                                                      QStringLiteral("getOpenFileUrls"));
+        QVERIFY(url.isEmpty());
+        QVERIFY(!mainWindow.get());
+    }
+}
+
 void tst_QFileDialog2::dontShowCompleterOnRoot()
 {
     if (!QGuiApplicationPrivate::platformIntegration()->hasCapability(QPlatformIntegration::WindowActivation))
@@ -1345,7 +1391,6 @@ void tst_QFileDialog2::dontShowCompleterOnRoot()
     fd.setAcceptMode(QFileDialog::AcceptSave);
     fd.show();
 
-    QApplication::setActiveWindow(&fd);
     QVERIFY(QTest::qWaitForWindowActive(&fd));
     QCOMPARE(fd.isVisible(), true);
     QCOMPARE(QApplication::activeWindow(), static_cast<QWidget*>(&fd));
