@@ -1,5 +1,5 @@
 // Copyright (C) 2020 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
@@ -19,6 +19,8 @@
 #include <qsettings.h>
 #endif
 
+#include <QtWidgets/private/qapplication_p.h>
+
 class SpecialRadioButton: public QRadioButton
 {
 public:
@@ -120,7 +122,6 @@ void tst_QButtonGroup::arrowKeyNavigation()
     layout.addWidget(&g2);
 
     dlg.show();
-    qApp->setActiveWindow(&dlg);
     QVERIFY(QTest::qWaitForWindowActive(&dlg));
 
     bt1.setFocus();
@@ -202,9 +203,7 @@ void tst_QButtonGroup::keyNavigationPushButtons()
     buttonGroup->addButton(pb3);
 
     dlg.show();
-    qApp->setActiveWindow(&dlg);
-    if (!QTest::qWaitForWindowActive(&dlg))
-        QSKIP("Window activation failed, skipping test");
+    QVERIFY(QTest::qWaitForWindowFocused(&dlg));
 
     QVERIFY2(le1->hasFocus(), qPrintable(qApp->focusWidget()->objectName()));
     QTest::keyClick(qApp->focusWidget(), Qt::Key_Tab);
@@ -433,7 +432,6 @@ void tst_QButtonGroup::task106609()
     qRegisterMetaType<QAbstractButton*>("QAbstractButton*");
     QSignalSpy spy1(buttons, SIGNAL(buttonClicked(QAbstractButton*)));
 
-    QApplication::setActiveWindow(&dlg);
     QTRY_COMPARE(QApplication::activeWindow(), static_cast<QWidget*>(&dlg));
 
     radio1->setFocus();
@@ -541,18 +539,18 @@ void tst_QButtonGroup::autoIncrementId()
     QRadioButton *radio3 = new QRadioButton(&dlg);
     radio3->setText("radio3");
 
-    buttons->addButton(radio1);
+    buttons->addButton(radio1, 2);
     vbox->addWidget(radio1);
-    buttons->addButton(radio2);
+    buttons->addButton(radio2, -1);
     vbox->addWidget(radio2);
     buttons->addButton(radio3);
     vbox->addWidget(radio3);
 
     radio1->setChecked(true);
 
-    QCOMPARE(buttons->id(radio1), -2);
-    QCOMPARE(buttons->id(radio2), -3);
-    QCOMPARE(buttons->id(radio3), -4);
+    QCOMPARE(buttons->id(radio1), 2);
+    QCOMPARE(buttons->id(radio2), -2);
+    QCOMPARE(buttons->id(radio3), -3);
 
     dlg.show();
 }
