@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
@@ -47,6 +22,10 @@
 #include <QStyle>
 #include <QProxyStyle>
 
+#include <QtWidgets/private/qapplication_p.h>
+
+#include <QtCore/private/qlocale_p.h> // for QSystemLocale
+
 class DoubleSpinBox : public QDoubleSpinBox
 {
     Q_OBJECT
@@ -207,6 +186,10 @@ private slots:
 
     void stepModifierPressAndHold_data();
     void stepModifierPressAndHold();
+
+    void digitGroupingControl_data();
+    void digitGroupingControl();
+
 public slots:
     void valueChangedHelper(const QString &);
     void valueChangedHelper(double);
@@ -880,8 +863,15 @@ void tst_QDoubleSpinBox::editingFinished()
     testFocusWidget.show();
     testFocusWidget.activateWindow();
     QVERIFY(QTest::qWaitForWindowActive(&testFocusWidget));
+
+    box->show();
+    QVERIFY(QTest::qWaitForWindowExposed(box));
     box->setFocus();
-    QTRY_VERIFY(box->hasFocus());
+
+    // Box may fail to acquire focus due to a system popup
+    // it is fair in that case to skip the test
+    if (!QTest::qWaitForWindowActive(box))
+        QSKIP("Focus acquisition failed.");
 
     QSignalSpy editingFinishedSpy1(box, SIGNAL(editingFinished()));
     QSignalSpy editingFinishedSpy2(box2, SIGNAL(editingFinished()));
@@ -890,37 +880,37 @@ void tst_QDoubleSpinBox::editingFinished()
     QTest::keyClick(box, Qt::Key_Up);
 
 
-    QCOMPARE(editingFinishedSpy1.count(), 0);
-    QCOMPARE(editingFinishedSpy2.count(), 0);
+    QCOMPARE(editingFinishedSpy1.size(), 0);
+    QCOMPARE(editingFinishedSpy2.size(), 0);
 
     QTest::keyClick(box2, Qt::Key_Up);
     QTest::keyClick(box2, Qt::Key_Up);
     box2->setFocus();
-    QCOMPARE(editingFinishedSpy1.count(), 1);
+    QCOMPARE(editingFinishedSpy1.size(), 1);
     box->setFocus();
-    QCOMPARE(editingFinishedSpy1.count(), 1);
-    QCOMPARE(editingFinishedSpy2.count(), 1);
+    QCOMPARE(editingFinishedSpy1.size(), 1);
+    QCOMPARE(editingFinishedSpy2.size(), 1);
     QTest::keyClick(box, Qt::Key_Up);
-    QCOMPARE(editingFinishedSpy1.count(), 1);
-    QCOMPARE(editingFinishedSpy2.count(), 1);
+    QCOMPARE(editingFinishedSpy1.size(), 1);
+    QCOMPARE(editingFinishedSpy2.size(), 1);
     QTest::keyClick(box, Qt::Key_Enter);
-    QCOMPARE(editingFinishedSpy1.count(), 2);
-    QCOMPARE(editingFinishedSpy2.count(), 1);
+    QCOMPARE(editingFinishedSpy1.size(), 2);
+    QCOMPARE(editingFinishedSpy2.size(), 1);
     QTest::keyClick(box, Qt::Key_Return);
-    QCOMPARE(editingFinishedSpy1.count(), 3);
-    QCOMPARE(editingFinishedSpy2.count(), 1);
+    QCOMPARE(editingFinishedSpy1.size(), 3);
+    QCOMPARE(editingFinishedSpy2.size(), 1);
     box2->setFocus();
-    QCOMPARE(editingFinishedSpy1.count(), 4);
-    QCOMPARE(editingFinishedSpy2.count(), 1);
+    QCOMPARE(editingFinishedSpy1.size(), 4);
+    QCOMPARE(editingFinishedSpy2.size(), 1);
     QTest::keyClick(box2, Qt::Key_Enter);
-    QCOMPARE(editingFinishedSpy1.count(), 4);
-    QCOMPARE(editingFinishedSpy2.count(), 2);
+    QCOMPARE(editingFinishedSpy1.size(), 4);
+    QCOMPARE(editingFinishedSpy2.size(), 2);
     QTest::keyClick(box2, Qt::Key_Return);
-    QCOMPARE(editingFinishedSpy1.count(), 4);
-    QCOMPARE(editingFinishedSpy2.count(), 3);
+    QCOMPARE(editingFinishedSpy1.size(), 4);
+    QCOMPARE(editingFinishedSpy2.size(), 3);
     testFocusWidget.hide();
-    QCOMPARE(editingFinishedSpy1.count(), 4);
-    QCOMPARE(editingFinishedSpy2.count(), 4);
+    QCOMPARE(editingFinishedSpy1.size(), 4);
+    QCOMPARE(editingFinishedSpy2.size(), 4);
 }
 
 void tst_QDoubleSpinBox::removeAll()
@@ -1170,7 +1160,6 @@ void tst_QDoubleSpinBox::taskQTBUG_5008_textFromValueAndValidate()
     spinbox.show();
     spinbox.activateWindow();
     spinbox.setFocus();
-    QApplication::setActiveWindow(&spinbox);
     QVERIFY(QTest::qWaitForWindowActive(&spinbox));
     QCOMPARE(static_cast<QWidget *>(&spinbox), QApplication::activeWindow());
     QTRY_VERIFY(spinbox.hasFocus());
@@ -1776,12 +1765,79 @@ void tst_QDoubleSpinBox::stepModifierPressAndHold()
                 QStyle::CC_SpinBox, &spinBoxStyleOption, subControl, &spin);
 
     QTest::mousePress(&spin, Qt::LeftButton, modifiers, buttonRect.center());
-    QTRY_VERIFY(spy.length() >= 3);
+    QTRY_VERIFY(spy.size() >= 3);
     QTest::mouseRelease(&spin, Qt::LeftButton, modifiers, buttonRect.center());
 
     const auto value = spy.last().at(0);
     QVERIFY(value.userType() == QMetaType::Double);
-    QCOMPARE(value.toDouble(), spy.length() * expectedStepModifier);
+    QCOMPARE(value.toDouble(), spy.size() * expectedStepModifier);
+}
+
+void tst_QDoubleSpinBox::digitGroupingControl_data()
+{
+    QTest::addColumn<QLocale>("locale");
+
+    QTest::addRow("en_US") << QLocale("en_US");
+    QTest::addRow("C") << QLocale("C");
+#if !defined(QT_NO_SYSTEMLOCALE) && defined(QT_BUILD_INTERNAL)
+    QTest::addRow("system") << QLocale(QLocale::AnyLanguage);
+#endif // !defined(QT_NO_SYSTEMLOCALE) && defined(QT_BUILD_INTERNAL)
+}
+
+void tst_QDoubleSpinBox::digitGroupingControl()
+{
+    QFETCH(QLocale, locale);
+
+#if !defined(QT_NO_SYSTEMLOCALE) && defined(QT_BUILD_INTERNAL)
+    class MySystemLocale : public QSystemLocale
+    {
+        Q_DISABLE_COPY_MOVE(MySystemLocale)
+    public:
+        MySystemLocale(const QLocale &fallback)
+            : m_fallback(fallback)
+        {}
+
+        QVariant query(QueryType type, QVariant &&/*in*/) const override
+        {
+            switch (type) {
+            case GroupSeparator:
+            case DecimalPoint:
+                return QVariant(".");
+            default:
+                break;
+            }
+            return QVariant();
+        }
+
+        QLocale fallbackLocale() const override { return m_fallback; }
+
+    private:
+        const QLocale m_fallback;
+    } customLocale(locale);
+
+    if (locale.language() == QLocale::AnyLanguage) {
+        // For the lifetime of `customLocale`, the system locale will use our
+        // custom implementation above.
+        locale = QLocale::system();
+    }
+#endif // !defined(QT_NO_SYSTEMLOCALE) && defined(QT_BUILD_INTERNAL)
+
+    QDoubleSpinBox spinbox;
+    spinbox.setLocale(locale);
+    spinbox.setRange(-100000, 100000);
+
+    constexpr int precision = 3;
+    constexpr double value = 12345.678;
+    spinbox.setDecimals(precision);
+    spinbox.setValue(value);
+
+    spinbox.setGroupSeparatorShown(true);
+    locale.setNumberOptions(locale.numberOptions() & ~QLocale::OmitGroupSeparator);
+    QCOMPARE(spinbox.text(), locale.toString(value, 'f', precision));
+
+    spinbox.setGroupSeparatorShown(false);
+    locale.setNumberOptions(locale.numberOptions() | QLocale::OmitGroupSeparator);
+    QCOMPARE(spinbox.text(), locale.toString(value, 'f', precision));
 }
 
 QTEST_MAIN(tst_QDoubleSpinBox)
