@@ -1,5 +1,5 @@
 // Copyright (C) 2019 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include "qmdisubwindow.h"
 #include "private/qmdisubwindow_p.h"
@@ -384,7 +384,6 @@ void tst_QMdiSubWindow::mainWindowSupport()
     mainWindow.setCentralWidget(workspace);
     mainWindow.show();
     mainWindow.menuBar()->setVisible(true);
-    QApplicationPrivate::setActiveWindow(&mainWindow);
     bool nativeMenuBar = mainWindow.menuBar()->isNativeMenuBar();
 
     // QMainWindow's window title is empty, so on a platform which does NOT have a native menubar,
@@ -510,7 +509,6 @@ void tst_QMdiSubWindow::emittingOfSignals()
     workspace.setWindowTitle(QLatin1String(QTest::currentTestFunction()));
     workspace.show();
     QCoreApplication::processEvents();
-    QApplicationPrivate::setActiveWindow(&workspace);
     QMdiSubWindow *window = qobject_cast<QMdiSubWindow *>(workspace.addSubWindow(new QWidget));
     QCoreApplication::processEvents();
     window->show();
@@ -954,6 +952,8 @@ void tst_QMdiSubWindow::mouseDoubleClick()
     if (!window->style()->styleHint(QStyle::SH_TitleBar_NoBorder, &options, window))
         height += window->isMinimized() ? 8 : 4;
     QPoint mousePosition(window->width() / 2, height - 1);
+    if (window->style()->inherits("QWindows11Style"))
+        mousePosition = QPoint(8, height - 1);
     sendMouseMove(window, mousePosition, Qt::NoButton);
 
     // Without Qt::WindowShadeButtonHint flag set
@@ -981,8 +981,10 @@ void tst_QMdiSubWindow::mouseDoubleClick()
 
     window->showMinimized();
     QVERIFY(window->isMinimized());
+    //Process QEvent::WindowStateChange
+    QCoreApplication::processEvents();
     sendMouseDoubleClick(window, mousePosition);
-    QVERIFY(!window->isMinimized());
+    QTRY_VERIFY(!window->isMinimized());
     QCOMPARE(window->geometry(), originalGeometry);
 }
 
@@ -1228,7 +1230,6 @@ void tst_QMdiSubWindow::restoreFocusOverCreation()
     subWidget1->m_lineEdit2->setFocus();
     subWindow1->show();
     mdiArea.show();
-    QApplicationPrivate::setActiveWindow(&mdiArea);
     QVERIFY(QTest::qWaitForWindowActive(&mdiArea));
     QCOMPARE(QApplication::focusWidget(), subWidget1->m_lineEdit2);
 
@@ -1373,7 +1374,7 @@ void tst_QMdiSubWindow::setWindowTitle()
     // other widgets which are not real top-level widgets).
     QCOMPARE(window->windowTitle(), expectedWindowTitle);
 
-    textEdit->setWindowModified(true);;
+    textEdit->setWindowModified(true);
     expectedWindowTitle = QLatin1String("Override child title");
     window->setWindowTitle(expectedWindowTitle);
     QVERIFY(window->isWindowModified());
@@ -1867,7 +1868,6 @@ void tst_QMdiSubWindow::closeOnDoubleClick()
 
 void tst_QMdiSubWindow::setFont()
 {
-    QSKIP("This test function is unstable in CI, please see QTBUG-22544");
     QMdiArea mdiArea;
     mdiArea.setWindowTitle(QLatin1String(QTest::currentTestFunction()));
     QMdiSubWindow *subWindow = mdiArea.addSubWindow(new TestPushButton(QLatin1String("test")));
@@ -1953,7 +1953,6 @@ void tst_QMdiSubWindow::task_182852()
     mainWindow.setCentralWidget(workspace);
     mainWindow.show();
     mainWindow.menuBar()->setVisible(true);
-    QApplicationPrivate::setActiveWindow(&mainWindow);
     if (mainWindow.menuBar()->isNativeMenuBar())
         return; // The main window's title is not overwritten if we have a native menubar (macOS, Unity etc.)
 
