@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef FONTDIALOGPANEL_H
 #define FONTDIALOGPANEL_H
@@ -22,7 +22,7 @@ public:
 
 public slots:
     void execModal();
-    void showModal();
+    void showModal(Qt::WindowModality modality);
     void showNonModal();
     void deleteNonModalDialog();
     void deleteModalDialog();
